# ----------------------------------------------------------------------------
# - scmn-os-environment                                                      -
# - wild cruncher - universal utility functions                              -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 amaury darsch                                    -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       scmn_os_environment
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher os environment functions

.. moduleauthor:: Amaury Darsch <amaury.darsch@silkan.com>
"""

# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------

import os
import platform


# ----------------------------------------------------------------------------
# - module section                                                           -
# ----------------------------------------------------------------------------

def scmn_get_platform():
    """
    Get the platfdorm name
    """
    # check for windows
    if platform.system().lower() == 'windows':
        return 'windows'
    # check for darwin
    if platform.system().lower() == 'darwin':
        return 'darwin'
    # check for unix/posix
    if os.name.lower() == 'posix':
        return 'posix'
    # not found
    raise Exception('Cannot detect platform')


def scmn_get_make():
    """
    Get the system make file path
    """
    # check for posix
    if os.name == 'posix':
        return scmn_get_unx_make()
    # check for wonderland
    if os.name == 'nt':
        return scmn_get_win_make()


def scmn_get_unx_make():
    """
    Get the system make file unix path
    """
    return '/usr/bin/make'


def scmn_get_win_make():
    """
    Get the system make file unix path
    """
    return 'C:\\MinGw\\msys\\1.0\\bin\\make.exe'


def scmn_get_ldname():
    """
    Return the os dependant library path environment name.
    """
    # check for windows
    if platform.system().lower() == 'windows':
        return 'PATH'
        # check for darwin
    if platform.system().lower() == 'darwin':
        return 'DYLD_FALLBACK_LIBRARY_PATH'
        # default to unix/elf
    return 'LD_LIBRARY_PATH'


def scmn_set_env_ldpath(path, env=None):
    """
    Set or create an environment with the library path.

    :param path: the path to set
    :type path: str
    :param env: an existing environment
    :type env: {str:str}
    """
    # check for valid path
    if path is None: return env
    # check for existing environment
    if env is None: env = {}
    # get the environment variable
    ldnm = scmn_get_ldname()
    # check for existence
    if ldnm not in env:
        env[ldnm] = os.path.normpath(path)
    else:
        if platform.system().lower() == 'windows':
            env[ldnm] = os.path.normpath(path) + ';' + env[ldnm]
        else:
            env[ldnm] = os.path.normpath(path) + ':' + env[ldnm]
            # here it is
    return env
