# ----------------------------------------------------------------------------
# - scmn-tuple                                                               -
# - silkan python library: mutable tuple                                     -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       scmn_tuple
   :platform:     Unix, Darwin, Windows
   :synopsis:     Silkan Python Library - mutable tuple

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from functools import total_ordering


@total_ordering
class ScmnMTuple(object):
    """
    Mutable tuple (like tuple, but mutable; like list, but of fixed length).
    """

    def __init__(self, *elements):
        """
        :param elementsIter: Tuple elements
        :type elementsIter: Iterable[object]
        """
        self.__elements = list(elements)

    def __len__(self):
        return len(self.__elements)

    def __iter__(self):
        return iter(self.__elements)

    def __and__(self, other):
        assert isinstance(other, tuple) or isinstance(other, ScmnMTuple)
        return ScmnMTuple(*(self.__elements + list(other)))

    def __bool__(self):
        return len(self) > 0

    def __contains__(self, item):
        return item in self.__elements

    def __eq__(self, other):
        if isinstance(other, ScmnMTuple):
            other = tuple(other.__elements)
        return tuple(self.__elements) == other

    def __lt__(self, other):
        if isinstance(other, ScmnMTuple):
            other = tuple(other.__elements)
        return tuple(self.__elements) < other

    def __reversed__(self):
        return ScmnMTuple(*reversed(self.__elements))

    def __getitem__(self, item):
        return self.__elements[item]

    def __setitem__(self, key, value):
        self.__elements[key] = value

    def __str__(self):
        return '`(' + ', '.join(str(e) for e in self.__elements) + ')'

    def __repr__(self):
        return str(self)
