# ----------------------------------------------------------------------------
# - wdbi-tree-wrapper                                                        -
# - wild cruncher:ish hierarchy                   core / dbstore / immutable -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbi_tree_wrapper
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher immutable hierarchy

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore.immutable.wdbi_tree import WdbiNodeP


class WdbiHierarchy(object):
    """
    Immutable hierarchy object.

    Manages hierarchy root node internally.
    """

    def __init__(self, topm):
        """
        :param topm: Root node
        :type topm: WdbiNodeP
        """
        self.__top = topm

    def getPath(self, path):
        return self.__top.getPath(path)

    def update(self, path, updateF):
        *p, l = path
        itms = [self.__top]
        cur = self.__top
        for e in p:
            cur = cur.getPath([e])
            itms.append(cur)
        itms.reverse()
        itm = cur.getPath([l])
        cur = updateF(itm)
        for i, p in zip(itms, reversed(path)):
            idnt = p.getUUID() if isinstance(p, WdbiNodeP) else p
            cur = i.updatedItem(idnt, cur)
        self.__top = cur

        # self.__top = self.__top.update(path, updateF)

    def add(self, path, *items):
        def _add(ilst):
            for item in items:
                ilst = ilst.add(item)
            return ilst

        self.update(path, _add)

    def remove(self, path, *items):
        def _remove(ilst):
            for item in items:
                ilst = ilst.remove(item)
            return ilst

        self.update(path, _remove)

    def current(self):
        return self.__top


class WdbiSubHierarchy(object):
    """
    Sub-hierarchy object, representing a subtree of a WdbiHierarchy.
    """

    def __init__(self, hierarchy, path):
        """
        :param hierarchy: Hierarchy to represent a subtree of
        :type hierarchy: WdbiHierarchy
        :param path: Path in the main hierarchy
        :type path: list[object]
        """
        self.__hierarchy = hierarchy
        self.__path = path

    def getPath(self, path):
        return self.current().getPath(path)

    def update(self, path, updateF):
        self.__hierarchy.update(self.__path + path, updateF)

    def add(self, path, *items):
        self.__hierarchy.add(self.__path + path, *items)

    def remove(self, path, *items):
        self.__hierarchy.remove(self.__path + path, *items)

    def current(self):
        return self.__hierarchy.getPath(self.__path)
