# ----------------------------------------------------------------------------
# - wmehdb-top-block                                                         -
# - wild cruncher:wsh top hierarchy object               core / Port / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 lambert philippe                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmehdb_top_block
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher top hierarchy object

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""

from wcr_core.dbstore.wdbs_meta_type import *
from wcr_core.modules.bus.db.wbhdb_top_block_meta import *
from wcr_core.db.whdb_block import WhdbBlock
from wcr_core.modules.bus.db.wbhdb_external_module_instance import \
    WbhdbExtModuleInstance
from silkan_common.scmn_file_path import scmn_path_rel2file, scmn_path_mkabs, \
    scmn_isfile, scmn_path_dir, scmn_relpath

BBM_MODULE = 'Blackboxmessages'


class WbhdbTopHierarchy(WhdbBlock):
    """
    The WmhdbTopHierarchy is the top hierarchy model class. A top hierarchy
    is at the same time a model and an instance. The class provides several
    methods for creating the hierarchy.
    """
    META = WBHDB_BUS_DATA_META_DESCRIPTION

    def __init__(self, block, wreg):
        """
        :param block: Raw block
        :type block: WdbsBlockP
        :param wreg: Wrapper registry
        :type wreg: WmhdbWrapperRegistry
        """
        super().__init__(block, wreg)
        self._srdfilepath = None
        self.Messages = {}
        """type: {path(str):(filename(str),
                             (messages (message top hierarchy block), name(str)
                             ),
                             {module(str):
                                 [messages (message operatorinstance)]
                             }
                            )}"""
        self.Ports = {}
        """type: {path(str):(filename(str),
                                     (Ports (Port top hierarchy block), name(str)
                                     ),
                                     {module(str):
                                         [Ports (Port operatorinstance)]
                                     }
                                    )}"""

    def getMessages(self):
        return self.Messages

    def getMessagesListe(self):
        msgl = []
        for f in self.Messages.keys():
            for mod in self.Messages[f][2].keys():
                for m in self.Messages[f][2][mod]:
                    msgl.append((f, self.Messages[f][0], mod, m.getName()))
        return msgl

    def setMessages(self, messages):
        self.Messages = messages

    def addMessagesFile(self, path, name):
        self.rawElement.setMeta(str(path), WBHDB_META_MESSAGES_KEY, str(name))

    def getMessagesFiles(self):
        lf = []
        for k in self.rawElement.getMeta(WBHDB_META_MESSAGES_KEY).keys():
            path = self.rawElement.getMeta(WBHDB_META_MESSAGES_KEY, k)
            abspathmess = scmn_path_mkabs(path,
                                          bdir=scmn_path_dir(
                                              self._srdfilepath))
            if not scmn_isfile(abspathmess):
                None
            else:
                lf.append(abspathmess)
        return lf

    def getMessageNameFile(self, path):
        for k in self.rawElement.getMeta(WBHDB_META_MESSAGES_KEY).keys():
            metapath = self.rawElement.getMeta(WBHDB_META_MESSAGES_KEY, k)
            abspath = scmn_path_mkabs(metapath,
                                      bdir=scmn_path_dir(self._srdfilepath))
            if self._srdfilepath:
                relpath = scmn_relpath(path, self._srdfilepath)
            else:
                relpath = path+"inval"
            if path == abspath or path == relpath:
                return k
        return None

    def getBBMessages(self):
        bbml = []
        for f in self.Messages.keys():
            for m in self.Messages[f][2][BBM_MODULE]:
                bbml.append((f, self.Messages[f][0], m.getName()))
        return bbml

    def getModulesPorts(self):
        prtl = []
        for o in self.getOperatorInstances():
            if isinstance(o, WbhdbExtModuleInstance):
                for p in o.getConnectionPoints():
                    prtl.append((o, p))
        return prtl

    def setSourcePath(self, path):
        if path is not None:
            self._srdfilepath = path

    def formatToSave(self, npath):
        ndir = scmn_path_dir(npath)
        msgpathl = self.getMessagesFiles()
        for path in msgpathl:
            if scmn_isfile(path):
                name = self.getMessageNameFile(path)
                msgrela = scmn_relpath(path, ndir)
                if name is None:
                    raise Exception("Message path given not in meta")
                self.addMessagesFile(msgrela, name)
        self.setSourcePath(npath)

# ----------------------------------

    def getPorts(self):
        return self.Ports

    def getPortsListe(self):
        msgl = []
        for f in self.Ports.keys():
            for mod in self.Ports[f][2].keys():
                for m in self.Ports[f][2][mod]:
                    msgl.append((f, self.Ports[f][0], mod, m.getName()))
        return msgl

    def setPorts(self, ports):
        self.Ports = ports

    def addPortsFile(self, path, name):
        self.rawElement.setMeta(str(path), WBHDB_META_PORTS_KEY, str(name))

    def getPortsFiles(self):
        lf = []
        for k in self.rawElement.getMeta(WBHDB_META_PORTS_KEY).keys():
            path = self.rawElement.getMeta(WBHDB_META_PORTS_KEY, k)
            abspathmess = scmn_path_mkabs(path,
                                          bdir=scmn_path_dir(
                                              self._srdfilepath))
            if not scmn_isfile(abspathmess):
                None
            else:
                lf.append(abspathmess)
        return lf

    def getPortNameFile(self, path):
        for k in self.rawElement.getMeta(WBHDB_META_PORTS_KEY).keys():
            metapath = self.rawElement.getMeta(WBHDB_META_PORTS_KEY, k)
            abspath = scmn_path_mkabs(metapath,
                                      bdir=scmn_path_dir(self._srdfilepath))
            if self._srdfilepath:
                relpath = scmn_relpath(path, self._srdfilepath)
            else:
                relpath = path+"inval"
            if path == abspath or path == relpath:
                return k
        return None

    def getBBPorts(self):
        bbml = []
        for f in self.Ports.keys():
            for m in self.Ports[f][2][BBM_MODULE]:
                bbml.append((f, self.Ports[f][0], m.getName()))
        return bbml
