# ----------------------------------------------------------------------------
# - wbmod-project-context                                                    -
# - wild cruncher - bus project context                    core / simulation -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbmod_project_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher Bus project context

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from wcr_core.project.wprj_project_context import WprjProjectContext
from wcr_core.modules.bus.controller.wbctl_controllers import \
    WbctlControllerFactory



class WbmodBusProjectContext(WprjProjectContext):
    """
    Default project context implementation.
    """

    def __init__(self, gContext, module, projectDB, logger):
        """
        :param gContext: Global context
        :type gContext: WcorGlobalContextP
        :param module: Application module
        :type module: WmodApplicationModuleP
        :param projectDB: Project database
        :type projectDB: WprjProjectDatabase
        :param logger: Logger
        :type logger: ScmnLoggerP
        """
        super().__init__(gContext, module, projectDB, logger)

        projectDB.getTopHierarchy().setSourcePath(projectDB.path)

        self.__ctrlFy = WbctlControllerFactory(module)



    def getControllerFactory(self):
        return self.__ctrlFy


def wbmod_retrieve_mes_obj(portList, messagesList):
    """
    Retrieve the logical buses linked to all physical buses in a dictionary
    from a list of simulation projects

    :param portList: List of ports with they module
    :type portList: Iterable[operatorinstance, port]
    :param messagesList: List of messages
    :type messagesList: Iterable[]
    :return:
    :rtype: Itertable[]
    """


    # res = {}
    """:type: dict[UUID, Iterable[messages]]"""
    boundMess = []
    for pb in portList:
        prod, cons = pb[1].getAllTransitMessages()
        """:type: list[WphdbPhyBusInstance]"""
        for m in cons:
            if m in messagesList:
                boundMess.append((pb,m,False))
        for m in prod:
            if m in messagesList:
                boundMess.append((pb, m, True))
    return boundMess