# ----------------------------------------------------------------------------
# - whhdb-Cetrac-switch-instance                                             -
# - wild cruncher: Cetrac switch instances              core / hardware / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whhdb_Cetrac_switch_instance
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - PCI Card instance

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore.wdbs_meta_description import *
from wcr_core.dbstore.wdbs_meta_type import *
from wcr_core.modules.hardware.db.whhdb_standard_operators import \
    WHHDB_CETRAC_SWITCH, WHHDB_CETRAC_SWITCH_CODE, WHHDB_WHALES_CODE,\
    WHHDB_WHALES_SWITCH
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_DIRECTION, \
    WHDB_XPOINT_NAME
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI
from wcr_core.modules.hardware.db.whhdb_operator_instance import \
    WhhdbOperatorInstance, WHHDB_IO_SWITCH_CARD_KEY
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_MODULES_IO_NAMES
from wcr_core.modules.hardware.db.whhdb_Host_instance_meta import \
    WHHDB_META_IOCTYPE_KEY
_WHHDB_SWITCH = {WHHDB_CETRAC_SWITCH_CODE: WHHDB_CETRAC_SWITCH,
                 WHHDB_WHALES_CODE: WHHDB_CETRAC_SWITCH,}


class WhhdbCetracSwitchInstance(WhhdbOperatorInstance):
    """
    Operator-instance for bus instances.
    """

    def __init__(self, inst, wreg):
        """
        :param inst: Raw instance
        :type inst: WdbsOperatorInstanceP
        :param wreg: Wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        assert inst.getOperator() in _WHHDB_SWITCH
        WhdbOperatorInstance.__init__(self, _WHHDB_SWITCH[inst.getOperator()], inst, wreg)
        self.CARD_TO_REMOVE = []
        self.IOCARD_TO_REMOVE = []
        self.initializeIoConnector()

    def _createRawConnectionPoint(self, ctyp, **kwargs):
        if WHDB_XPOINT_DIRECTION not in kwargs:
            kwargs[WHDB_XPOINT_DIRECTION] = WDBS_XPNT_BI
        return super()._createRawConnectionPoint(ctyp, **kwargs)

    def initializeIoConnector(self):
        # assign Io connector to modules
        for point in self.getConnectionPoints():
            if point.getCode() in WHHDB_MODULES_IO_NAMES:
                ioCard = '{}Module'.format(point.getCode())
                self.setIoCard(ioCard)
                self.setIoCardBind(ioCard, WHHDB_IO_SWITCH_CARD_KEY)
                point.setCard('.'.join((WHHDB_META_IOCTYPE_KEY, ioCard)))
                self.addPortUUIDToIoCard(ioCard, point.getUUID())