# ----------------------------------------------------------------------------
# - whhdb-IoCard-instance                                                    -
# - wild cruncher: Io Card instances                   core / hardware / db  -
# ----------------------------------------------------------------------------
# - author    (c) 2018-2018 achraf mahti                                     -
# - copyright (c) 2018-2018 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whhdb_rack_instance
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Io Card instance

.. moduleauthor:: Achraf Mahti <mahti@silkan.net>
"""
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.modules.hardware.db.whhdb_operator import WHHDB_OPER
from wcr_core.dbstore.wdbs_meta_description import *
from wcr_core.dbstore.wdbs_meta_type import *
from wcr_core.modules.hardware.db.whhdb_rack_instance_meta import IO_TYPES
from wcr_core.modules.hardware.db.whhdb_rack_instance_meta import *


class WhhdbIoCardInstance(WhdbOperatorInstance):

    """
    Operator-instance for IoCard instances.
    """
    TYPES = IO_TYPES
    META = WHHDB_IOCARD_META

    def __init__(self, inst, wreg):
        """
        :param inst: Raw instance
        :type inst: WdbsOperatorInstanceP
        :param wreg: Wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        self.op = inst.getOperator()
        self.code = self.op.lower().split('.')[-1]
        assert self.code in [t.lower() for t in self.TYPES]

        super().__init__(WHHDB_OPER[self.op], inst, wreg)
        # at init name and type are the same
        self.setName(self.code)
        self.setType(self.code)
        self.__linkedEquipement = None

    @property
    def _card(self):
        return self.rawElement.getMeta(WHHDB_META_CTYPE_KEY)

    def getName(self):
        return self._card[WHHDB_CARD_NAME_KEY]

    def setName(self, card):
        self.rawElement.setMeta({WHHDB_CARD_NAME_KEY: card},
                                 WHHDB_META_CTYPE_KEY)

    def getType(self):
        return self._card[WHHDB_CARD_TYPE_KEY]

    def setType(self, cardType):
        self.rawElement.setMeta({WHHDB_CARD_TYPE_KEY: cardType},
                                 WHHDB_META_CTYPE_KEY)

    def getIDTStatus(self):
        return self._card[WHHDB_CARD_POSITION_KEY]

    def setIDTStatus(self, pos):
        self.rawElement.setMeta({WHHDB_CARD_POSITION_KEY: pos},
                                 WHHDB_META_CTYPE_KEY)

    def InitChannelsId(self):
        for channelName, channel in self.parameters.items():
            channelId = int(channelName.strip('channel'))
            idStatus = hex(int(self.getIDTStatus(), 16) +channelId)
            channel['idt'].setValue(idStatus)

    def getChannel(self, num):
        return self.parameters[num]

    def getChannelParam(self, num, paramName):
        return self.parameters[num][paramName]

    def setChannelParamValue(self, num, paramName, value):
        chanPrm = self.getChannelParam(num, paramName)
        chanPrm.setValue(value)

    def getEquipement(self):
        return self.__linkedEquipement

    def setEquipement(self, eqp):
        self.__linkedEquipement = eqp

    def __str__(self):
        return self.getType()
