# ----------------------------------------------------------------------------
# - wbmod-connection-point-type                                              -
# - wild cruncher: connection point type descriptor               core / bus -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2014-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbmod_connection_point_type
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Connection point type descriptor

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WhmodConnectionPointTypeP(metaclass=ABCMeta):
    """
    Connection point type protocol.
    """

    @abstractmethod
    def getCode(self):
        """
        Get the code for the connection point type.

        :return: Code
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getName(self):
        """
        Get the name for the connection point type.

        :return: Name
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getPrefix(self):
        """
        Get the name prefix for the connection point type.

        :return: Name prefix
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getConfigMeta(self):
        """
        Get the configuration meta-data for the connection point type.

        :return: Configuration meta-data
        :rtype: WdbsMetaTypeP | None
        """
        raise NotImplementedError()

    @abstractmethod
    def getClass(self):
        """
        Get the class for the connection point type.

        :return: Class / constructor
        :rtype: () -> WphdbConnectionPointBase
        """
        raise NotImplementedError()

    @abstractmethod
    def getDefaultCreationArguments(self):
        """
        Get the default creation arguments for new connection points of this
        type.

        :return: Default creation arguments
        :rtype: dict[str, object] | None
        """
        raise NotImplementedError()


class WhmodConnectionPointType(WhmodConnectionPointTypeP):
    def __init__(self, code, name, prefix, clazz, cfgMeta=None,
                 cfgDisMeta=None, cfgFiberMeta=None, cfgAfdxMeta=None,
                 cfg1553Meta=None, cfgARINC825Meta=None, cfgARINC429Meta=None, crtnArgs=None):
        """
        :param code: Connection point type code
        :type code: str
        :param name: Connection point type name
        :type name: str
        :param prefix: Name prefix
        :type prefix: str
        :param clazz: Class / constructor
        :type clazz: () -> WhhdbConnectionPointBase
        :param cfgMeta: Configuration meta-data
        :type cfgMeta: WdbsMetaTypeP | None
        :param crtnArgs: Default creation arguments
        :type crtnArgs: dict[str, object] | None
        """
        self.__code = code
        self.__name = name
        self.__prefix = prefix
        self.__cfgMeta = cfgMeta
        self.__cfgDisMeta = cfgDisMeta
        self.__cfgFiberMeta = cfgFiberMeta
        self.__cfgAfdxMeta = cfgAfdxMeta
        self.__cfg1553Meta = cfg1553Meta
        self.__cfgARINC825Meta = cfgARINC825Meta
        self.__cfgARINC429Meta = cfgARINC429Meta
        self.__creationArgs = crtnArgs
        self.__class = clazz

    def getCode(self):
        return self.__code

    def getName(self):
        return self.__name

    def getPrefix(self):
        return self.__prefix

    def getConfigMeta(self):
        return self.__cfgMeta

    def getConfigDisMeta(self):
        return self.__cfgDisMeta

    def getConfigFiberMeta(self):
        return self.__cfgFiberMeta

    def getConfigAfdxMeta(self):
        return self.__cfgAfdxMeta

    def getConfig1553Meta(self):
        return self.__cfg1553Meta

    def getConfigARINC825Meta(self):
        return self.__cfgARINC825Meta

    def getConfigARINC429Meta(self):
        return self.__cfgARINC429Meta

    def getClass(self):
        return self.__class

    def getDefaultCreationArguments(self):
        return self.__creationArgs
