# ----------------------------------------------------------------------------
# - wihdb-wrapper-registry                                                   -
# - wild cruncher: raw data wrapper registry                 core / icd / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wihdb_wrapper_registry
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy raw data wrapper registry

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_wrapper_registry import WhdbWrapperRegistry
from wcr_core.dbstore_api.wdbs_instance_api import WdbsOperatorInstanceP
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.db.whdb_block import WhdbBlock
from wcr_core.dbstore_api.wdbs_block_api import WdbsBlockP
from wcr_core.modules.icd.db.wihdb_top_block import WihdbTopHierarchy
from wcr_core.db_api.whdb_block_api import WHDB_TOPLEVEL_BLOCK_CODE
from wcr_core.modules.icd.db.wihdb_standard_operators import WIHDB_MODULE_CODE, \
    WIHDB_WBMESSAGE_CODE
from wcr_core.modules.icd.db.wihdb_module_instance import WihdbModuleInstance, \
    WIHDB_DATAS_KEY
from wcr_frontend.modules.icd.gui.wigui_props_dialog_factory import \
    WIHDB_STD_OPERINST_PROPDLG
from wcr_core.modules.icd.db.wihdb_meta_provider import WIHDB_STD_OPERINST_META
from wcr_core.db_api.whdb_protocols_api import WHDB_HALIGN_LEFT, \
    WHDB_HALIGN_RIGHT
from wcr_frontend.modules.icd.gui.wigui_module_instance_props_dialog import \
    WiguiModuleInstancePropertiesDialog
from wcr_frontend.modules.icd import _WIGUI_ICONS
from wcr_core.modules.icd.db.wihdb_operator import WIHDB_OPER, WihdbOperator
from wcr_core.modules.icd.db.wihdb_connection_point_manager import \
    WihdbConnectionPointManager
from wcr_core.modules.icd.db.wihdb_wgtbxmessage_instance import \
    WihdbWightBoxMessageInstance

WIHDB_STD_OPERINST_CLS = {
                          WIHDB_MODULE_CODE: WihdbModuleInstance,
                          WIHDB_WBMESSAGE_CODE: WihdbWightBoxMessageInstance
                         }
"""{OPERAOT_CODE : OperatorInstance}"""


class WihdbWrapperRegistry(WhdbWrapperRegistry):
    """
    Manages HDS-HSB wrappers.
    """
    def __init__(self, blockClzzs):
        """

        """
        super().__init__()
        self.__blockClasses = blockClzzs

    def createWrapper(self, obj, *args):
        if isinstance(obj, WdbsOperatorInstanceP):
            return self._createOperatorInstance(obj)
        elif isinstance(obj, WdbsBlockP):
            if obj.getEntityType() == WHDB_TOPLEVEL_BLOCK_CODE:
                return WihdbTopHierarchy(obj, self)
            else:
                return WhdbBlock(obj, self)

        return super().createWrapper(obj, *args)

    def _createOperatorInstance(self, opInst):
        """
        Create an operator instance object based on the type of operator

        :param opInst: Raw operator instance
        :type opInst: WdbsOperatorInstanceP
        :return: Operator instance created
        :rtype: WhdbOperatorInstanceP
        """
        operator = opInst.getOperator()
        cls = WIHDB_STD_OPERINST_CLS.get(operator)
        if cls is not None:
            return cls(opInst, self)
        else:
            create_new_imported_module(operator, opInst, opInst.getMeta('moduleType'))
            cls = WIHDB_STD_OPERINST_CLS.get(operator)
            return cls(opInst, self) if cls is not None else \
                WhdbOperatorInstance(WIHDB_OPER[operator], opInst, self)


def create_new_imported_module(module_code, opInst = None, moduleType=None):
    from wcr_core.modules.icd.wimod_application_module import WIMOD_ICD_APP_MODULE
    mod_name = module_code.split('.')[-1]
    WIHDB_STD_OPERINST_CLS[module_code] = WihdbModuleInstance
    WIHDB_STD_OPERINST_META[module_code] = WihdbModuleInstance.META
    _WIGUI_ICONS[module_code] = ':/img/wpgui_clstr.png'
    oper = WihdbOperator(code=module_code, xmngr=WihdbConnectionPointManager(xpntTypes=None),
                         name=mod_name, info=mod_name, symbol=mod_name,
                         labelAlign=WHDB_HALIGN_LEFT)
    if moduleType:
        oper.setModuleType(moduleType)
    WIHDB_STD_OPERINST_PROPDLG[oper] = WiguiModuleInstancePropertiesDialog
    if opInst:
        fillModuleDataFromOperator(oper, opInst)
    WIMOD_ICD_APP_MODULE.operatorRegistry.addOperators([oper])
    return oper


def fillModuleDataFromOperator(oper, opInst):
    oper.setEntries(opInst.getMeta(WIHDB_DATAS_KEY))


def wihdb_create_and_instanciate_msg(msgName):
    oper = WihdbOperator(code=WIHDB_WBMESSAGE_CODE,
                          xmngr=None,
                          name=msgName, info=msgName, symbol=msgName,
                          labelAlign=WHDB_HALIGN_LEFT)
    return oper