# ----------------------------------------------------------------------------
# - wmhdb-wrapper-registry                                                   -
# - wild cruncher: raw data wrapper registry               core / model / db -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmhdb_wrapper_registry
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy raw data wrapper registry

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_wrapper_registry import WhdbWrapperRegistry
from wcr_core.dbstore_api.wdbs_block_api import WdbsBlockP
from wcr_core.dbstore_api.wdbs_connection_api import WdbsConnectionP
from wcr_core.dbstore_api.wdbs_instance_api import WdbsOperatorInstanceP, \
    WdbsBlockInstanceP, WdbsModelInstanceP
from wcr_core.dbstore_api.wdbs_model_api import WdbsModelP
from wcr_core.dbstore_api.wdbs_connection_point_api import WdbsConnectionPointP
from wcr_core.dbstore_api.wdbs_parameter_api import WdbsParameterP
from wcr_core.modules.model.db.wmhdb_block import WmhdbBlock
from wcr_core.modules.model.db.wmhdb_function import WmhdbFunction
from wcr_core.modules.model.db.wmhdb_instance import WmhdbBlockInstance, \
    WmhdbModelInstance
from wcr_core.modules.model.db.wmhdb_parameter import WmhdbParameter
from wcr_core.modules.model.db.wmhdb_text_model import WmhdbModelText
from wcr_core.modules.model.db.wmhdb_wire import WmhdbWire
from wcr_core.modules.model.db.wmhdb_connection_point import WmhdbConnector


class WmhdbWrapperRegistry(WhdbWrapperRegistry):
    """
    Manages HDS-HSB wrappers.
    """
    def __init__(self, blockClzzs):
        """
        :param blockClzzs: Block HDB classes
        :type blockClzzs: dict[str,
                               (WdbsBlockP, WhdbWrapperRegistryP)
                                   -> WmhdbBlockP]
        """
        super().__init__()

        self.__blockClasses = blockClzzs

    def createWrapper(self, obj, *args):
        if isinstance(obj, WdbsConnectionP):
            return WmhdbWire(obj, self)
        elif isinstance(obj, WdbsConnectionPointP):
            return WmhdbConnector(obj, self)
        elif isinstance(obj, WdbsOperatorInstanceP):
            return WmhdbFunction(obj, self)
        elif isinstance(obj, WdbsBlockInstanceP):
            return WmhdbBlockInstance(obj, self)
        elif isinstance(obj, WdbsModelInstanceP):
            return WmhdbModelInstance(obj, self)
        elif isinstance(obj, WdbsBlockP):
            btyp = obj.getEntityType()
            if btyp:
                clazz = self.__blockClasses.get(btyp)
                if clazz:
                    return clazz(obj, self)
            return WmhdbBlock(obj, self)
        elif isinstance(obj, WdbsModelP):
            return WmhdbModelText(obj, self)
        elif isinstance(obj, WdbsParameterP):
            return WmhdbParameter(obj, *args)
        else:
            return super().createWrapper(obj, *args)
