# ----------------------------------------------------------------------------
# - wmmod-sgroup-execution                                                   -
# - wild cruncher: scheduling group exeuction                   core / model -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmmod_sgroup_execution
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Scheduling group execution

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WmmodSgroupExecutionP(metaclass=ABCMeta):
    """
    Protocol to represent a single execution of a scheduling group.

    Executions are intended to be run only once.
    """

    @abstractmethod
    def start(self, strm):
        """
        Start the execution.

        :param strm: Process output/error stream handler
        :type strm: ScmnProcStreamHandlerP
        """
        raise NotImplementedError()

    @abstractmethod
    def stop(self, strm):
        """
        Stop the execution.

        :param strm: Process output/error stream handler
        :type strm: ScmnProcStreamHandlerP
        """
        raise NotImplementedError()


class WmmodSgroupExecutionFactoryP(metaclass=ABCMeta):
    """
    Scheduling group execution factory procotol.

    Scheduling group execution factories create scheduling group execution
    instances, `WmmodSgroupExecutionP`, that allow to execute a scheduling
    group once from inside Wild Cruncher Model.
    """

    @abstractmethod
    def createExecution(self, sgroup, busName, busPeriod, cycles, pctx,
                        prof=None):
        """
        Create a new scheduling execution run.

        :param sgroup: Scheduling group to execute
        :type sgroup: str
        :param busName: Bus name
        :type busName: str
        :param busPeriod: Bus period
        :type busPeriod: float
        :param cycles: Number of cycles
        :type cycles: int
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param prof: Profile
        :type prof: str | None
        :return: Newly created execution run
        :rtype: WmmodSgroupExecutionP
        """
        raise NotImplementedError()
