# ----------------------------------------------------------------------------
# - wfhdb-connection-point-manager                                           -
# - wild cruncher: connection point manager         core / statemachine / db -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfhdb_connection_point_manager
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher connection point manager

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_connection_point import WHDB_DEFAULT_XPOINT_CODE
from wcr_core.db.whdb_connection_point_manager import \
    WhdbConnectionPointManagerBase, WHDB_XPOINT_DIRECTION, \
    WHDB_XPOINT_POSITION, WHDB_XPOINT_TYPE, WHDB_XPOINT_NAME, \
    WHDB_XPOINT_FACING, WhdbXPointDesc
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_IN, \
    WDBS_XPNT_OUT


class WfhdbConnectionPointManager(WhdbConnectionPointManagerBase):
    """
    Connection point manager for states.
    """

    def getInitialXPointSet(self):
        return []

    def getDynamicXPoints(self):
        return [('in', 'In-Transition',
                 {WHDB_XPOINT_DIRECTION: WDBS_XPNT_IN}),
                ('out', 'Out-Transition',
                 {WHDB_XPOINT_DIRECTION: WDBS_XPNT_OUT})]

    def canBatchCreateXPoint(self, xpnts, **kwargs):
        return True

    def canInstanceCreateXPoint(self, clst, **kwargs):
        return True

    def canBatchRemoveXPoint(self, xpnts, **kwargs):
        return True

    def canInstanceRemoveXPoint(self, clst, xpnt):
        return True

    def createInstanceXPoint(self, clst, connF, **kwargs):
        assert WHDB_XPOINT_DIRECTION in kwargs

        xdsc = wfhdb_createXPointDesc(
            cdir=kwargs.get(WHDB_XPOINT_DIRECTION),
            cpos=kwargs.get(WHDB_XPOINT_POSITION),
            fcng=kwargs.get(WHDB_XPOINT_FACING))
        xpnt = connF(xdsc)
        xpnt.setMeta(str(xpnt.getUUID()), WHDB_XPOINT_NAME)
        return xpnt

    def removeInstanceXPoint(self, clst, xpnt):
        clst.remove(xpnt)

    def setInstanceXPoints(self, clst, connF, data):
        raise Exception('Cannot set connection points')

    def areValidXPoints(self, clst):
        return True, None

    def resetXPointPositions(self, clst):
        pass


def wfhdb_createXPointDesc(cdir, name=None, cpos=None, fcng=None, xpnt=None,
                           **kwargs):
    """
    Create a connection point description.

    :param cdir: Direction
    :type cdir: int
    :param name: Name
    :type name: str | None
    :param cpos: Position
    :type cpos: (float, float) | None
    :param fcng: Facing/gravity
    :type fcng: int | None
    :param xpnt: Referenced connection point
    :type xpnt: WhdbConnectionPointP | None
    :param kwargs: Additional arguments
    :type kwargs: dict[str, object]
    :return: Newly created connection point description
    :rtype: WhdbXPointDesc
    """
    assert WHDB_XPOINT_TYPE not in kwargs
    assert 'gvty' not in kwargs

    isin = cdir == WDBS_XPNT_IN
    return WhdbXPointDesc(
        ctyp=WHDB_DEFAULT_XPOINT_CODE,
        cpos=cpos or ((-1.0, 0.0) if isin else (1.0, 0.0)),
        cdir=cdir, name=name, fcng=fcng, xpnt=xpnt, **kwargs)
