# ----------------------------------------------------------------------------
# - wfhdb-datamodel-support                                           -
# - wild cruncher: datamodel support                core / statemachine / db -
# ----------------------------------------------------------------------------
# - author    (c) 2015-2015 Philippe Lambert                                 -
# - copyright (c) 2015-2015 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfhdb_datamodel_support
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher connection point manager

.. moduleauthor:: Philippe.lambert <philippe.lambert@silkan.com>
"""


from wcr_core.modules.statemachine.db.wfhdb_protocols import *


WFHDB_DATAMODEL_TYPE_KEY = 'datamodel-type'

WFHDB_DATAMODEL_KEY = 'datamodel'

WFHDB_DM_TYPE_KEY = 'type'

WFHDB_DM_KIND_KEY = 'kind'

WFHDB_DM_EXPRESSION_KEY = 'expression'

WFHDB_DM_SOURCE_KEY = 'source'

class WfhdbDatamodelSupport():
    META = WdbsMetaDescription(
        WdbsMetaField(WFHDB_DATAMODEL_KEY,
                      WdbsOrderedDictionaryMetaType(
                          WDBS_STRING_META,
                          WdbsRecordMetaType(
                              (WFHDB_DM_TYPE_KEY, WDBS_STRING_META),
                              (WFHDB_DM_KIND_KEY, WDBS_STRING_META),
                              (WFHDB_DM_EXPRESSION_KEY,
                               WDBS_STRING_META),
                              (WFHDB_DM_SOURCE_KEY, WDBS_STRING_META)
                          ))))

    def __init__(self, block, wreg):
        """
        :param block: Raw block
        :type block: WdbsBlockP
        :param wreg: Wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        super().__init__(block, wreg)

    def addToDataModel(self, raw, name, dtyp=None, kind=None, expr=None,
                       src=None):
        """
        Add an element to the data model.

        :param name: Name of variable
        :type name: str
        :param dtyp: Variable type
        :type dtyp: str | None
        :param kind: Kind (triggered / continuous)
        :type kind: str | None
        :param expr: Expression for initial value
        :type expr: str | None
        :param src: Value source (URI)
        :type src: str | None
        """
        raw.setMeta((dtyp, kind, expr, src),
                    WFHDB_DATAMODEL_KEY, name)


    def getDataModelType(self, raw):
        """
        Get the data-model type (ecmascript, xpath, ...) for the state machine.

        :return: Data-model type
        :rtype: str
        """
        return raw.getMeta(WFHDB_DATAMODEL_TYPE_KEY)

    def setDataModelType(self, raw, dtyp):
        """
        Set the data-model type (ecmascript, xpath, ...) for the state machine.

        :param dtyp: Data-model type
        :type dtyp: str
        """
        raw.setMeta(dtyp, WFHDB_DATAMODEL_TYPE_KEY)


    def setDataModel(self, raw, dmdl):
        """
        Set the data model for the state machine.

        :param dmdl: Data model as dictionary {Name:record}
        :type dmdl: dict[str, dict[str, object] | (str,str,str,str)]
        """
        raw.setMeta(dmdl, WFHDB_DATAMODEL_KEY)

    def getDataModel(self, raw):
        """
        Get the data model for the state machine.

        :return: Data model as dictionary {Name:record}
        :rtype: dict[str, dict[str, object]]
        """
        dmdl = raw.getMeta(WFHDB_DATAMODEL_KEY)
        return dmdl if dmdl is not None else {}


    def removeFromDataModel(self, raw, name):
        """
        Remove an element from the data model.

        :param name: Name of variable to be removed
        :type name: str
        """
        # Todo: Use rawElement.removeMeta(*keys)
        raw.setMeta(None, WFHDB_DATAMODEL_KEY, name)

