# ----------------------------------------------------------------------------
# - wcact-reload-logical-action                                              -
# - wild cruncher: reload logical source action     core / sys_map / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2019-2019 Achraf Mahti                                     -
# - copyright (c) 2014-2019 silkan / CETRAC                                  -
# ----------------------------------------------------------------------------

"""
.. module::       wcact_reload_hardware_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Hardware source reload action

.. moduleauthor:: achraf mahti <achraf@cetrac.io>
"""
import os
from wcr_core.modules.icd.db.wihdb_top_block import WihdbTopHierarchy
from wcr_core.modules.icd.wimod_application_module import \
    WIMOD_ICD_APP_CODE
from wcr_core.modules.sys_map.actions.wcact_action_base import WcactActionBase
from wcr_core.project.wprj_project import wprj_load_project


class _WcactReloadLogicalAction(WcactActionBase):
    """
    Project editor action to reload the hardware source.
    """

    def __init__(self):
        super().__init__(
            'Reload logical...',
            'Reload the logical from disk')

    def _doExecute(self, actx, paramValuePairs):
        pctx = actx.getProjectContext()
        """:type: WpmodSystemProjectContextMP"""
        wcact_reload_logical(pctx)


def wcact_reload_logical(pctx):
    """
    Reload the logical source.

    :param pctx: Project context
    :type pctx: WpmodSystemProjectContextMP
    """
    tblk = pctx.getProjectDB().getTopHierarchy()
    """:type: WphdbTopBlock"""
    file = tblk.getLogicFile()
    if os.path.isfile(str(file)):
        pdb = wprj_load_project(file, pctx)
        if pdb is not None and pdb.getModule() == WIMOD_ICD_APP_CODE:
            simulation = pdb.getTopHierarchy() if pdb else None
            assert isinstance(simulation, WihdbTopHierarchy)
            pctx.setLogicFile(simulation)
            return

    pctx.setLogicFile(None)


#: Logical simulation reload action
WCACT_RELOAD_LOGICAL_ACTION = _WcactReloadLogicalAction()
