# ----------------------------------------------------------------------------
# - wchdb-host-decoy-instance                                                -
# - wild cruncher: Host decoy instances                  core / sys_map / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wchdb_host_decoy_instance
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Host decoy instance

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore.wdbs_meta_description import *
from wcr_core.dbstore.wdbs_meta_type import *
from wcr_core.modules.sys_map.db.wchdb_standard_operators import \
    WCHDB_HOST_LURE, WCHDB_HOST_LURE_CODE
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_DIRECTION, \
    WHDB_XPOINT_NAME
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI

WCHDB_SOURCE_UUID_KEY = "uuid_src"
WCHDB_BIND_MODULE_KEY = "modules-bind"

class WchdbHostLureInstance(WhdbOperatorInstance):
    """
    Operator-instance for bus instances.
    """

    META = WdbsMetaDescription(
        WhdbOperatorInstance.META,
        WdbsMetaField(WCHDB_SOURCE_UUID_KEY, WDBS_UUID_META),
        WdbsMetaField(WCHDB_BIND_MODULE_KEY,
                      WdbsListMetaType(WDBS_STRING_META)))

    def __init__(self, inst, wreg):
        """
        :param inst: Raw instance
        :type inst: WdbsOperatorInstanceP
        :param wreg: Wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        assert inst.getOperator() == WCHDB_HOST_LURE_CODE
        WhdbOperatorInstance.__init__(self, WCHDB_HOST_LURE, inst, wreg)
        self.__cards = []
        self.__ioCards = []
        self.__ioBinding = {}
        self.__modulesF = None
        self.__pciPorts = {}
        self.__unboundModules = []

    def _createRawConnectionPoint(self, ctyp, **kwargs):
        if WHDB_XPOINT_DIRECTION not in kwargs:
            kwargs[WHDB_XPOINT_DIRECTION] = WDBS_XPNT_BI
        return super()._createRawConnectionPoint(ctyp, **kwargs)

    def setUUIDsource(self, uuid):
        self.rawElement.setMeta(uuid, WCHDB_SOURCE_UUID_KEY)

    def getUUIDsource(self):
        return self.rawElement.getMeta(WCHDB_SOURCE_UUID_KEY)

    def setModulesFunc(self, func):
        self.__modulesF = func

    def setCards(self, cards):
        self.__cards = cards

    def getCards(self):
        return self.__cards

    def addModule(self, module_):
        self.__unboundModules.append(module_)

    def removeModule(self, module_):
        self.__unboundModules.remove(module_)

    def getModules(self):
        return self.__unboundModules

    def setIoCards(self, ioCards):
        self.__ioCards = ioCards

    def getIoCards(self):
        return self.__ioCards

    def setIoCardBind(self, ioBinding, card):
        self.__ioBinding[ioBinding] = card

    def getIoCardBind(self):
        return self.__ioBinding

    def getInfo(self):
        if super().getInfo():
            return super().getInfo()
        #needed fo trigering personal description bubble when mouse over.
        #see wcgui_scene_item_factory
        return ' '

    def getDescription(self):
        descbase=''
        if self.__modulesF is None:
            return descbase
        if len(self.__modulesF(self.getUUIDsource())) < 1 :
            return descbase
        #absolute alphabetic sorting : no case considered
        for mn in sorted(self.__modulesF(self.getUUIDsource()),
                         key=lambda x:x.lower()):
            descbase+=mn+"<br/>"
        return descbase

    def setPCIPort(self, port):
        self.__pciPorts[port.getCard()] = port

    def getPCIPort(self, card):
        if card in self.__pciPorts:
            return self.__pciPorts[card]
