# ----------------------------------------------------------------------------
# - wcgui-route-generation-container                                         -
# - wild cruncher - cockpit : route generation container                     -
# ----------------------------------------------------------------------------
# - author    (c) 2019-2019 achraf mahti                                     -
# - copyright (c) 2019-2019 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_route_generation_container
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher instance properties dialog

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""
from operator import methodcaller


class RouteGenerationContainer:

    def __init__(self):
        self.__message = None
        self.__moduleProducer = None
        self.__modulesConsumer = set()
        self.__equipmentProducer = set()
        self.__equipmentConsumer = set()
        self.__modulePeriod = None
        self.__portProducer = set()
        self.__portConsumer = set()
        self.__nbConsumer = 0

    def getMessage(self):
        return self.__message

    def setMessage(self, msg):
        self.__message = msg

    def getModuleProducer(self):
        return self.__moduleProducer

    def setModuleProducer(self, module_):
        self.__moduleProducer = module_

    def getModulesConsumer(self):
        return sorted(list(self.__modulesConsumer))

    def setModulesConsumer(self, modules):
        self.__modulesConsumer = set(modules)

    def addModuleConsumer(self, module_):
        self.__modulesConsumer.add(module_)

    def removeModuleConsumer(self, module_):
        self.__modulesConsumer.remove(module_)

    def getEquipmentProducer(self):
        return sorted(list(self.__equipmentProducer))

    def addEquipmentsProducer(self, eqp):
        self.__equipmentProducer.add(eqp)

    def removeEquipmentsProducer(self, eqp):
        self.__equipmentProducer.remove(eqp)

    def setEquipmentProducer(self, eqp):
        self.__equipmentProducer = set(eqp)

    def getEquipmentsConsumer(self):
        return sorted(list(self.__equipmentConsumer))

    def setEquipmentsConsumer(self, eqps):
        self.__equipmentConsumer = eqps

    def addEquipmentsConsumer(self, eqp):
        self.__equipmentConsumer.add(eqp)

    def removeEquipmentsConsumer(self, eqp):
        self.__equipmentConsumer.remove(eqp)

    def getModulePeriod(self):
        return self.__modulePeriod

    def setModulePeriod(self, modulePeriod):
        self.__modulePeriod = modulePeriod

    def getPortProducer(self):
        return sorted(list(self.__portProducer), key=methodcaller('getName'))

    def getPortProducerFromCard(self, targetCardName):
        allPorts = sorted(list(self.__portProducer), key=methodcaller('getName'))
        cardPorts = []
        for portProd in allPorts:
            cardName = portProd.getCard()
            if cardName:
                if cardName == 'IoCard':
                    cardName = portProd.getIoCard()
                if cardName == targetCardName:
                    cardPorts.append(portProd)
        return cardPorts

    def setPortProducer(self, ports):
        self.__portProducer = set(ports)

    def addPortProducer(self, port):
        self.__portProducer.add(port)

    def removePortProducer(self, port):
        self.__portProducer.remove(port)

    def getPortConsumer(self):
        return sorted(list(self.__portConsumer), key=methodcaller('getName'))

    def getPortConsumerFromCard(self, targetCardName):
        allPorts = sorted(list(self.__portProducer), key=methodcaller('getName'))
        cardPorts = []
        for portProd in allPorts:
            cardName = portProd.getCard()
            if cardName:
                if cardName == 'IoCard':
                    cardName = portProd.getCard()
                if cardName == targetCardName:
                    cardPorts.append(portProd)

    def setPortConsumer(self, ports):
        self.__portConsumer = set(ports)

    def addPortConsumer(self, port):
        self.__portConsumer.add(port)

    def removePortConsumer(self, port):
        self.__portConsumer.remove(port)

    def getNbConsumer(self):
        return self.__nbConsumer

    def setNbConsumer(self, nb):
        self.__nbConsumer = nb