# ----------------------------------------------------------------------------
# - wcmod-project-context                                                    -
# - wild cruncher - sys project context                      core / sys_map  -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcmod_project_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher system mapper project context

.. moduleauthor:: Philippe lambert <lambert@silkan.com>
"""
from wcr_core.modules.sys_map.actions.wcact_reload_hardware_action \
    import wcact_reload_Hardware
from wcr_core.modules.sys_map.actions.wcact_reload_logic_action \
    import wcact_reload_logical
from wcr_core.modules.sys_map.controller.wcctl_controllers import \
    WcctlControllerFactory
from wcr_core.modules.sys_map.wcmod_project_context_api import \
    WcmodSystemProjectContextMP
from wcr_core.project.wprj_project_context import WprjProjectContext
from wcr_core.modules.sys_map.wcmod_mapping_manager import WcmodMappingManager
from wcr_core.modules.sys_map.db.wchdb_standard_operators import \
    WCHDB_SWITCH_LURE_CODE, WCHDB_HOST_LURE_CODE, WCHDB_RACK_LURE_CODE
from wcr_core.modules.hardware.db.whhdb_standard_operators import \
    WHHDB_KAZYMIR_CODE
from wcr_core.modules.sys_map.db.wchdb_connection_points import \
    WCHDB_XPOINT_STD_CODE
from wcr_core.modules.sys_map.wcmod_bus_file_handler \
    import WcmodVLinkFileHandler
from wcr_core.modules.hardware.db.whhdb_Cetrac_switch_instance import \
WhhdbCetracSwitchInstance
from wcr_core.modules.hardware.db.whhdb_kazymir_switch_instance import \
WhhdbKazymirSwitchInstance

class WcmodSystemProjectContext(WprjProjectContext,
                                WcmodSystemProjectContextMP):
    """
    Default project context implementation.
    """

    def __init__(self, gContext, module, projectDB, actx=None, logger=None, ctrlfct=None):
        """
        :param gContext: Global context
        :type gContext: WcorGlobalContextP
        :param module: Application module
        :type module: WpmodSystemApplicationModuleP
        :param projectDB: Project database
        :type projectDB: WprjProjectDatabase
        :param logger: Logger
        :type logger: ScmnLoggerP
        """
        super().__init__(gContext, module, projectDB, logger)
        if ctrlfct is None :
            self.__ctrlFy = WcctlControllerFactory()
        else:
            self.__ctrlFy = ctrlfct
        self.__top = projectDB.getTopHierarchy()
        self.__top.setSourcePath(projectDB.path)

        #: hardware simulation, when set
        self.__hardware = None
        #: logical simulation, when set
        self.__logical = None
        self.__actx = actx
        # #: Mapping Manager
        # # noinspection PyTypeChecker
        self.__mappingManager = WcmodMappingManager(
            self.__top, self)

        #scen interactions functions:
        #function to add operator in scene
        self.OperInstAppearfunc = None
        #function to add connector in scene
        self.ConnectionAppearfunc = None
        #function to remove an operator oa a connector from the scene
        self.SceneRemovefunction = None
        self.SideUpdatefunc = None

        if self.__top.getVLinkFiles():
            f = self.__top.getVLinkFiles()
            self.updateVL(f.path)
        vlAutoF = self.__top.getVLinkFiles()
        if vlAutoF:
            self.updateVLAuto(vlAutoF.path)

        wcact_reload_Hardware(self)
        wcact_reload_logical(self)

    def getHardwareFile(self):
        return self.__hardware

    def getLogicFile(self):
        self.__logical = self.__top.getLogicFile()
        return self.__logical

    def getLFile(self):
        return self.__top.getLFile()

    def getMappingManager(self):
        return self.__mappingManager

    def setHardwareFile(self, lsim):
        self.__hardware = lsim
        # maping the hardware model
        self.__mappingManager.setHardware(self.__hardware)
        self.unbindInvalidBinding()
        if not self.__hardware is None:
            self.updatesceen()
        if not self.SideUpdatefunc is None:
            self.SideUpdatefunc()

    def setLogicFile(self, lsim):

        self.__logical = lsim
        # maping the hardware model
        self.__mappingManager.setLogic(self.__logical)
        if not self.SideUpdatefunc is None:
            self.SideUpdatefunc()
        if self.__actx:
            if self.__actx.getEditor():
                ed = self.__actx.getEditor()
                from wcr_frontend.modules.vlink.gui.wvgui_scene import WvguiGraphicsScene
                if not getattr(ed, "scene", None):
                    return
                elif isinstance(ed.scene, WvguiGraphicsScene):
                    return
                self.__actx.getEditor().initialize()
                self.__actx.getEditor().initialize()

    def getControllerFactory(self):
        return self.__ctrlFy

    def unbindInvalidBinding(self):
        ctblk = self.getProjectDB().getTopHierarchy()
        pportuuidl = []
        for host in self.__mappingManager.getHosts():
            for port in host.getConnectionPoints():
                pportuuidl.append(port.getUUID())

        sportperml = []
        for sw in self.getMappingManager().getSwitchs():
            if (isinstance(sw, WhhdbCetracSwitchInstance) or
                    isinstance(sw, WhhdbKazymirSwitchInstance)):
                for conIo in sw.getAllIoConnectionPoints():
                    sportperml.append(conIo.getUUID())
                if sw.getArmPort() == 'Yes':
                    con = sw.hasArmPort()
                    sportperml.append(con.getUUID())

        for pp, lp in ctblk.getProdBinds():
            if pp not in pportuuidl and pp not in sportperml:
                ctblk.removeProdBind(pp, lp)

        for pp, lp in ctblk.getConsBinds():
            if pp not in pportuuidl and pp not in sportperml:
                ctblk.removeConsBind(pp, lp)

    def unbindInvalidVLinkBinding(self):
        ctblk = self.getProjectDB().getTopHierarchy()
        vluuids = []
        for vlFileName in self.__mappingManager.getAllVirtualLinks():
            for (n, vl) in self.__mappingManager.getAllVirtualLinks()[vlFileName]:
                vluuids.append(vl.getUUID())
        if self.__top.getVLinkBlock():
            for vl in self.__top.getVLinkBlock().getInstances():
                if vl.getVLMessagesName():
                    vluuids.append(vl.getUUID())

        for pp, lp in ctblk.getProdBinds():
            if lp not in vluuids:
                ctblk.removeProdBind(pp, lp)

        for pp, lp in ctblk.getConsBinds():
            if lp not in vluuids:
                ctblk.removeConsBind(pp, lp)


    def updatesceen(self):
        ctblk = self.getProjectDB().getTopHierarchy()

        #cleaning the scene from old operator and connectors
        self.delOperatorAndConnections()

        #temporary revers maping for acces operator instance from they sources
        reversmap = {}

        for host in self.__mappingManager.getHosts():
            #creating hostlures
            nhost = ctblk.createOperatorInstance(WCHDB_HOST_LURE_CODE,
                                                 name=host.getName())
            nhost.setCards(host.getCards())
            nhost.setIoCards(host.getIoCards())
            for ioModule in host.getIoCards():
                card = host.getIoCardBind(ioModule)
                nhost.setIoCardBind(ioModule, card)
            nhost.setGuiData(host)
            nhost.setId(host.getId())
            reversmap[nhost.getUUIDsource()] = nhost
            #creating ports
            for hcon in host.getConnectionPoints():
                #createconnection point
                con = nhost.createConnectionPoint(
                    ctyp=WCHDB_XPOINT_STD_CODE,
                    name=hcon.getName(),
                    cpos=hcon.getPosition(),
                    gvty=hcon.getGravity())
                con.setUUIDSource(hcon.getUUID())
                con.setSpeed(hcon.getSpeed())
                con.setCard(hcon.getCard())
                con.setCode(hcon.getCode())
                con.setActive(hcon.getActive())
                con.setDescription(hcon.getDescription())
                reversmap[con.getUUIDSource()] = con
            # add pci ports
            for card in host.getCards():
                pciPort = host.getPCIPort(card)
                if pciPort:
                    nhost.setPCIPort(pciPort)

            if not self.OperInstAppearfunc is None:
                self.OperInstAppearfunc(nhost)

        for rack in self.__mappingManager.getRacks():
            #creating hostlures
            nrack = ctblk.createOperatorInstance(WCHDB_RACK_LURE_CODE,
                                                 name=rack.getName())
            rackSymbol = '{} \n {}/{}'.format(rack.rackSymbol, rack.nb_card,
                                              rack.MAX_CARD)
            nrack.setCards(rack.getCards())
            nrack.setCustomText(rackSymbol)
            nrack.setGuiData(rack)
            nrack.setId(rack.getId())
            reversmap[nrack.getUUIDsource()] = nrack
            #creating ports
            for rcon in rack.getConnectionPoints():
                con = nrack.createConnectionPoint(
                    ctyp=WCHDB_XPOINT_STD_CODE,
                    name=rcon.getName(),
                    cpos=rcon.getPosition(),
                    gvty=rcon.getGravity())
                con.setUUIDSource(rcon.getUUID())
                con.setSpeed(rcon.getSpeed())
                con.setCard(rcon.getCard())
                con.setCode(rcon.getCode())
                con.setActive(rcon.getActive())
                con.setDescription(
                    rcon.getDescription())
                reversmap[con.getUUIDSource()] = con

            if not self.OperInstAppearfunc is None:
                self.OperInstAppearfunc(nrack)

        for switch in self.__mappingManager.getSwitchs():
            # creating switchlures
            nswitch = ctblk.createOperatorInstance(WCHDB_SWITCH_LURE_CODE,
                                                   name=switch.getName())
            nswitch.setGuiData(switch)
            nswitch.setId(switch.getId())
            reversmap[nswitch.getUUIDsource()] = nswitch
            if switch.getOperator().getCode() == WHHDB_KAZYMIR_CODE:
                nswitch.setArmPort(switch.hasArmPort())
            try:
                nswitch.setIoCards(switch.getIoCards())
                for ioModule in switch.getIoCards():
                    card = switch.getIoCardBind(ioModule)
                    nswitch.setIoCardBind(ioModule, card)
            except:
                pass
            # creating ports
            for scon in switch.getConnectionPoints():
                # createconnection point
                con = nswitch.createConnectionPoint(
                    ctyp=WCHDB_XPOINT_STD_CODE,
                    name=scon.getName(),
                    cpos=scon.getPosition(),
                    gvty=scon.getGravity())
                con.setUUIDSource(scon.getUUID())
                con.setSpeed(scon.getSpeed())
                con.setCode(scon.getCode())
                con.setCard(scon.getCard())
                con.setActive(scon.getActive())
                con.setDescription(scon.getDescription())
                reversmap[con.getUUIDSource()] = con

            if not self.OperInstAppearfunc is None:
                self.OperInstAppearfunc(nswitch)

        # create connections
        for conn in self.__mappingManager.getConections():
            sobj = conn.getStartObject()
            scon = conn.getStartConnectionPoint()
            eobj = conn.getEndObject()
            econ = conn.getEndConnectionPoint()
            ncon = ctblk.createConnection(reversmap[sobj.getUUID()],
                                          reversmap[scon.getUUID()],
                                          reversmap[eobj.getUUID()],
                                          reversmap[econ.getUUID()],
                                          name=conn.getName(),
                                          path=conn.getPath().getPoints())
            if not self.ConnectionAppearfunc is None:
                self.ConnectionAppearfunc(ncon)

    def getbindedVlink(self, hostuuid):
        modulesnamelist=[]
        for h,vl in self.getProjectDB().getTopHierarchy().getProdBinds():
            if str(h) == str(hostuuid):
                if self.__mappingManager. \
                        getVlinkByUUID(vl):
                    modulesnamelist.append(self.__mappingManager.\
                                           getVlinkNameByUUID(vl))

        for h,vl in self.getProjectDB().getTopHierarchy().getConsBinds():
            if str(h) == str(hostuuid):
                if self.__mappingManager. \
                        getVlinkByUUID(vl):
                    modulesnamelist.append(self.__mappingManager. \
                                           getVlinkNameByUUID(vl))
        return modulesnamelist

    def setaddOperInstguifunc(self, func):
        self.OperInstAppearfunc = func

    def setaddConnectionguifunc(self, func):
        self.ConnectionAppearfunc = func

    def setremovefunc(self, func):
        self.SceneRemovefunction = func

    def setsideupdatefunc(self, func):
        self.SideUpdatefunc = func

    def delOperatorAndConnections(self):
        ctblk = self.getProjectDB().getTopHierarchy()
        lops = []
        for op in ctblk.getOperatorInstances():
            lops.append(op)
        for a in lops:
            if not self.SceneRemovefunction is None:
                self.SceneRemovefunction(a)
            ctblk.remove(a)

        lcons = []
        for c in ctblk.getConnections():
            lcons.append(c)
        for con in lcons:
            if not self.SceneRemovefunction is None:
                self.SceneRemovefunction(con)
            ctblk.remove(con)

    def updateVL(self, path):
        """
        Add the path file to the Mapping manager
        """
        # Recuperation des messages
        handler = WcmodVLinkFileHandler(self.getProjectDB().getModule())
        vlink = handler.handleFile(path, self)
        if not vlink:
            raise ValueError("Wrong source file for Traffic Route Source.")
            # error
        self.__top.setVLinkBlock(vlink)
        vlink.setSourcePath(path)
        self.getMappingManager().addVLinkfiletb(vlink, path, hasDelVL=True)

    def updateVLAuto(self, path):
        """
        Add the path file to the Mapping manager
        """
        # Recuperation des messages
        handler = WcmodVLinkFileHandler(self.getProjectDB().getModule())
        try:
            vlink = handler.handleFile(path, self)
        except ValueError:
            return
        if not vlink:
            raise ValueError("Wrong source file for Traffic Route Source.")
            # error
        self.__top.setVLinkBlock(vlink)
