# ----------------------------------------------------------------------------
# - wpact-load-logical-simulation-action                                     -
# - wild cruncher: load logical simulation action    core / system / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpact_load_logical_simulation_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Logical simulation load action

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.action.wact_action_parameter import WactActionParameter
from wcr_core.action.wact_action_parameter_types import \
    WactActionOpenFileByPayloadParameterType
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_BLOCK_SIMULATION
from wcr_core.modules.system.actions.wpact_action_base import WpactActionBase
from wcr_core.modules.system.actions.wpact_reload_logical_simulation_action \
    import wpact_reloadLogicalSimulation


WPACT_LOGICAL_SIMULATION_FILE_PARAMETER = WactActionParameter(
    'source', 'Logical Simulation', 'Logical simulation file to import',
    WactActionOpenFileByPayloadParameterType(
        WCMN_PAYLOAD_BLOCK_SIMULATION))


class _WpactLoadLogicalSimulationAction(WpactActionBase):
    """
    Project editor action to set & load the logical simulation.
    """

    def __init__(self):
        super().__init__(
            'Set Logical Simulation...',
            'Set & load the logical simulation for this system project',
            WPACT_LOGICAL_SIMULATION_FILE_PARAMETER)

    def _doExecute(self, actx, paramValuePairs):
        file = self._getParamValue(
            WPACT_LOGICAL_SIMULATION_FILE_PARAMETER, actx, paramValuePairs)
        """:type: WcmnFile | None"""
        if not file:
            return
        pctx = actx.getProjectContext()
        """:type: WpmodSystemProjectContextMP"""
        tblk = pctx.getProjectDB().getTopHierarchy()
        """:type: WphdbTopBlock"""
        tblk.setLogicalSimulationFile(file)
        wpact_reloadLogicalSimulation(pctx)


#: Logical simulation load action
WPACT_LOAD_LOGICAL_SIMULATION_ACTION = _WpactLoadLogicalSimulationAction()
