# ----------------------------------------------------------------------------
# - wphdb-wrapper-registry                                                   -
# - wild cruncher: raw data wrapper registry              core / system / db -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - author    (c) 2014-2014 samuel longchamps                                -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wphdb_wrapper_registry
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher system mapper hierarchy raw data wrapper
                  registry

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
.. moduleauthor:: Samuel Longchamps <samuel.longchamps@silkan.com>
"""
from wcr_core.db.whdb_block import WHDB_TOPLEVEL_BLOCK_CODE
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.db.whdb_wrapper_registry import WhdbWrapperRegistry
from wcr_core.dbstore_api.wdbs_block_api import WdbsBlockP
from wcr_core.dbstore_api.wdbs_instance_api import WdbsOperatorInstanceP
from wcr_core.dbstore_api.wdbs_connection_point_api import WdbsConnectionPointP
from wcr_core.modules.system.db.wphdb_block import WphdbClusterBlock, \
    WPHDB_CLUSTER_BLOCK_CODE, WphdbTopBlock
from wcr_core.modules.system.db.wphdb_phybus_instance import WphdbPhyBusInstance
from wcr_core.modules.system.db.wphdb_host_instance import WphdbHostInstance
from wcr_core.modules.system.db.wphdb_operator import WPHDB_OPER
from wcr_core.modules.system.db.wphdb_standard_operator import WPHDB_HOST_CODE, \
    WPHDB_BUS_CODE

WPHDB_STD_OPERINST_CLS = {WPHDB_HOST_CODE: WphdbHostInstance,
                          WPHDB_BUS_CODE: WphdbPhyBusInstance,
                          }


class WphdbWrapperRegistry(WhdbWrapperRegistry):
    """
    Manages DBS-HDB wrappers.
    """
    def __init__(self, xpntClzzs):
        """
        :param xpntClzzs: Connection point classes/constructors, by code
        :type xpntClzzs: dict[str, () -> WphdbConnectionPointBase]
        """
        super().__init__()

        #: Connection point classes, by code
        self.__xpntClzzs = xpntClzzs

    def createWrapper(self, obj, *args):
        if isinstance(obj, WdbsBlockP):
            if obj.getEntityType() == WPHDB_CLUSTER_BLOCK_CODE:
                return WphdbClusterBlock(obj, self)
            elif obj.getEntityType() == WHDB_TOPLEVEL_BLOCK_CODE:
                return WphdbTopBlock(obj, self)
            else:
                raise ValueError('Unknown block type:', obj.getEntityType())
        elif isinstance(obj, WdbsOperatorInstanceP):
            return self._createOperatorInstance(obj)
        elif isinstance(obj, WdbsConnectionPointP):
            clazz = self.__xpntClzzs.get(obj.getEntityType())
            assert clazz, str(self.__xpntClzzs)
            return clazz(obj, self)
        else:
            return super().createWrapper(obj, *args)

    def _createOperatorInstance(self, opInst):
        """
        Create an operator instance object based on the type of operator

        :param opInst: Raw operator instance
        :type opInst: WdbsOperatorInstanceP
        :return: Operator instance created
        :rtype: WhdbOperatorInstanceP
        """
        operator = opInst.getOperator()
        cls = WPHDB_STD_OPERINST_CLS.get(operator)

        return cls(opInst, self) if cls is not None else \
            WhdbOperatorInstance(WPHDB_OPER[operator], opInst, self)
