# ----------------------------------------------------------------------------
# - wpmod-connection-point-type                                              -
# - wild cruncher: connection point type descriptor            core / system -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpmod_connection_point_type
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Connection point type descriptor

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WpmodConnectionPointTypeP(metaclass=ABCMeta):
    """
    Connection point type protocol.
    """

    @abstractmethod
    def getCode(self):
        """
        Get the code for the connection point type.

        :return: Code
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getName(self):
        """
        Get the name for the connection point type.

        :return: Name
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getPrefix(self):
        """
        Get the name prefix for the connection point type.

        :return: Name prefix
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getConfigMeta(self):
        """
        Get the configuration meta-data for the connection point type.

        :return: Configuration meta-data
        :rtype: WdbsMetaTypeP | None
        """
        raise NotImplementedError()

    @abstractmethod
    def getClass(self):
        """
        Get the class for the connection point type.

        :return: Class / constructor
        :rtype: () -> WphdbConnectionPointBase
        """
        raise NotImplementedError()

    @abstractmethod
    def getDefaultCreationArguments(self):
        """
        Get the default creation arguments for new connection points of this
        type.

        :return: Default creation arguments
        :rtype: dict[str, object] | None
        """
        raise NotImplementedError()


class WpmodConnectionPointType(WpmodConnectionPointTypeP):
    def __init__(self, code, name, prefix, clazz, cfgMeta=None, crtnArgs=None):
        """
        :param code: Connection point type code
        :type code: str
        :param name: Connection point type name
        :type name: str
        :param prefix: Name prefix
        :type prefix: str
        :param clazz: Class / constructor
        :type clazz: () -> WphdbConnectionPointBase
        :param cfgMeta: Configuration meta-data
        :type cfgMeta: WdbsMetaTypeP | None
        :param crtnArgs: Default creation arguments
        :type crtnArgs: dict[str, object] | None
        """
        self.__code = code
        self.__name = name
        self.__prefix = prefix
        self.__cfgMeta = cfgMeta
        self.__creationArgs = crtnArgs
        self.__class = clazz

    def getCode(self):
        return self.__code

    def getName(self):
        return self.__name

    def getPrefix(self):
        return self.__prefix

    def getConfigMeta(self):
        return self.__cfgMeta

    def getClass(self):
        return self.__class

    def getDefaultCreationArguments(self):
        return self.__creationArgs
