# ----------------------------------------------------------------------------
# - wplg-library-reader-plugin                                               -
# - wild cruncher: library reader                              core / plugin -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wplg_library_reader_plugin
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Library reader

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from functools import partial
from xml.etree.ElementTree import ElementTree

from silkan_common.scmn_file_path import scmn_isfile, scmn_listdir, \
    scmn_path_dir, scmn_abspath
from wcr_core.plugin.wplg_installer import WplgPluginDescriptor
from wcr_core.plugin.wplg_plugin_reader_plugin_api import \
    WplgPluginReaderPluginP


class WplgLibraryReaderPlugin(WplgPluginReaderPluginP):
    """
    Plugin-reader plugin to read operator libraries from XML descriptor files.
    """

    def readAll(self, path, installer):
        if scmn_isfile(path):
            self.readLibrary(path, installer)
        else:
            for f in scmn_listdir(
                    path, fullpath=True, pattern=r'^lib_\w+_allxx.xml$'):
                self.readLibrary(f, installer)

    def _readOperator(self, paths, lcod, ictx):
        for path in paths:
            ictx.installOperatorFile(path, lcod)

    def readLibrary(self, path, installer):
        """
        Read the library at the given path.

        :param path: Library path
        :type path: str
        :param installer: Installer
        :type installer: WplgInstaller
        """

        def mkabs(fpth):
            return scmn_abspath(fpth, base=scmn_path_dir(path))

        tree = ElementTree(file=path)
        libNode = tree.getroot()
        if libNode.tag != 'wcr-library':
            return

        plgCode = libNode.get('code')
        plgName = libNode.get('name')
        plgModule = libNode.get('module')

        descNode = libNode.find('description')
        plgDesc = descNode.text if descNode is not None else None

        paths = [mkabs(libNode.get('python-dir') or '.')]
        opths = [mkabs(n.get('path')) for n
                 in libNode.findall('wcr-operator-ref')]
        installs = [('core', plgModule,
                     partial(self._readOperator, opths, plgCode))]
        """:type: list[(str, str, (WplgInstallContextP) -> _)]"""

        plgDescriptor = WplgPluginDescriptor(
            plgCode, plgName, plgDesc, installs, paths)

        installer.installPlugin(plgDescriptor)
