# ----------------------------------------------------------------------------
# - wgui-about-dialog                                                        -
# - wild cruncher - cockpit: About-dialog                                    -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2013 clemens oertel                                   -
# - copyright (c) 2013-2013 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wgui_about_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher 'About'-dialog

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QHBoxLayout, QVBoxLayout, QTabWidget, \
	QDialogButtonBox, QLabel
from PyQt5.QtGui import QFont, QPixmap, QIcon

from silkan_common import __version__, __name__, __date__

class WguiAboutDialog(QDialog):
    """
    Wild Cruncher 'About'-dialog.
    """

    def __init__(self, parent=None):
        """
        :param parent: Parent widget
        """
        super().__init__(parent)

        self.setWindowTitle('About {}'.format(__name__))
        self.setWindowIcon(QIcon(':/img/app_was_slki.png'))
        self.setMinimumWidth(500)
        self.setModal(True)

        self.setLayout(self._createLayout())

    def _createLayout(self):
        """
        Create the layout for the 'About'-dialog.

        :return: Layout for the 'About'-dialog
        :rtype: QLayout
        """
        layout = QVBoxLayout()

        topLayout = QHBoxLayout()

        logo = QLabel('')
        logo.setPixmap(QPixmap(':/img/app_was_slkv.png'))
        logo.setAlignment(Qt.AlignLeft | Qt.AlignVCenter)
        topLayout.addWidget(logo)

        title = QLabel('{} {}'.format(__name__, __version__))
        title.setFont(QFont(None, 24, 99))
        title.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
        topLayout.addWidget(title)

        layout.addLayout(topLayout)
        layout.addSpacing(20)

        layout.addWidget(QLabel('<b>© {} by Silkan</b>'.format(__date__)))

        layout.addSpacing(20)

        tabs = QTabWidget()
        tabs.addTab(QLabel('A lot of copyright text'), 'Copyright')
        tabs.addTab(QLabel('A lot of licencing text'), 'Licence')

        layout.addWidget(tabs)
        layout.addSpacing(10)

        buttons = QDialogButtonBox(QDialogButtonBox.Ok)
        buttons.accepted.connect(self.accept)
        layout.addWidget(buttons)

        return layout
