# ----------------------------------------------------------------------------
# - wgui-toolbox-item                                                        -
# - wild cruncher - cockpit : diagram icon item widgets                      -
# ----------------------------------------------------------------------------
# - author    (c) 2012-2012 amaury darsch                                    -
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------

# noinspection PyUnresolvedReferences
import wcr_frontend.gui.wgui_icons

from wcr_frontend.gui.wgui_scene_item_ids import *
from wcr_core.db_api.whdb_block_api import WHDB_DEFAULT_BLOCK_CODE
from wcr_core.plugin.wplg_library import WLIB_ICON_SVG, WLIB_ICON_PNG32


# ----------------------------------------------------------------------------
# - base icon section                                                        -
# ----------------------------------------------------------------------------

WGUI_ICON_PNG32 = WLIB_ICON_PNG32

WGUI_ICON_SVG = WLIB_ICON_SVG


class WguiToolboxItem(object):
    """
    Description of a toolbox item (besides operators).
    """

    def __init__(self, key, icon, name, desc):
        """
        :param key: Key for look-ups (action, etc.)
        :type key: str
        :param icon: Path to icon for button
        :type icon: str
        :param name: Name
        :type name: str
        :param desc: Description
        :type desc: str
        """
        #: Key for lookups
        self.__key = key
        #: Path to button icon
        self.__icon = icon
        #: Name
        self.__name = name
        #: Description
        self.__description = desc

    @property
    def key(self):
        """
        Get the item's key.

        :return: Lookup-key for the item
        :rtype: str
        """
        return self.__key

    @property
    def icon(self):
        """
        Get the item's icon.

        :return: Path to the item's button icon
        :rtype: str
        """
        return self.__icon

    @property
    def name(self):
        """
        Get the item's name.

        :return: Name of the icon
        :rtype: str
        """
        return self.__name

    @property
    def description(self):
        """
        Get the item's description.

        :return: Description for the item
        :rtype: str
        """
        return self.__description


#: Dictionary of toolbox icons
WGUI_ICONS = {
    SLCT_ITEM_ID: ':/img/app_was_xpnt.png',
    'wcr.shape.line': ':/img/app_was_line.png',
    'wcr.shape.arrow': ':/img/app_was_arow.png',
    'wcr.shape.rectangle': ':/img/app_was_rect.png',
    'wcr.shape.oval': ':/img/app_was_elps.png',
    'wcr.shape.text': ':/img/app_was_text.png',
    CNCT_ITEM_ID: ':/img/app_was_wire.png',
    XPNT_ITEM_ID: ':/img/app_was_nicn.png',
    ACTN_ITEM_ID: ':/img/app_was_cmdb.png',
    WHDB_DEFAULT_BLOCK_CODE: ':/img/app_was_blok.png'
}


#: Dictionary of toolbox items (by key)
WGUI_TOOLBOX_ITEMS = {}


def _wgui_addToolboxItem(key, name, desc, mime=None):
    """
    Add an item to the dictionary of toolbox items.

    :param key: Lookup-key of the item
    :type key: str
    :param name: Name of the item
    :type name: str
    :param desc: Description for the item
    :type desc: str
    :param mime: MIME-type of the item, if applicable
    :type mime: str
    """
    icon = WGUI_ICONS.get(key)
    if icon is None and mime is not None:
        icon = WGUI_ICONS.get(key[:key.rindex('.') + 1] + mime)
    WGUI_TOOLBOX_ITEMS[key] = WguiToolboxItem(
        key, icon, name, desc)


# ----------------------------------------------------------------------------
# - edit icon section                                                        -
# ----------------------------------------------------------------------------

_wgui_addToolboxItem(SLCT_ITEM_ID, 'select', 'Select')


# ----------------------------------------------------------------------------
# - shape icon section                                                       -
# ----------------------------------------------------------------------------

_wgui_addToolboxItem(LINE_ITEM_ID, 'line', 'Line object')
_wgui_addToolboxItem(AROW_ITEM_ID, 'arrow', 'Arrow object')
_wgui_addToolboxItem(RECT_ITEM_ID, 'rectangle', 'Rectangle object')
_wgui_addToolboxItem(ELPS_ITEM_ID, 'oval', 'Ellipse/oval object')
_wgui_addToolboxItem(TEXT_ITEM_ID, 'text', 'Text object')

_wgui_addToolboxItem(ACTN_ITEM_ID, 'action', 'Action button')


# ----------------------------------------------------------------------------
# - editing icon section                                                     -
# ----------------------------------------------------------------------------

_wgui_addToolboxItem(CNCT_ITEM_ID, 'connection', 'Block connection')
_wgui_addToolboxItem(XPNT_ITEM_ID, 'connector', 'Instance connector')
