# ----------------------------------------------------------------------------
# - wegui-blbxmessage-instance-props-dialog                                  -
# - wild cruncher - cockpit :                                                -
# -         Black box message properties dialog widgets                      -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wegui_blbxmessage_instance_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher instance properties dialog

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from PyQt5.QtCore import QAbstractTableModel, Qt
from PyQt5.QtWidgets import QComboBox, QHBoxLayout, QWidget, QVBoxLayout, \
	QPushButton, QItemDelegate, QLabel, QTableView, QHeaderView, QAbstractItemView
    
from wcr_frontend.gui.wgui_props_dialog_widget import WguiPropertiesWidgetP
from wcr_frontend.gui.wgui_operator_instance_props_dialog import \
    WguiOperatorInstancePropertiesDialog
from wcr_frontend.gui.wgui_utils import wgui_qcb_get_data

class WbguiExtModulePropertiesDialog(
    WguiOperatorInstancePropertiesDialog):
    _TYPE_NAME = 'External module Instance'

    _SHOW_OPERATOR_WIDGET = False

    def __init__(self, inst, pctx, updateF, selectXpnt=None, parent=None):
        """
        :param inst: the instance object
        :type inst: WhdbInstanceP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: () -> _
        :param parent: Parent widget
        """
        super().__init__(inst, pctx, updateF, selectXpnt, parent)


    def _createAdditionalInstancePropertiesWidgets(self, inst, pctx):
        return []

    # noinspection PyMethodMayBeStatic
    def _createAdditionalPropertiesWidgets(self, inst, pctx):
        return [('Messages', _WbguiExtModuleMessageProperties(pctx, inst))]

    def _createConnectionPointsPropertiesWidget(self, inst, pctx):
        return None


class _WbguiExtModuleMessageProperties(WguiPropertiesWidgetP):
    """
    Property widget for host capabilities.
    """

    def __init__(self, pctx, instance, parent=None):
        """
        :param module:
        :type module:
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: externalmodule instance
        self.__instance = instance
        # : Project context
        self.__pctx = pctx


        self.__messageFormatF = lambda a: a[1]+' - '+a[2] if a else None
        #: Bus binding table model
        self.__bndgTblM = _WbguiMessageBindingTableModel(self.__messageFormatF)
        #
        self.__bbmessageListPrvF = lambda : self.__pctx.getProjectDB().\
            getTopHierarchy().getBBMessages()
        self.__bndgItmDlgt = _WbguibbmessageBindDelegate(self.__bbmessageListPrvF, self.__messageFormatF)

            #: Bus binding table widget
        self.__bndgTblW = QTableView()

        #: Add entry button
        self.__addButton = QPushButton('Add')
        #: Delete entry button
        self.__deleteButton = QPushButton('Delete')

        self._initElements()
        self.setLayout(self._createLayout())
        self.updateFields()

    def _initElements(self):
        self.__bndgTblW.setModel(self.__bndgTblM)
        self.__bndgTblW.horizontalHeader().setSectionResizeMode(
            0, QHeaderView.Stretch)
        self.__bndgTblW.horizontalHeader().setSectionResizeMode(
            1, QHeaderView.Stretch)
        self.__bndgTblW.setEditTriggers(QAbstractItemView.AllEditTriggers)
        self.__bndgTblW.setItemDelegate(self.__bndgItmDlgt)

        self.__addButton.clicked.connect(self._addEntry)
        self.__deleteButton.clicked.connect(self._deleteSelEntries)



    def _createLayout(self):
        """
        Create the layout.

        :return: Layout
        :rtype: QLayout
        """
        layout = QVBoxLayout()
        layout.setContentsMargins(0,0,0,0)

        layout.addWidget(self.__bndgTblW)

        btnLayout = QHBoxLayout()
        btnLayout.addWidget(self.__addButton)
        btnLayout.addWidget(self.__deleteButton)
        layout.addLayout(btnLayout)

        return layout

    def updateFields(self):
        for i in reversed(range(0, self.__bndgTblM.rowCount())):
            self.__bndgTblM.deleteRow(i)
        messages = self.__instance.getMessages()
        for bbm in self.__bbmessageListPrvF():
            for m in messages:
                if bbm[0] == m[0] and bbm[2] == m[1]:
                    self.__bndgTblM.addRow(bbm)

    def save(self):
        self.__instance.removeAllMessages()
        for i in range(0,self.__bndgTblM.rowCount()):
            d=self.__bndgTblM.data(self.__bndgTblM.index(i,0), Qt.UserRole)
            """Type : (file_path, file_name, message_name)"""
            if d :
                self.__instance.addMessage((d[0],d[2]))

    def _addEntry(self):
        """
        Add entry in the model
        """
        self.__bndgTblM.addRow()

    def _deleteSelEntries(self):
        """
        Delete selected entries from the model
        """
        selIdx = self.__bndgTblW.selectedIndexes()
        """:type: list[QModelIndex]"""
        self.__bndgTblM.deleteRows([idx.row() for idx in selIdx])


class _WbguiMessageBindingTableModel(QAbstractTableModel):
    """
    Bus Binding table model.
    """

    TITLE_MESSAGES = 'Binded BlackBox Messages'

    HEADERS = [TITLE_MESSAGES]

    def __init__(self, formatF, parent=None):
        """
        :param formatF: Formatting function
        :type formatF: WshdbBusInstance -> str
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: Buses formatting function
        self.__formatF = formatF

        #: Entries as list of blackbox message
        self.__entries = []

        self._update()

    def _update(self):
        self.beginResetModel()
        self.endResetModel()

    def _doUpdate(self):
        pass

    def beginResetModel(self):
        super().beginResetModel()

    def endResetModel(self):
        self._doUpdate()
        super().endResetModel()

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
               0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def addRow(self, message=None):
        """
        Add a row at the end of the list (append)

        :param pbus: Physical bus to be set
        :type pbus: WphdbPhyBusInstance | None
        :return: The appended row index
        :rtype: int
        """
        index = self.index(self.rowCount(), 0)
        self.beginInsertRows(index, self.rowCount(), self.rowCount())

        self.__entries.append((message,0))

        self.endInsertRows()
        self._update()

    def deleteRows(self, rowIndices):
        """
        :param rowIndices: Iterable of indices
        :type rowIndices: Iterable[int]
        """
        # Make sure no duplicate exist
        rowList = list(set(rowIndices))
        rowList.sort(reverse=True)

        for row in rowList:
            self.deleteRow(row)

    def deleteRow(self, rowIndex):
        """
        Delete a row entry

        :param rowIndex: Row to the delete
        :type rowIndex: int
        """
        if not (0 <= rowIndex < self.rowCount()):
            raise ValueError('Index of row is out of bounds')

        index = self.index(rowIndex, 0)
        self.beginRemoveRows(index, rowIndex, rowIndex)

        message = self.__entries[rowIndex]
        if message is not None:
            try:
                #unbinding function
                None
            except ValueError:
                pass

        self.__entries.pop(rowIndex)

        self.endRemoveRows()
        self._update()

    def index(self, row, column, parentIndex=None, *args, **kwargs):
        return self.createIndex(row, column)

    def data(self, index, role=None):
        row = index.row()
        column = index.column()
        if 0 <= column <= 1:
            bus = self.__entries[row][column]
            if role in [Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole]:
                return self.__formatF(bus)
            elif role == Qt.UserRole:
                return bus
        return None

    def setData(self, index, data, role=None):
        """
        :type index: QModelIndex
        :type data: object
        """
        assert role == Qt.UserRole
        self._setBusEntry(index, data)

    def _setBusEntry(self, index, newBBMessage):
        """
        :param index: Index of the physical bus in the table
        :type index: QModelIndex
        :param newBBMessage: New Black Box message info
        :type newBBMessage: (path, prokect/file name, bbm name)
        """

        pbus, _ = self.__entries[index.row()]

        self.__entries[index.row()] = (newBBMessage, 0)


    def flags(self, index):
        flgs = super().flags(index)
        flgs |= Qt.ItemIsEditable
        return flgs

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__entries)

    def columnCount(self, parent=None, *args, **kwargs):
        return 1





class _WbguibbmessageBindDelegate(QItemDelegate):
    """
    Delegate for bus items (both physical and logical)
    """

    def __init__(self, bbmessListPrv, formatF, parent=None):
        """
        :type bbmessListPrv: () -> list[blackboxmessage]
        :type formatF: (blackboxmessage) -> str
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(parent)

        self.formatF = formatF
        self.__bbmessLstF = bbmessListPrv

    def createEditor(self, parent, option, index):
        """
        Create a combobox containing adequate choices for the selected item

        :type parent: QWidget
        :type option: QStyleOptionViewItem
        :type index: QModelIndex
        """
        def save(*_):
            self.setModelData(editor, index.model(), index)

        editor = QComboBox(parent)

        # Iterate through the messages using the adequate provider
        # and add items to the combo box
        busPrvF = self.__bbmessLstF
        for bus in busPrvF():
            editor.addItem(self.formatF(bus), bus)
        editor.currentIndexChanged.connect(save)

        return editor

    def setEditorData(self, editor, index):
        """
        :type editor: QComboBox
        :type index: QModelIndex
        """
        data = index.model().data(index, Qt.UserRole)
        for index in range(editor.count()):
            if editor.itemData(index, Qt.UserRole) == data:
                editor.setCurrentIndex(index)
                break
        else:
            editor.setCurrentIndex(0)

    def setModelData(self, editor, model, index):
        """
        :type editor: QComboBox
        :type model: QAbstractItemModel
        :type index: QModelIndex
        """
        model.setData(index, wgui_qcb_get_data(editor), Qt.UserRole)


