# ----------------------------------------------------------------------------
# - wbgui-scene                                                              -
# - wld cruncher - cockpit : graphic scene widget                           -
# ----------------------------------------------------------------------------
# - author    (c) 2017-2017 achraf mahti                                     -
# - copyright (c) 2017-2017 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbgui_scene
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wbgui graphics scene editor

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""

from wcr_core.db.whdb_utils import whdb_getNextInstanceName, \
    whdb_getNextFunctionName
from wcr_core.modules.bus.db.wbhdb_standard_operators import WBHDB_BUS_CODE
from wcr_core.modules.bus.db.wbhdb_connection_points import \
    WBHDB_XPOINT_UDP_CODE
from wcr_frontend.gui.wgui_scene import WguiGraphicsScene
from wcr_frontend.gui.wgui_utils import wgui_snap_point

class WbguiGraphicsScene(WguiGraphicsScene):
    """
    The WbguiGraphicsScene class is the class responsible to manage
    specific scene action in hardware view that WguiGraphicsScene don't handle
    """

    def __init__(self, pjdb, config, actions, pctx, cctx, actx, mod,
                 parent=None):

        # todo: initialise the right library to consider as cards
        super().__init__(pjdb, config, actions, pctx, cctx, actx, parent=parent)
        self.__projectCtx = pctx
        self.__actx = actx
        
    def release(self):
        pass

    def createFunctionItem(self, pos, oper=None, inst=None):
        """
        Handle the fact that the shape of a bus is like a rectangle
        and not to create it in the scene view.
        Else add a function item by position it not a card

        :param pos: the function position
        :type pos: QPointF
        """
        # select the operator by mode
        if oper is None:
            oper = self.getOperatorMode()

        if oper is None: return
        name = whdb_getNextFunctionName(oper, self.topLevelModelBlock)
        width = None
        height = None
        # get the function by operator
        if oper.getCode() == WBHDB_BUS_CODE:
            width = 150
            height = 30
        func = self.topLevelModelBlock.createOperatorInstance(oper, name=name,
                                                              width=width,
                                                              height=height)
        # set the function position
        cfg = self.config
        pos = wgui_snap_point(pos, cfg.snapGridX, cfg.snapGridY)
        func.setPosition(pos.x(), pos.y())

        if inst:
            # add msgs to the module
            for data in inst.getData():
                func.addMessage(data)
        self.addFunctionItem(func).focusNameField()
        return func

    def addFunctionItem(self, func):
        """
        Create a function graphics item for the given function database
        object.

        :param func: The function object
        :type func: WhdbOperatorInstanceP
        :return: The newly created connection scene item
        :rtype: WguiOperatorInstanceSceneItem
        """
        if func is None: return None
        if func.getZ() is None:
            func.setZ(self.getTopZValue() + 1)

        item = self.sitmF.createOperatorInstanceSceneItem(
            func, self.viewContext)
        self.addItem(item)
        return item

