# ----------------------------------------------------------------------------
# - whgui-connection-parameter-edition-delegate                              -
# - wild cruncher - cockpit : delegate for editing connection parameters     -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whgui_connection_parameter_edition_delegate
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher connection point parameter edition delegate

.. moduleauthor:: Philippe lambert <lambert@silkan.com>
"""

import re

from PyQt5.QtCore import Qt, QRegExp
from PyQt5.QtWidgets import QComboBox, QWidget, QLineEdit, QItemDelegate
from PyQt5.QtGui import QRegExpValidator
from wcr_frontend.gui.wgui_utils import wgui_qcb_get_data
from .whgui_scene_item_factory import BANDWIDTH_XPOINT_SPEED_1,\
    BANDWIDTH_XPOINT_SPEED_0_1, BANDWIDTH_XPOINT_SPEED_10,\
    BANDWIDTH_XPOINT_SPEED_0_0_1, BANDWIDTH_XPOINT_ARINC825_SPEED_83_33,\
    BANDWIDTH_XPOINT_ARINC825_SPEED_125,\
    BANDWIDTH_XPOINT_ARINC825_SPEED_250, BANDWIDTH_XPOINT_ARINC825_SPEED_500,\
    BANDWIDTH_XPOINT_ARINC825_SPEED_1000
from wcr_core.modules.hardware.db.whhdb_connection_point_base_meta import *


class WhguiConnectionParameterBindDelegate(QItemDelegate):
    """
    Delegate for editing hardware connections properties
    """

    def __init__(self, ConTypl, formatF=None, parent=None):
        """
        :type ConTypl: list[ConTyp]
        :type formatF: (elem,column) -> str
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(parent)
        self.formatF = formatF
        self.__ConnTyp = ConTypl

    def createEditor(self, parent, option, index):
        """
        Create a editors for card table the selected item

        :type parent: QWidget
        :type option: QStyleOptionViewItem
        :type index: QModelIndex
        """

        def save(*_):
            self.setModelData(editor, index.model(), index)

        editor = None
        col = index.column()
        # the table have only tow column and the first is not editable
        assert col == 1
        editor = QComboBox(parent)
        strtline = index.model().data(index.model().index(index.row(), 0))
        # riw is type row combobox with connection types
        if strtline == WHHDB_PORT_TYPE_TAG:
            for type in self.__ConnTyp:
                editor.addItem(type,type)
                editor.currentIndexChanged.connect(save)
        elif strtline == WHHDB_PORT_BBONE_TAG:
            editor.addItem('Yes', 'Yes')
            editor.addItem('No', 'No')
            editor.currentIndexChanged.connect(save)
        elif strtline == WHHDB_PORT_STATUS_MONITOR_TAG:
            editor.addItem('On', 'On')
            editor.addItem('Off', 'Off')
            editor.currentIndexChanged.connect(save)
        elif strtline == WHHDB_PORT_PTP_ACTIVE_TAG:
            editor.addItem('On', 'On')
            editor.addItem('Off', 'Off')
            editor.currentIndexChanged.connect(save)
        elif strtline == WHHDB_PORT_SPEED_TAG:
            editor.addItem(BANDWIDTH_XPOINT_SPEED_0_0_1,
                           BANDWIDTH_XPOINT_SPEED_0_0_1)
            editor.addItem(BANDWIDTH_XPOINT_SPEED_0_1,
                           BANDWIDTH_XPOINT_SPEED_0_1)
            editor.addItem(BANDWIDTH_XPOINT_SPEED_1,
                           BANDWIDTH_XPOINT_SPEED_1)
            editor.addItem(BANDWIDTH_XPOINT_SPEED_10,
                           BANDWIDTH_XPOINT_SPEED_10)
            editor.currentIndexChanged.connect(save)
        elif strtline == WHHDB_PORT_AFDX_INTERFACE_ID_TAG:
            editor.addItem('', '')
            editor.addItem('1', '1')
            editor.addItem('2',
                           '2')
        elif strtline == WHHDB_PORT_ARINC825_SPEED_TAG:
            editor.addItem('', '')
            editor.addItem(BANDWIDTH_XPOINT_ARINC825_SPEED_83_33,
                           BANDWIDTH_XPOINT_ARINC825_SPEED_83_33)
            editor.addItem(BANDWIDTH_XPOINT_ARINC825_SPEED_125,
                           BANDWIDTH_XPOINT_ARINC825_SPEED_125)
            editor.addItem(BANDWIDTH_XPOINT_ARINC825_SPEED_250,
                           BANDWIDTH_XPOINT_ARINC825_SPEED_250)
            editor.addItem(BANDWIDTH_XPOINT_ARINC825_SPEED_500,
                           BANDWIDTH_XPOINT_ARINC825_SPEED_500)
            editor.addItem(BANDWIDTH_XPOINT_ARINC825_SPEED_1000,
                           BANDWIDTH_XPOINT_ARINC825_SPEED_1000)
            editor.currentIndexChanged.connect(save)
        elif strtline == WHHDB_PORT_ARINC825_USER_SPEED_TAG:
            editor = QLineEdit(parent)
            # ReGex Check: values between [80 : 1000]
            reg_ex = QRegExp("^([89][0-9]|[1-9][0-9][0-9]|1000)")
            editor_validator = QRegExpValidator(reg_ex, editor)
            editor.setValidator(editor_validator)
        elif strtline == WHHDB_PORT_ARINC825_ARBITRATION_SPEED_TAG:
            editor = QLineEdit(parent)
            #  ReGex Check: values between [80 : 1000]
            reg_ex = QRegExp("^([89][0-9]|[1-9][0-9][0-9]|1000)")
            editor_validator = QRegExpValidator(reg_ex, editor)
            editor.setValidator(editor_validator)
        elif strtline == WHHDB_PORT_ARINC825_DATA_SPEED_TAG:
            editor = QLineEdit(parent)
            #  ReGex Check: values between [80 : 8000]
            reg_ex = QRegExp("^([89][0-9]|[1-9][0-9][0-9]|[1-7][0-9][0-9][0-9]|8000)")
            editor_validator = QRegExpValidator(reg_ex, editor)
            editor.setValidator(editor_validator)
        elif strtline == WHHDB_PORT_ARINC825_CYCLIC_TAG:
            editor.addItem('Yes', 'Yes')
            editor.addItem('No', 'No')
        elif strtline == WHHDB_PORT_ARINC825_LISTEN_MODE_TAG:
            editor.addItem('Yes', 'Yes')
            editor.addItem('No', 'No')
        elif strtline == WHHDB_PORT_ARINC825_TYPE_TAG:
            editor.addItem('ISO', 'ISO')
            editor.addItem('ISO FD', 'ISO FD')
        elif strtline == WHHDB_PORT_NUMB_TAG:
            editor = QLineEdit(parent)
            #  ReGex Check: values between [80 : 8000]
            reg_ex = QRegExp("^[0-9]+$")
            editor_validator = QRegExpValidator(reg_ex, editor)
            editor.setValidator(editor_validator)
        elif strtline == WHHDB_PORT_ARINC429_MODE_TAG:
            editor.addItem('Periodic', 'Periodic')
            editor.addItem('Transparent', 'Transparent')
            editor.addItem('Max message', 'Max message')
            editor.addItem('Max data payload', 'Max data payload')
            editor.addItem('Gap', 'Gap')
            editor.addItem('A429 label', 'A429 label')
        elif strtline == WHHDB_PORT_ARINC429_IS50K_TAG:
            editor.addItem('Yes', 'Yes')
            editor.addItem('No', 'No')
        elif strtline == WHHDB_PORT_ARINC429_TX_RATE_TAG:
            editor.addItem('highSpeed', 'highSpeed')
            editor.addItem('lowSpeed', 'lowSpeed')
        elif strtline == WHHDB_PORT_ARINC429_RX_RATE_TAG:
            editor.addItem('highSpeed', 'highSpeed')
            editor.addItem('lowSpeed', 'lowSpeed')
        elif strtline == WHHDB_PORT_ARINC429_TX_USAGE_TAG:
            editor.addItem('data', 'data')
            editor.addItem('parity', 'parity')
        elif strtline == WHHDB_PORT_ARINC429_RX_USAGE_TAG:
            editor.addItem('data', 'data')
            editor.addItem('parity', 'parity')
        elif strtline == WHHDB_PORT_ARINC429_TX_PARITY_TAG:
            editor.addItem('odd', 'odd')
            editor.addItem('even', 'even')
        elif strtline == WHHDB_PORT_ARINC429_RX_PARITY_TAG:
            editor.addItem('odd', 'odd')
            editor.addItem('even', 'even')
        elif strtline == WHHDB_PORT_ARINC429_SDI_COMPARE_TAG:
            editor.addItem('Yes', 'Yes')
            editor.addItem('No', 'No')
        else:
            editor = QLineEdit(parent)
            editor.editingFinished.connect(save)

        return editor

    def setEditorData(self, editor, index):
        """
        :type editor: QComboBox | QLineEdit
        :type index: QModelIndex
        """
        data = index.model().data(index, Qt.EditRole)
        if isinstance(editor, QComboBox):
            # for index in range(editor.count()):
            if editor.itemData(index.row(), Qt.UserRole) == data:
                editor.setCurrentIndex(index.row())
            else:
                editor.setCurrentIndex(0)
        elif isinstance(editor, QLineEdit):
            editor.setText(str(data))

    def setModelData(self, editor, model, index):
        """
        :type editor: QComboBox | QLineEdit
        :type model: QAbstractItemModel
        :type index: QModelIndex
        """
        strtline = index.model().data(index.model().index(index.row(), 0))
        condition = strtline in (WHHDB_PORT_NUMB_TAG,)
        if condition:
            if re.match("^[0-9]+$", editor.text()) is None:
                return
        if isinstance(editor, QComboBox):
            model.setData(index, wgui_qcb_get_data(editor), Qt.EditRole)
        elif isinstance(editor, QLineEdit):
            model.setData(index, editor.text(), Qt.EditRole)
