# ----------------------------------------------------------------------------
# - whgui-kazymir-connection-points-props-widget                             -
# - wild cruncher - cockpit : connection point properties dialog widgets     -
# ----------------------------------------------------------------------------
# - author    (c) 2018-2018 achraf mahti                                     -
# - copyright (c) 2018-2018 silkan /                                         -
# ----------------------------------------------------------------------------

"""
.. module::       whgui_cetrac_kazymir_connection_points_props_widget
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher connection point properties dialog widgets

.. moduleauthor:: achraf mahti <achraf@cetrac.io>
"""
from collections import OrderedDict
from PyQt5.QtCore import Qt, QSortFilterProxyModel
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QLabel, QTableView, \
	QHeaderView, QLineEdit, QComboBox, QAbstractItemView
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_NAME, \
    WHDB_XPOINT_TYPE
from wcr_core.modules.hardware.db.whhdb_connection_point_base_meta import *
from wcr_core.modules.hardware.db.whhdb_connection_point_base_meta import \
    WHHDB_TYPE_KEY
from wcr_core.modules.hardware.db.whhdb_connection_point_base import \
    WHHDB_XPOINT_CONFIG_KEY
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_XPOINT_NAMES, WHHDB_XPOINT_AFDX_CODE, WHHDB_MODULES_IO_NAMES
from wcr_frontend.gui.wgui_props_dialog_widget import \
    WguiXPointPropsTextField, \
    WGUI_NAME_PLACEHOLDER, WGUI_XPNT_CODE, \
    WguiXPointPropsFieldInstance, \
    WguiXPointPropsField
from wcr_frontend.modules.hardware.gui.whgui_Operator_connection_points_props_widget import \
    _WHGUI_XPOINT_INFO, WguiXPointPropsComboField, _WHGUI_WGUI_CONF_OPTIONS, _WHGUI_COL_CONF, \
    _WhguiXPointConfigDataAdapter, WHHDB_DISCRET_KEYS, WHHDB_FIBER_KEYS, WHHDB_AFDX_KEYS, \
    WHHDB_XPOINT_AFDX_KEY, _WhguiXPointConfigTableModel, WHHDB_1553_KEYS
from wcr_frontend.modules.hardware.gui.whgui_Cetrac_switch_connection_points_props_widget import \
    WhguiCetracSwitchXPointProperties, WHDB_STATUS_MASK
from wcr_frontend.modules.hardware.gui.whgui_connection_parameter_edition_delegate \
    import WhguiConnectionParameterBindDelegate

_WHGUI_XPNT_CONFIG = '__config'
_WHGUI_CETRAC_BASE_HIDDEN_OPTION = [WHHDB_PORT_CARD_KEY, WHHDB_TYPE_KEY,
                                    WHHDB_PORT_STATUS_MONITOR_KEY,
                                    WHHDB_PORT_ACTIVE_KEY,
                                    WHHDB_PORT_STATUS_MASK_KEY]
_WHGUI_CETRAC_DEFAULT_HIDDEN_OPTIONS = _WHGUI_CETRAC_BASE_HIDDEN_OPTION + WHHDB_FIBER_KEYS + \
                                       WHHDB_DISCRET_KEYS + WHHDB_AFDX_KEYS + WHHDB_1553_KEYS

# noinspection PyAbstractClass
class WhguiCetracKazymirXPointProperties(WhguiCetracSwitchXPointProperties):
    def __init__(self, instance, pctx, parent=None, ConverifF=None):
        """
        :param instance: The instance object
        :type instance: WhhdbHostInstance
        :param pctx: Project context
        :type pctx: WhmodHardwareProjectContextP
        :param parent: Parent widget
        """
        """:type: WpmodSystemApplicationModuleP"""
        super(WhguiCetracKazymirXPointProperties, self).\
            __init__(instance, pctx, parent=parent, ConverifF=ConverifF)
        self.__xpntCfgMetas = self.getCfgMeta()
        self._armPort = QComboBox()
        self._armPort.addItem('No', 'No')
        self._armPort.addItem('Yes', 'Yes')
        if self.__class__ is WhguiCetracKazymirXPointProperties:
            self._finishInitialization()
            self.table = self.ui.xpntTableF
            self._loadFromData()

    def _loadFromData(self):
        armValue = self.instance.getArmPort()
        if armValue is None:
            armValue = 'No'
        armPortIdx = self._armPort.findText(armValue)
        self._armPort.setCurrentIndex(armPortIdx)
        super()._loadFromData()

    def getButtons(self):
        label, self._configStatus, spacer, addBtn, delBtn = super().getButtons()
        labelArm = QLabel('Arm: ')
        return [label, self._configStatus, labelArm, self._armPort, spacer, addBtn, delBtn]

    def createFields(self):
        return {
            WHDB_XPOINT_NAME: WguiXPointPropsTextField(
                WHDB_XPOINT_NAME, 'Conn. Point', WGUI_NAME_PLACEHOLDER),
            _WHGUI_XPNT_CONFIG: WguiXPointPropsField(
                _WHGUI_XPNT_CONFIG, 'Information',
                WhguiXPointConfigFieldInstance, self.__xpntCfgMetas),
            _WHGUI_XPOINT_INFO: WguiXPointPropsComboField(
                _WHGUI_XPOINT_INFO, 'Config', _WHGUI_WGUI_CONF_OPTIONS),
            WHDB_STATUS_MASK: WguiXPointPropsTextField(
            WHDB_STATUS_MASK, 'status mask')
        }

    def resetModel(self, model, instance):
        model.setXPoints(
            {WGUI_XPNT_CODE: x,
             WHDB_XPOINT_NAME: x.getName(),
             WHDB_XPOINT_TYPE: x.getEntityType(),
             _WHGUI_XPNT_CONFIG:
                 OrderedDict(x.rawElement.getMeta(WHHDB_XPOINT_CONFIG_KEY))}
            for x in instance.getNonCardConnectionPoints() if x.getPortNum() != 3)
        self._loadFromData()

    def save(self):
        armValue = self._armPort.currentText()
        self.instance.setArmPort(armValue)
        armPort = self.instance.hasArmPort()
        if armValue == 'Yes':
            if armPort is None:
                name = 'PortArm'
                point = self.instance.createConnectionPoint('CetraC', name=name)
                point.setPortNum('3')
                point.setCard('ArmCard')
        else:
            if armPort:
                self.instance.removeConnectionPoint(armPort)
        super().save()

    def saveData(self):
        model = self.model
        xpnts = set(model.getValuesForCode(WGUI_XPNT_CODE))
        mark = []
        for xpnt in list(self.instance.getNonCardConnectionPoints()):
            if xpnt not in xpnts:
                if self.instance.hasArmPort():
                    continue
                mark.append(xpnt)
                # self.removeXPoint(xpnt)
        xpntAdded = False
        for idx in model.getIndices():
            xpnt = model.getValue(idx, WGUI_XPNT_CODE)
            if not xpnt:
                xpnt = self.createXPoint(idx, model, self.instance)
                xpntAdded = True
            self.updateXPoint(idx, model, xpnt)
        for xpnt in mark:
            self.removeXPoint(xpnt)
        if xpntAdded:
            self.instance.resetConnectionPointPositions()


class _WhguiXPointKazymirConfigTableModel(_WhguiXPointConfigTableModel):
    def __init__(self, model, context, cfgAdapter, parent=None):
        """
        :param model: Connection points data model
        :type model: WguiXPointPropsModelP
        :param cfgAdapter: Configuration data adapter
        :type cfgAdapter: _WhguiXPointConfigDataAdapter
        :param parent: Parent item
        """
        super().__init__(model, context, cfgAdapter, parent)

    def _hideOptions(self, index):
        """
        function that hide the option that is not necessary and update the data
        :param index:
        :return:
        """
        connType = None
        conf = self.model.getValuesForIndex(index)[_WHGUI_XPNT_CONFIG]
        xpnt = self.model.getValuesForIndex(index)[WGUI_XPNT_CODE]
        confValues = None
        if self.context.table:
            # update data on selecting row
            comboConf = self.context.table.cellWidget(index - 1, _WHGUI_COL_CONF)
            if not comboConf:
                self.endResetModel()
                return
            connType = comboConf.currentText()
        else:
            metaData = self.model.getValue(index, _WHGUI_XPNT_CONFIG)
            connType = metaData['type']
        # update data on opening the window
        if connType == WHHDB_XPOINT_AFDX_CODE:
            confValues = xpnt._getConfigAfdx()

        if confValues:
            conf.update(confValues)

        hidOptions = self.context.updateHiddenOption(connType)
        # hide options depending on port type
        for hiddenOpt in hidOptions:
            if hiddenOpt in self.cfgKeys:
                self.cfgKeys.remove(hiddenOpt)

    def getFlags(self, flgs, index):
        if index.column() == 1:
            # Fix the port Number only
            if index.row() == 0:
                return flgs & Qt.ItemIsEditable
            else:
                return flgs | Qt.ItemIsEditable
        else:
            return flgs & ~Qt.ItemIsEditable


class WhguiXPointConfigFieldInstance(WguiXPointPropsFieldInstance):
    def __init__(self, code, label, context, xpntCfgMetas):
        """
        :param code: Value code for the field
        :type code: str
        :param label: Label for the field
        :type label: str
        :param context: Field installation context
        :type context: WguiXPointPropsFieldCtx
        """
        super().__init__(code, label)

        self.__context = context

        #: Configuration data adapter
        self.__cfgAdapter = _WhguiXPointConfigDataAdapter(
            context.model, xpntCfgMetas)

        #: Configuration

        #: Configuration table view
        self.__configTblF = QTableView()
        #: Configuration table model
        self.__configTblM = _WhguiXPointKazymirConfigTableModel(
            context.model, context, self.__cfgAdapter)
        self.__configTblDelegate = \
            WhguiConnectionParameterBindDelegate(sorted(WHHDB_XPOINT_NAMES))

        #: Configuration table filter proxy model
        self.__ctblFilter = QSortFilterProxyModel(self.__configTblM)
        self.__ctblFilter.setFilterCaseSensitivity(Qt.CaseInsensitive)
        #: Configuration table filter input
        self.__configFilterF = QLineEdit()

        self.__auxWidget = None

    def _createAuxWidget(self):
        self.__ctblFilter.setSourceModel(self.__configTblM)
        self.__configTblF.setModel(self.__ctblFilter)
        self.__configTblF.setSelectionMode(QAbstractItemView.NoSelection)
        self.__configTblF.horizontalHeader().setSectionResizeMode(
            0, QHeaderView.ResizeToContents)
        self.__configTblF.horizontalHeader().setSectionResizeMode(
            1, QHeaderView.Stretch)
        self.__configTblF.setItemDelegate(self.__configTblDelegate)
        self.__configFilterF.setPlaceholderText('Filter configuration keys')

        self.__context.activeXPointChanged.connect(
            self.__configTblM.activeXPointChanged)
        self.__configFilterF.textChanged.connect(
            self.__ctblFilter.setFilterFixedString)

        wdgt = QWidget()
        layout = QVBoxLayout()
        layout.addWidget(
            QLabel('Configuration values for the selected connection point:'))
        layout.addWidget(self.__configTblF)
        layout.addWidget(self.__configFilterF)
        wdgt.setLayout(layout)
        return wdgt

    @property
    def configTblM(self):
        return self.__configTblM

    def installWidget(self, index, widgetF, textF):
        # noinspection PyShadowingNames
        def formatConfig():
            cfg = self.__context.model.getValue(index, _WHGUI_XPNT_CONFIG)
            """:type: dict[str, str]"""
            if not cfg:
                return ''
            else:
                # set default Values
                if not cfg[WHHDB_PORT_BBONE_KEY]:
                    cfg[WHHDB_PORT_BBONE_KEY] = 'No'
                if not cfg[WHHDB_PORT_SPEED_KEY]:
                    cfg[WHHDB_PORT_SPEED_KEY] = '1G'
                if not cfg[WHHDB_PORT_NUM_KEY]:
                    cfg[WHHDB_PORT_NUM_KEY] = '1'
                if not cfg[WHHDB_PORT_PTP_ACTIVE_KEY]:
                    cfg[WHHDB_PORT_PTP_ACTIVE_KEY] = 'Off'
            if cfg['type'] in WHHDB_MODULES_IO_NAMES:
                return ''
            return '; '.join(
                '{}: {}'.format(self.__cfgAdapter.getLabel(index, k) or k,
                                cfg.get(k))
                for k in self.__cfgAdapter.getKeys(index) if cfg.get(k))

        def configChanged(idx, code):
            try:
                if idx == index and code == self.code:
                    textUpdateF(formatConfig())
            except:
                pass

        textUpdateF = textF(formatConfig())
        self.__context.model.valueChanged.connect(configChanged)

    def getAuxWidget(self):
        if not self.__auxWidget:
            self.__auxWidget = self._createAuxWidget()
        return self.__auxWidget
