# ----------------------------------------------------------------------------
# - wigui-creation-message-props-dialog                                       -
# - wild cruncher - cockpit : graphic scene widget                           -
# ----------------------------------------------------------------------------
# - author    (c) 2018-2018 achraf mahti                                     -
# - copyright (c) 2018-2018 silkan                                           -
# ----------------------------------------------------------------------------

"""
.. module::       wigui_creation_type_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Dezygner wigui graphics scene editor

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""

import re
from PyQt5.QtCore import Qt, QAbstractTableModel
from PyQt5.QtWidgets import QComboBox, QHBoxLayout, QWidget, QVBoxLayout, \
	QPushButton, QLineEdit, QTableView, QItemDelegate
from wcr_frontend import isNameValid, showErrorWindow, NameFormatMessageError
from wcr_core.db.whdb_utils import whdb_getNextName
from wcr_core.modules.icd.db.wihdb_module_instance import WIHDB_MODULE_STORAGE_VALUES
from wcr_frontend.gui.wgui_utils import wgui_qcb_get_data


# noinspection PyAbstractClass
class _WiguiModuleMessageInstanciate(QWidget):
    GLB_COL = 0
    LOC_COL = 1

    def __init__(self, instance, pctx, scene, parent=None, ConverifF=None):
        """
        :param instance: The instance object
        :type instance: WhhdbHostInstance
        :param pctx: Project context
        :type pctx: WhmodHardwareProjectContextP
        :param parent: Parent widget
        """
        super().__init__(parent)
        self.instance = instance
        self.__module = pctx.getModule()
        """:type: WpmodSystemApplicationModuleP"""
        self.__xpntTypeNames = ()
        self.msg2variable = {}
        self.__messages = []
        self.__topBlock = pctx.getProjectDB().getTopHierarchy()
        # init table necessary for
        self.__msgTblW = QTableView(self)
        self._initModels()
        self._initElements()
        self.createLayout()


    @property
    def msgTblM(self):
        return self.__msgTblM

    @property
    def msgTblW(self):
        return self.__msgTblW

    @property
    def msgDlgt(self):
        return self.__msgDlgt

    def _initModels(self):
        self.__msgTblM = _WhguiMessageTableInstanceModel(self.formatMsgs,
                                                         parent=self)
        self.__msgDlgt = _WhguiMessageBindDelegate(top=self.__topBlock,
                                                   parent=self)

    def _initElements(self):
        #: Add entry button
        self.__addButton = QPushButton('Add')
        #: Delete entry button
        self.__deleteButton = QPushButton('Delete')
        self.__msgTblW.setModel(self.msgTblM)
        self.__msgTblW.horizontalHeader().setSectionResizeMode(1)
        self.__msgTblW.setMaximumWidth(500)
        self.__msgTblW.setItemDelegate(self.__msgDlgt)

        self.__addButton.clicked.connect(self._addEntry)
        self.__deleteButton.clicked.connect(self._deleteSelEntries)

    def resettab(self):
        """
        reset self.__msgTblM
        """
        self.msgTblM.deleteRows(range(0, self.msgTblM.rowCount()))
        self._loadFromData()

    def getMsgs(self):
        """
        return the description of all card describes in the Table
        """
        datas = {}
        # add default type as None
        for moduleMsg, msgType, consprod, storage, period in self.msgTblM.getFullMessages():
            datas[moduleMsg] = (msgType, consprod, storage, period)

        return datas

    def formatMsgs(self, data, column):
        """
        :param data: data to format
        :type data: Cardname | Cardtype | [PortUUID]
        """
        if data is None:
            return ''
        else:
            return data

    def createLayout(self):
        layout = QVBoxLayout()
        layout.addWidget(self.__msgTblW)

        btnLayout = QHBoxLayout()
        btnLayout.addWidget(self.__addButton)
        btnLayout.addWidget(self.__deleteButton)
        btnLayout.addStretch()
        layout.addLayout(btnLayout)

        self.setLayout(layout)

        return layout

    def _addEntry(self):
        """
        Add entry in the model
        """
        self.msgTblM.addRow()

    def _deleteSelEntries(self):
        """
        Delete selected entries from the model
        """
        selIdx = self.msgTblW.selectedIndexes()
        """:type: list[QModelIndex]"""
        self.msgTblM.deleteRows([idx.row() for idx in selIdx])
        rl = []
        for idx in selIdx:
            rl.append(idx.row())
        rowlist = sorted(set(rl))
        if self.msgTblM.rowCount() > 0 and rowlist:
            if rowlist[0] >= self.msgTblM.rowCount():
                self.msgTblW.setCurrentIndex(self.msgTblM.
                                                index(self.msgTblM.rowCount() - 1, 0))
            else:
                self.msgTblW.setCurrentIndex(self.msgTblM.
                                                index(rowlist[0], 0))

    def removeMsg(self, glbMsg):
        """
        remove message and variables associated to it
        """
        if glbMsg in self.msg2variable:
            del self.msg2variable[glbMsg]

    def selectedMsg(self, selectignor=False, role=Qt.UserRole):
        lrow = []
        for index in self.getSelectedIndexes():
            lrow.append(index.row())
        ulrow = set(lrow)
        if len(ulrow) == 1 and not selectignor:
            return self.msgTblM.data(self.msgTblM.index(lrow[0], 0),
                                      role=role)
        else:
            return None

    def getSelectedIndexes(self):
        return self.msgTblW.selectionModel().selectedIndexes()

    def save(self):
        self.instance.operator.setEntries(self.getMsgs())


class _WhguiMessageTableInstanceModel(QAbstractTableModel):
    """
    icd Binding table model.
    """

    TITLE_NAME = 'Name'
    TITLE_TYPE = 'Type'
    TITLE_CONSPROD = 'Con/Prod'
    TITLE_STORAGE = 'Storage'
    TITLE_PERIOD = 'Period (ms)'
    HEADERS = [TITLE_NAME, TITLE_TYPE, TITLE_CONSPROD, TITLE_STORAGE, TITLE_PERIOD]

    def __init__(self, formatF, parent=None):
        """
        :param XpntTypeNames: The Names of Xpoint of the mule
        :param parent: Parent widget
        """
        super().__init__(parent)
        self.__parent = parent
        self.__formatF = formatF

        #: Entries as list of tuples of
        #  port, message and type
        self.__entries = []
        self._update()

    @property
    def entries(self):
        return self.__entries

    def _update(self):
        self.beginResetModel()
        self.endResetModel()

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
                                    0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__entries)

    def columnCount(self, parent=None, *args, **kwargs):
        return len(self.HEADERS)

    def data(self, index, role=None):
        row = index.row()
        column = index.column()
        if 0 <= column <= self.columnCount():
            info = self.__entries[row][column]
            if role in [Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole]:
                return self.__formatF(info, column)
        return None

    def setData(self, index, data, role=None):
        assert role == Qt.UserRole
        self._setMessageEntry(index, data)

    def _setMessageEntry(self, index, data):
        msg, type_, consprod, storage, period = self.__entries[index.row()]
        if index.column() == 0:
            if not self.checkUnicityMsg(data, index.row()):
                return
            actualentry = (data, type_, consprod, storage, period)
        elif index.column() == 1:
            actualentry = (msg, data, consprod, storage, period)
        elif index.column() == 2:
            actualentry = (msg, type_, data, storage, period)
        elif index.column() == 3:
            actualentry = (msg, type_, consprod, data, period)
        elif index.column() == 4:
            actualentry = (msg, type_, consprod, storage, data)
        self.__entries[index.row()] = actualentry

    def checkUnicityMsg(self, msg, row):
        for i, (entry) in enumerate(self.__entries):
            if i != row:
                # check uniqueness on msg name only
                if msg == entry[0]:
                    return False
        return True

    def getFullMessages(self):
        return self.__entries

    def addRow(self, moduleMsg='defMsg', msgType='int', consprod='C', storage='event', period=''):
        """
        Add a row at the end of the list (append)

        :param moduleMsg: The message name local
        :type moduleMsg: str | None
        :param msgType: The message name local
        :type msgType: str | None
        :param consprod: The message state (produced or consumed)
        :type consprod: str | None
        :param storage: The message storage methode (event, cyclic, polling)
        :type storage: str | None
        :param period: The period used for cycle storage method
        :type period: str | None
        """
        index = self.index(self.rowCount(), 0)
        self.beginInsertRows(index, self.rowCount(), self.rowCount())
        # help uniqueness by auto increment name
        allMsg = [msg for msg, _, _, _, _ in self.getFullMessages()]
        nextMsg = whdb_getNextName(moduleMsg, allMsg)
        self.__entries.append((nextMsg, msgType, consprod, storage, period))

        self.endInsertRows()
        self._update()

    def deleteRows(self, rowIndices):
        """
        :param rowIndices: Iterable of indices
        :type rowIndices: Iterable[int]
        """
        # Make sure no duplicate exist
        rowList = list(set(rowIndices))
        rowList.sort(reverse=True)

        for row in rowList:
            self.deleteRow(row)

    def deleteRow(self, rowIndex):
        """
        Delete a row entry

        :param rowIndex: Row to the delete
        :type rowIndex: int
        """
        if not (0 <= rowIndex < self.rowCount()):
            raise ValueError('Index of row is out of bounds')

        index = self.index(rowIndex, 0)
        self.beginRemoveRows(index, rowIndex, rowIndex)

        name, msgType, consprod, storage, period = self.__entries[rowIndex]
        if name is not None:
            try:
                # Unbound function of message
                self.__parent.removeMsg(name)
                pass
            except ValueError:
                pass
        self.__entries.pop(rowIndex)
        self.endRemoveRows()
        self._update()

    def index(self, row, column, parentIndex=None, *args, **kwargs):
        return self.createIndex(row, column)

    def checkFlagCyclicPeriod(self, index, flgs):
        row = index.row()
        storage = index.sibling(row, 3).data()
        if storage == 'cyclic':
            flgs |= Qt.ItemIsEditable
        else:
            flgs &= ~Qt.ItemIsEditable
        return flgs

    def flags(self, index):
        flgs = super().flags(index)
        if index.column() == 4:
            flgs = self.checkFlagCyclicPeriod(index, flgs)
        else:
            flgs |= Qt.ItemIsEditable
        return flgs


class _WhguiMessageBindDelegate(QItemDelegate):
    """
    Delegate for message items
    """

    def __init__(self, top, parent=None):
        """
        :type CardTyp: () -> list[CardTyp]
        :type formatF: (elem,column) -> str
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(parent)
        self.__topBlock = top

    def createEditor(self, parent, option, index):
        """
        Create a editors for message table the selected item

        :type parent: QWidget
        :type option: QStyleOptionViewItem
        :type index: QModelIndex
        """

        def save(*_):
            self.setModelData(editor, index.model(), index)

        col = index.column()
        # id col = 0 is str for cardname
        if col == 0:
            editor = QLineEdit(parent)
        # message type
        elif col == 1:
            editor = QComboBox(parent)
            for type_, _ in self.__topBlock.getGlobalTypes():
                editor.addItem(type_, type_)
        # message consumed or produced
        elif col == 2:
            editor = QComboBox(parent)
            for consprod in ('C', 'P'):
                editor.addItem(consprod, consprod)
        # message storage method
        elif col == 3:
            editor = QComboBox(parent)
            for consprod in WIHDB_MODULE_STORAGE_VALUES:
                editor.addItem(consprod, consprod)
        # id col = 0 is str for cardname
        elif col == 4:
            editor = QLineEdit(parent)
        # editor.editingFinished.connect(save)
        return editor

    def setEditorData(self, editor, index):
        """
        :type editor: QComboBox | QLineEdit
        :type index: QModelIndex
        """
        data = index.model().data(index, Qt.EditRole)
        if isinstance(editor, QComboBox):
            for index in range(editor.count()):
                if editor.itemData(index, Qt.UserRole) == data:
                    editor.setCurrentIndex(index)
                    break
            else:
                editor.setCurrentIndex(0)
        elif isinstance(editor, QLineEdit):
            editor.setText(data)

    def setModelData(self, editor, model, index):
        """
        :type editor: QComboBox | QLineEdit
        :type model: QAbstractItemModel
        :type index: QModelIndex
        """
        if isinstance(editor, QComboBox):
            model.setData(index, wgui_qcb_get_data(editor), Qt.UserRole)
        elif isinstance(editor, QLineEdit):
            if editor.text():
                if index.column() == 0:
                    if isNameValid(editor.text()):
                        model.setData(index, editor.text(), Qt.UserRole)
                    else:
                        showErrorWindow("Name Error", NameFormatMessageError,
                                        editor.text())
                elif index.column() == 4:
                    if re.match("^([0-9]*|\d*\.\d{1}?\d*)$", editor.text()):
                        model.setData(index, editor.text(), Qt.UserRole)
                    else:
                        showErrorWindow("Period",
                                        "Value '{}' is incorrect,"
                                        " needs to be a decimal value.",
                                        editor.text())
