# ----------------------------------------------------------------------------
# - wigui-scene-item-factory                                                 -
# - wild cruncher: scene item factory                     icd cockpit system -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wigui_scene_item_factory
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher model scene item factory

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from PyQt5.QtGui import QColor
from wcr_core.db_api.whdb_instance_api import WhdbBlockInstanceP
from wcr_core.modules.icd.db.wihdb_module_instance import wihdb_get_realdirection
from wcr_frontend.gui.wgui_scene_item import WguiXPointRectPositionProvider
from wcr_frontend.gui.wgui_scene_item_factory import WguiDefaultSceneItemFactoy


class WiguiSceneItemFactory(WguiDefaultSceneItemFactoy):
    """
    ICD scene item factory.
    """

    def __init__(self, appModule):
        """
        :param appModule: Application module
        :type appModule: WmmodModelApplicationModuleP
        """
        super().__init__(WguiXPointRectPositionProvider())

        self.__instTooltipF = wigui_instance_tooltip_func(appModule)

    def createBlockInstanceSceneItem(self, obj, *args, **kwargs):
        item = super().createBlockInstanceSceneItem(obj, *args, **kwargs)
        item.setTooltipF(self.__instTooltipF)
        return item

    def createConnectionPoint(self, conn, item, *args, **kwargs):
        conpoint = super().createConnectionPoint(conn, item, *args, **kwargs)
        conpoint.setbackgroundcolor(QColor(180, 220, 50))
        conn = conpoint.getDataObject()
        conpoint.setToolTip("{}".format(conn.getName()))
        return conpoint

    def createConnectionSceneItem(self, obj, *args, **kwargs):
        connection = super().createConnectionSceneItem(obj,*args,**kwargs)
        conn = connection.getDataObject()
        connection.setToolTip("{}".format(conn.getName()))
        return connection


def wigui_instance_tooltip_func(appModule):
    """
    :param appModule: Application module
    :type appModule: WmmodModelApplicationModuleP
    :rtype: (WhdbInstanceP) -> str
    """
    def tooltip(instance):
        ttip = '<b>{}</b>'.format(instance.getName())

        if isinstance(instance, WhdbBlockInstanceP):
            block = instance.getBlock()
            btyp = appModule.getBlockType(block.getEntityType())
            if btyp:
                bnam = btyp.getName()
                if bnam:
                    ttip += '<br/><i>{}</i>'.format(bnam.title())

        if instance.getInfo():
            ttip += '<br/>' + instance.getInfo()

        return ttip

    return tooltip
