# ----------------------------------------------------------------------------
# - wegui-wgtbxmessage-instance-props-dialog                                 -
# - wild cruncher - cockpit :                                                -
# -         Wight box message properties dialog widgets                      -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wegui_wgtbxmessage_instance_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher instance properties dialog

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from PyQt5.QtWidgets import QVBoxLayout, QLineEdit, QLabel

from wcr_frontend.gui.wgui_props_dialog_widget import WguiPropertiesWidgetP
from wcr_frontend.gui.wgui_operator_instance_props_dialog import \
    WguiOperatorInstancePropertiesDialog

class WmeguiWgtBxMessageInstancePropertiesDialog(
    WguiOperatorInstancePropertiesDialog):
    _TYPE_NAME = 'Wight Box Message Instance'

    _SHOW_OPERATOR_WIDGET = False

    def __init__(self, inst, pctx, updateF, selectXpnt=None, parent=None):
        """
        :param inst: the instance object
        :type inst: WhdbInstanceP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: () -> _
        :param parent: Parent widget
        """
        super().__init__(inst, pctx, updateF, selectXpnt, parent)


    def _createAdditionalInstancePropertiesWidgets(self, inst, pctx):
        return [_WmeguiWightBoxMessageProperties(inst)]

    # noinspection PyMethodMayBeStatic
    def _createAdditionalPropertiesWidgets(self, inst, pctx):

        return []

    def _createConnectionPointsPropertiesWidget(self, inst, pctx):
        return None


class _WmeguiWightBoxMessageProperties(WguiPropertiesWidgetP):
    """
    Property widget for host capabilities.
    """

    def __init__(self, bbm, parent=None):
        """
        :param bbm: Black Box Message instance
        :type bbm: WmehdbWightBoxMessageInstance
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: Host instance
        self.__wgtbxmessage = bbm

        #: Size field
        self.__sizeF = QLineEdit()

        self.setLayout(self._createLayout())
        self.updateFields()

    def _createLayout(self):
        """
        Create the layout.

        :return: Layout
        :rtype: QLayout
        """
        layout = QVBoxLayout()
        layout.addWidget(QLabel('Size:'))
        layout.addWidget(self.__sizeF)
        return layout

    def updateFields(self):
        wbm = self.__wgtbxmessage

        self.__sizeF.setText(str(wbm.getSize()))

    def save(self):
        wbm = self.__wgtbxmessage

        wbm.setSize(int(self.__sizeF.text()))
        self.updateFields()