# ----------------------------------------------------------------------------
# - wfgui-db-tree                                                            -
# - wild cruncher: database browser tree       frontend / statemachine / gui -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfgui_db_tree
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Database browser tree

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.modules.statemachine.db.wfhdb_transition import WfhdbTransition
from wcr_frontend.gui.wgui_db_tree import WguiDbTreeItemFactory, \
    WguiDbTreeConnectionItem
from wcr_frontend.modules.statemachine.gui.wfgui_utils import \
    wfhdb_get_transition_description


class WfguiDbTreeConnectionItem(WguiDbTreeConnectionItem):
    def __init__(self, transition, parentItem, icon=None, path=None):
        """
        :param transition: Represented database entity
        :type transition: WfhdbTransition
        :param parentItem: Parent tree item
        :type parentItem: WguiTreeItemP
        :param icon: Icon
        :type icon: QIcon | None
        :param path: Entity path
        :type path: list[WhdbBlockInstanceP]
        """
        super().__init__(transition, parentItem, icon, path)

        self.__transition = transition

    def _getName(self):
        return wfhdb_get_transition_description(self.__transition)

    def _getType(self):
        return self.__transition.getConnectionTypeDescription()


class WfguiDbTreeItemFactory(WguiDbTreeItemFactory):
    def __init__(self, iconPathFy):
        """
        :param iconPathFy: Icon path factory
        :type iconPathFy: WguiIconPathFactoryP
        """
        super().__init__(iconPathFy)

    def _createTreeNode(self, itemData, parentItem, **kwargs):
        icon = self._getIcon(itemData)
        path = kwargs.get('path')

        if isinstance(itemData, WfhdbTransition):
            parentItem = parentItem.getChild(self.CONNECTION_IDX)
            item = WfguiDbTreeConnectionItem(itemData, parentItem, icon, path)
            return item, parentItem, None
        else:
            return super()._createTreeNode(itemData, parentItem, **kwargs)
