# ----------------------------------------------------------------------------
# - wcgui-scene-context-menu-factory                                         -
# - wild cruncher - cockpit: scene-based context menu factory                -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_scene_context_menu_factory
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher scene-based context menu factory

.. moduleauthor:: Philippe lambert <lambert@silkan.com>
"""

from functools import partial
from wcr_frontend.gui.wgui_scene_context_menu_factory import \
    WguiSceneContextMenuFactory
from wcr_core.modules.sys_map.db.wchdb_Host_lure_instance import \
    WchdbHostLureInstance
from PyQt5.QtWidgets import QAction


class WcguiSceneContextMenuFactory(WguiSceneContextMenuFactory):
    """
    Context menu generate based on the WCR graphics scene.
    """

    def __init__(self, scene, actions, clpbrd, pctx, pdgf, sfilf):
        """
        :param scene: Graphics scene
        :type scene: WguiSceneAdapterP
        :param actions: Action dictionary
        :type actions: dict[str, WactActionP]
        :param clpbrd: Clipboard handler
        :type clpbrd: WguiClipboardHandler
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param pdgf: Properties dialog factory
        :type pdgf: WguiPropertiesDialogFactoryP
        """
        super().__init__(scene, actions, clpbrd, pctx, pdgf)

        self.__scene = scene
        self.__prjContext = pctx
        self.__setfilterF = sfilf

    def _sethostfilter(self, hostinst):
        if hostinst is not None:
            self.__setfilterF(hostinst.getUUIDsource())
        else:
            self.__setfilterF(None)

    def _createOperatorInstanceContextMenuActions(self, inst, menu, parent):
        moreaction=[]
        if isinstance(inst, WchdbHostLureInstance):
            moreaction.append(
                QAction('Set as Filter', menu,
                        statusTip='Make side bare wiget display only bindings '
                                  'related to the host',
                        triggered=partial(self._sethostfilter, inst)))
        moreaction.append(
            QAction('Reset Filter', menu,
                    statusTip='restore default side bare wiget display',
                    triggered=partial(self._sethostfilter,None))
                          )
        return moreaction

    def _createViewContextMenuActions(self, menu, parent):
        moreaction=[
            QAction('Reset Filter', menu,
                    statusTip='restore default side bare wiget display',
                    triggered=partial(self._sethostfilter, None))]
        return moreaction