# ----------------------------------------------------------------------------
# - wpgui-selection-provider                                                 -
# - wild cruncher: selection providers               frontend / system / gui -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpgui_selection_provider
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Selection providers

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod, ABCMeta

from PyQt5.QtWidgets import QComboBox


class WpguiSelectionProvider(metaclass=ABCMeta):
    """
    Current selection provider.
    """

    @abstractmethod
    def getSelection(self):
        """
        Get the currently selected item.

        :return: Currently selected item text and user data
        :rtype: (str, object)
        """
        raise NotImplementedError()


class WpguiComboboxSelectionProvider(WpguiSelectionProvider):
    def __init__(self, comboBox):
        """
        :param comboBox: Combobox
        :type comboBox: QComboBox
        """
        #: Combobox
        self.__serviceComboBox = comboBox

    def getSelection(self):
        cbx = self.__serviceComboBox
        idx = cbx.currentIndex()
        if idx < 0:
            return None
        text = cbx.itemText(idx)
        if not text:
            return None
        return text, cbx.itemData(idx)
