# ----------------------------------------------------------------------------
# - sys-gui-cstcx                                                            -
# - wild cruncher:system mapper cockpit system context                       -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpgui_cstcx
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher system mapper cockpit system contex

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
# noinspection PyUnresolvedReferences
import wcr_core.modules.system.wpmod_application_module
# noinspection PyUnresolvedReferences
from wcr_frontend.modules.system.gui.wpgui_bus_binding_table import \
    WpguiBusBindingTable
import wcr_frontend.modules.system.gui.wpgui_icons

from wcr_frontend.modules.system.gui.wpgui_controller_value_provider import \
    WpguiControllerValueProvider
from wcr_core.modules.system.actions.wpact_action_base import \
    WPACT_ACTION_W_LSIM_MODIFY
from wcr_core.common.wcmn_protocols import WcmnReleasableP
from wcr_frontend.gui.wgui_scene_view_mouse_handler import \
    WguiDefaultSceneViewMouseHandler
from wcr_frontend.gui.wgui_gui_module import \
    WguiCockpitSystemContextBase, wgui_getClassEntry, \
    WGUI_ACTION_SECTION_PROJECT, WguiCockpitSystemModuleBase
from wcr_core.db_api.whdb_instance_api import WhdbBlockInstanceP
from wcr_frontend.gui.wgui_scene import WguiGraphicsScene
from wcr_frontend.gui.wgui_scene_context_menu_factory import \
    WguiSceneContextMenuFactory
from wcr_frontend.gui.wgui_clipboard import WguiClipboardHandler
from wcr_frontend.gui.wgui_db_tree import wgui_createDbTree, \
    WguiDbTreeItemFactory
from wcr_core.modules.system.db.wphdb_block import WphdbClusterBlock, \
    WPHDB_CLUSTER_BLOCK_CODE
from wcr_core.modules.system.wpmod_application_module import \
    WPMOD_SYSTEM_APP_MODULE, WPMOD_SYSTEM_APP_CODE
from wcr_frontend.modules.system.gui.wpgui_logical_simulation_tree import \
    WpguiLogicalSimulationTree, WpguiLogicalSimulationSelectionAdapter
from wcr_frontend.modules.system.gui.wpgui_props_dialog_factory import \
    WpguiPropertiesDialogFactory
from wcr_frontend.modules.system.gui.wpgui_scene_item_factory import \
    WPGUI_SCENE_ITEM_FACTORY


_WPGUI_ICONS = {
    WPHDB_CLUSTER_BLOCK_CODE: ':/img/wpgui_clstr.png',
    'wcr.system.std.host': ':/img/wpgui_phost.png',
    'wcr.system.std.bus': ':/img/wpgui_phbus.png',
    'editor-tab': ':/img/wpgui_ticon.png',
    'wcr.backend.system.generate': ':/img/app_was_gene.png',
}

_WPGUI_SECTION_ACTIONS = {
    WGUI_ACTION_SECTION_PROJECT: ['project.system.setlogicalsim'],
}

WPGUI_DEFAULT_ICON_CODES = {
    WphdbClusterBlock: WPHDB_CLUSTER_BLOCK_CODE,
    WhdbBlockInstanceP: lambda x: WPGUI_DEFAULT_ICON_CODES.get(
        x.getBlock().__class__)
}


class _WpguiCockpitSystemContext(WguiCockpitSystemContextBase):
    def __init__(self, cmod, pctx, pcfg, actions, actx, parent):
        """
        :param cmod: Cockpit system module
        :type cmod: WguiCockpitSystemModuleP
        :param pctx: Project context
        :type pctx: WpmodSystemProjectContextP
        :param pcfg: Project editor configuration
        :type pcfg: WguiProjectEditorCfgP
        :param actions: Actions
        :type actions: dict[str, WactActionP]
        :param actx: Action execution context
        :type actx: WguiActionExecutionContextP
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(cmod)

        modActions = list(WPGUI_COCKPIT_SYSTEM_MODULE.getActions())
        """:type: list[(WactActionP, str, str, tuple[str])]"""

        self.__ctrlValuePrv = WpguiControllerValueProvider(pctx)
        self.__sceneItemFy = WPGUI_SCENE_ITEM_FACTORY
        self.__scene = WguiGraphicsScene(
            pctx.getProjectDB(), pcfg, actions, pctx, self, actx, parent)
        self.__propDiagF = WpguiPropertiesDialogFactory(
            pctx, pcfg, self.__scene, actions, actx, parent)
        self.__clpbrdHandler = WguiClipboardHandler(
            self.__scene, cmod.getAppModule())
        self.__cntxtMenuF = WguiSceneContextMenuFactory(
            self.__scene, actions, self.__clpbrdHandler, pctx,
            self.__propDiagF)
        self.__mouseHandler = WguiDefaultSceneViewMouseHandler(
            pctx, self.__propDiagF, self.__scene, self.__cntxtMenuF)

        self.__sidebarModules = self._createSidebarModules(
            pctx, self.__scene, self.__cntxtMenuF, modActions, parent)

    def release(self):
        for _, _, smod in self.__sidebarModules:
            if isinstance(smod, WcmnReleasableP):
                smod.release()

        for item in [self.__scene, self.__propDiagF, self.__clpbrdHandler,
                     self.__cntxtMenuF, self.__mouseHandler,
                     self.__sidebarModules]:
            if isinstance(item, WcmnReleasableP):
                item.release()

        self.__scene = None
        self.__propDiagF = None
        self.__clpbrdHandler = None
        self.__cntxtMenuF = None
        self.__mouseHandler = None
        self.__sidebarModules = []
        self.__ctrlValuePrv = None

        super().release()

    def _createSidebarModules(self, pctx, scene, cntxtMenuF, actions, parent):
        modules = []

        dbTree = wgui_createDbTree(
            pctx.getProjectDB().getTopHierarchy(), scene,
            WguiDbTreeItemFactory(self), cntxtMenuF)
        modules.append(('dbtree', 'Database', dbTree))

        lsTreeObsActions = (
            a for a, _, _, fs in actions
            if WPACT_ACTION_W_LSIM_MODIFY in fs
            and WPMOD_SYSTEM_APP_CODE in fs)
        lsimTree = WpguiLogicalSimulationTree(
            pctx, self, lsTreeObsActions)
        slctAdptr = WpguiLogicalSimulationSelectionAdapter(
            scene, lsimTree, pctx.getMappingManager())
        # Ensure adapter does not get garbage-collected too soon
        setattr(self, 'slctAdptr-' + str(id(slctAdptr)), slctAdptr)
        modules.append(('lsimtree', 'Logical Simulation', lsimTree))

        bndg = WpguiBusBindingTable(pctx, scene, parent)
        modules.append(('bindings', 'Log. to Phy. Bus Bindings', bndg))

        return modules

    def getExcludedSystemToolboxItems(self):
        return []

    def getHighlightingModules(self):
        return []

    def getDirectoryDescription(self, level):
        return 'directory'

    def getScene(self):
        return self.__scene

    def getSceneViewMouseHandler(self):
        return self.__mouseHandler

    def getClipboardHandler(self):
        return self.__clpbrdHandler

    def getSceneItemFactory(self):
        return self.__sceneItemFy

    def getContextMenuFactory(self):
        return self.__cntxtMenuF

    def getSidebarModules(self):
        return self.__sidebarModules

    def getPropertiesDialogFactory(self):
        return self.__propDiagF

    def getControllerValueProvider(self):
        return self.__ctrlValuePrv


class _WpguiCockpitSystemModule(WguiCockpitSystemModuleBase):
    _ICON_CODES = dict(WguiCockpitSystemModuleBase._ICON_CODES)
    _ICON_CODES.update(WPGUI_DEFAULT_ICON_CODES)

    def __init__(self):
        super().__init__(WPMOD_SYSTEM_APP_MODULE)

    def createContext(self, pctx, pcfg, actions, actx, parent):
        return _WpguiCockpitSystemContext(
            self, pctx, pcfg, actions, actx, parent)

    def getIconForCode(self, code, iconType):
        return _WPGUI_ICONS.get(code) or \
            super().getIconForCode(code, iconType)

    def _getRawIconCode(self, entity):
        return wgui_getClassEntry(entity, self._ICON_CODES)


#: GUI Cockpit system context singleton
WPGUI_COCKPIT_SYSTEM_MODULE = _WpguiCockpitSystemModule()
