# ----------------------------------------------------------------------------
# - vlink-gui-cstcx                                                          -
# - wild cruncher:virtual link  design cockpit system contex                 -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wvgui_cstcx
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wcs cockpit system contex

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
# noinspection PyUnresolvedReferences
# import wcr_frontend.modules.model.gui.wmgui_icons
from wcr_core.modules.vlink.wvmod_application_module import \
    WVMOD_VLINK_APP_MODULE, WVMOD_VLINK_APP_CODE
from wcr_frontend.gui.wgui_scene_item_ids import XPNT_ITEM_ID, \
    CNCT_ITEM_ID
from wcr_core.common.wcmn_protocols import WcmnReleasableP
from wcr_frontend.gui.wgui_scene_view_mouse_handler import \
    WguiDefaultSceneViewMouseHandler
from wcr_frontend.modules.vlink.gui.wvgui_scene import WvguiGraphicsScene
from wcr_frontend.gui.wgui_workspace_tree import WguiWorkspaceTree
from wcr_frontend.gui.wgui_gui_module import \
    WguiCockpitSystemContextBase, WguiCockpitSystemModuleBase
from wcr_frontend.gui.wgui_clipboard import WguiClipboardHandler
from wcr_frontend.modules.vlink.gui.wvgui_scene_context_menu_factory import \
    WvguiSceneContextMenuFactory
from wcr_frontend.modules.vlink.gui.wvgui_props_dialog_factory import \
    WvguiPropertiesDialogFactory
from wcr_frontend.modules.vlink.gui.wvgui_scene_item_factory import \
    WvguiSceneItemFactory
from wcr_core.modules.vlink.db.wvhdb_standard_operators import WVHDB_VLINK_CODE
from wcr_core.action.wact_action_api import WACT_WCR_ACTION_CODE
from wcr_frontend.modules.vlink.gui.wvgui_project_actions import \
    WVGUI_IMPORT_CSV_ACTION
from wcr_frontend.modules.vlink.gui.wvgui_logic_table import \
    WvguiLogicViewTable
from wcr_frontend.modules.vlink.gui.wvgui_bus_binding_table import \
    WvguiBusBindingTable

#TODO: get proper icones for modules
_WBGUI_ICONS = {WVHDB_VLINK_CODE: ':/img/wpgui_phost.png'}


class _WvguiCockpitSystemContext(WguiCockpitSystemContextBase):
    def __init__(self, cmod, pctx, pcfg, actions, actx, parent):
        """
        :param cmod: Cockpit system module
        :type cmod: WguiCockpitSystemModuleP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param pcfg: Project editor configuration
        :type pcfg: WguiProjectEditorCfgP
        :param actions: Actions
        :type actions: dict[str, WactActionP]
        :param actx: Action execution context
        :type actx: WguiActionExecutionContextP
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(cmod)

        modActions = list(WVGUI_COCKPIT_SYSTEM_MODULE.getActions())
        """:type: list[(WactActionP, str, str, tuple[str])]"""

        self.__sceneItmFy = WvguiSceneItemFactory(WVMOD_VLINK_APP_MODULE)
        self.__scene = WvguiGraphicsScene(
            pctx.getProjectDB(), pcfg, actions, pctx, self.__sceneItmFy, actx, parent)
        self.__propDiagF = WvguiPropertiesDialogFactory(
            pctx, pcfg, self.__scene, actions, parent)
        self.__clpbrdHandler = WguiClipboardHandler(
            self.__scene, cmod.getAppModule())
        self.__cntxtMenuF = WvguiSceneContextMenuFactory(
            self.__scene, actions, self.__clpbrdHandler, pctx,
            self.__propDiagF)
        self.__mouseHandler = WguiDefaultSceneViewMouseHandler(
            pctx, self.__propDiagF, self.__scene, self.__cntxtMenuF)

        self.__sidebarModules = self._createSidebarModules(pctx, self.__scene,
                                                           self.__cntxtMenuF,
                                                           modActions,
                                                           parent)

    def release(self):
        for _, _, smod in self.__sidebarModules:
            if isinstance(smod, WcmnReleasableP):
                smod.release()

        for item in [self.__scene, self.__propDiagF, self.__clpbrdHandler,
                     self.__cntxtMenuF, self.__mouseHandler,
                     self.__sidebarModules]:
            if isinstance(item, WcmnReleasableP):
                item.release()

        self.__scene = None
        self.__propDiagF = None
        self.__clpbrdHandler = None
        self.__cntxtMenuF = None
        self.__mouseHandler = None
        self.__sidebarModules = []

        super().release()

    def _createSidebarModules(self, pctx, scene, cntxtMenuF, actions, parent):
        modules = []

        return modules

    def getExcludedSystemToolboxItems(self):
        return [XPNT_ITEM_ID,CNCT_ITEM_ID]

    def getHighlightingModules(self):
        return []

    def getDirectoryDescription(self, level):
        return 'project' if level == 0 else 'sched. group'

    def getScene(self):
        return self.__scene

    def getSceneViewMouseHandler(self):
        return self.__mouseHandler

    def getClipboardHandler(self):
        return self.__clpbrdHandler

    def getSceneItemFactory(self):
        return self.__sceneItmFy

    def getContextMenuFactory(self):
        return self.__cntxtMenuF

    def getSidebarModules(self):
        return iter(self.__sidebarModules)

    def getPropertiesDialogFactory(self):
        return self.__propDiagF

    def updateSides(self):
        #: update bus binding table
        self.__sidebarModules[1][2].reloadFromData()


class _WvguiCockpitSystemModule(WguiCockpitSystemModuleBase):
    def __init__(self):
        super().__init__(WVMOD_VLINK_APP_MODULE)
        self._createDefaultActions()

    def _createDefaultActions(self):
        actSection = '{}.{}'.format(WACT_WCR_ACTION_CODE,
                                    WVMOD_VLINK_APP_CODE)
        self.addAction(
            WVGUI_IMPORT_CSV_ACTION, 'import', actSection, 'import')

    def createContext(self, pctx, pcfg, actions, actx, parent):
        return _WvguiCockpitSystemContext(
            self, pctx, pcfg, actions, actx, parent)

    def getIconForCode(self, code, iconType):
        return _WBGUI_ICONS.get(code) or \
            super().getIconForCode(code, iconType)


#: GUI Cockpit system cotnext singleton
WVGUI_COCKPIT_SYSTEM_MODULE = _WvguiCockpitSystemModule()
