#!/usr/bin/env python
# -*- coding: utf-8 -*-

##############################################################################################################################
# 											Satusryprt V 006	17/07/2019
##############################################################################################################################

from __future__ import absolute_import, division, print_function

import sys
from datetime import datetime
from functools import partial
import time
import os
import ctypes as ct
import configparser as cp
from status_utils import isFrameCetrac, isMMResponse, computeCetracFrame, \
						 computeMMResponse, createCetraCMMread, getErrorMessage,\
						 readMessages
from constants import *

import libpcap as pcap

from PyQt5 import QtCore, QtGui, QtWidgets, QtNetwork
from PyQt5.QtWidgets import *
from PyQt5.QtCore import  Qt, QTimer, QTime, QFile
from PyQt5.QtGui import *


def InitPcapList():
	global ebuf
	global tabDevName
	global tabDevDescription
	global countDev
	
	tabDevName = []
	tabDevDescription = []
	countDev = 0
	ebuf = ct.create_string_buffer(pcap.PCAP_ERRBUF_SIZE)

	devlist = ct.POINTER(pcap.pcap_if_t)()
	nextdev = ct.POINTER(pcap.pcap_if_t)()
	if pcap.findalldevs(ct.byref(devlist), ebuf) == -1:
		print("{!s}", ebuf.value.decode("utf-8", "ignore"))
	if not devlist:
		print("no interfaces available for capture")
	while devlist:
		tabDevName.append(devlist[0].name)
		tabDevDescription.append(devlist[0].description)
		countDev = countDev + 1
		devlist = devlist[0].next		
	pcap.freealldevs(devlist)
	return
	
def InitPcap(device):
	global pd
	global ebuf
	
	ebuf = ct.create_string_buffer(pcap.PCAP_ERRBUF_SIZE)
	ebuf.value = b""
	pd = pcap.open_live(device, 65535, True, 10000, ebuf)
	if not pd:
		print("{!s}", ebuf.value.decode("utf-8", "ignore"))
	elif ebuf.value:
		print("{!s}", ebuf.value.decode("utf-8", "ignore"))
	pcap.setnonblock(pd, True, ebuf)
	localnet = pcap.bpf_u_int32()
	netmask  = pcap.bpf_u_int32()
	if pcap.lookupnet(device, ct.byref(localnet), ct.byref(netmask), ebuf) < 0:
		localnet = pcap.bpf_u_int32(0)
		netmask  = pcap.bpf_u_int32(0)
		print("{!s}", ebuf.value.decode("utf-8", "ignore"))
	return
#----------------------------------------------------------------------------------------
# --- Libpcap handler called for each incoming packet Warning : the next line is mandatory
#----------------------------------------------------------------------------------------	
@pcap.pcap_handler
def myHandler(arg, hdr, pkt):
	global ErrDevice
	global ErrPort
	global ErrData
	global ErrFlag
	global ErrNb
	global ErrLock
	global ErrManaged
	global RespFlag
	global RespData
	global RespComp
	global RespBoard
	global RespEquip

	# notifId = 0x400 not used anymore
	trapId = 0x401

	#  Check if the frame is CetraC frame
	cetracId = isFrameCetrac(pkt)
	if cetracId is not None:
		#  Check if it is a Status CetraC frame and if we have to manage it
		if (cetracId == trapId) and ErrManaged:
			ErrLock = True
			ErrFlag = True
			if (ErrNb < 100):
				err_device, err_port, err_data = computeCetracFrame(pkt)
				ErrDevice[ErrNb] = err_device
				ErrPort[ErrNb] = err_port
				ErrData[ErrNb]= err_data
				ErrNb += 1
			ErrLock = False
		#  Check if it is a MMResponse CetraC frame
		elif isMMResponse(pkt):
			resp_data, resp_equip, resp_board, resp_comp = computeMMResponse(pkt)
			RespData = resp_data
			RespEquip = resp_equip
			RespBoard = resp_board
			RespComp = resp_comp
			RespFlag = True
	return
#================================================================================
#				Main class MyCETRAC
#================================================================================
class MyCETRAC():
	def __init__(self, dialog):

		self.InitWindowMain(dialog)
		global verboseMode
		global ethernetMode
		#  Global values for the error frames content received by the libpacp frame handler
		global ErrDevice
		global ErrPort
		global ErrData
		global ErrFlag
		global ErrNb
		global ErrLock
		global ErrManaged
		global LineByEquipment
		#  Global values for MMread and MMresponse handling
		global RespFlag
		global RespData
		global RespComp
		global RespBoard
		global RespEquip
		global RprtDevice
		global RprtPort
		global RprtStep
		global RespTimeCount
		#  Global values for Ethernet devices
		global tabDevName
		global tabDevDescription
		global countDev
		#  Display parameters values
		self.ErrorDisplayDuration = 5.0
		self.nbDevicesDisplay = 3
		self.largDataPort = DATAPORT_WIDTH
		self.largWindow = WINDOW_WIDTH_INIT
		self.hauteurLog = LOG_HEIGHT
		self.hauteurDevice = PORTLABEL_HEIGHT + PORTBUTTON_HEIGHT
		self.nbMaxDevices = 10
		self.nbMaxPorts = 30
		self.nbMinPorts = 3
			
		self.BorderBouton = "#505050"		
		self.BackgroundBouton = "#A0A0A0"
		self.BackgroundLabel = "#00A000"
		self.BackgroundDevName = "#A0A0A0"
		self.BackgroundBoutonSelect = "#808080"
		self.BackgroundLabelError = "#FF4040"
		self.BackgroundDevNameSelect = "#808080"		
		#  Messages table
		self.TabMessages = [[""]*100 for _ in range(CETRAC_MAXTYPES)]
		self.TabMessages[0][0] = "Unknown error"
		self.TabMessages[0][1] = "Local fault received"
		self.TabMessages[0][2] = "Remove fault received"
		self.TabMessages[0][3] = "PHY symbol receive error / CRC32 error / Framing error"
		self.TabMessages[0][4] = "Transmission error ( link down while transfering )"
		self.TabMessages[0][5] = "Invalid packet"			
		#  Values initialization
		self.Timer_IntervalMs=25
		# Global variables for the status frames handler
		ErrDevice = [0]*100
		ErrPort = [0]*100
		ErrData = [0]*100
		ErrFlag = False
		ErrLock = False
		# Is the port initialized for this device
		self.PortsInitialyzed = [[0]*self.nbMaxPorts for _ in range(self.nbMaxDevices)]
		# Number of ports displayed in the interface by device
		self.PortsDisplayed = [0]*self.nbMaxDevices
		# Reverse table of LineByEquipment gives the equipment number per interface line number starting from 0
		self.EquipmentByLine = [0]*self.nbMaxDevices
		self.nbMaxPortsDetected = 0
		self.logFile = False
		self.logDirectory ="."
		self.discover = True
		self.readStatus = True
		self.ethenetPortNumber = -1
		#  First step only MMresponse frames are handled for CetraC network discovery
		ErrManaged = False
		ErrNb = 0
		RespFlag = False
		RespData = [0]*4
		RespTimeCount = 0
		RprtDevice = -1
		RprtPort = -1
		RprtStep = 0
		# Table giving the interface line number by equipment number interface line number starts from 0
		LineByEquipment = [-1]*30
		self.nbDevicesDetected = 0
		# Selected device line from 0 to max-1  -1 for no device selected
		self.DevSelected = -1
		self.selectedPort = -1
		self.selectedDevice = -1
		self.BtDevSelected = 0
		self.BtPortSelected = 0
		self.Total_Error = 0
		self.CurPort = 0
		self.ErrCount = 0
		# Tables to read the configuration file
		self.devNumberCfg = [0]*self.nbMaxDevices
		self.portNumberCfg = [0]*self.nbMaxDevices
		# Tables to store the ports currently in error and the time it started to be in error
		self.FlagError = [0]*self.nbMaxDevices*self.nbMaxPorts
		self.TimeError = [0]*self.nbMaxDevices*self.nbMaxPorts
		# Ports types per device and internal port number
		self.PortsTypes = [[0]*self.nbMaxPorts for _ in range(self.nbMaxDevices)]
		# Displayed port number in The ports Label per device and internal port number
		self.PortsLabel = [[0]*self.nbMaxPorts for _ in range(self.nbMaxDevices)]
		# Internal port index per device and displayed port number
		self.PortsDisplay = [[0]*self.nbMaxPorts for _ in range(self.nbMaxDevices)]
		# Initialization of the default port type for all the possible devices as a Babelya device
		for i in range(self.nbMaxDevices):
			self.PortsTypes[i][0:len(CETRAC_Babelya_Types)] = CETRAC_Babelya_Types
			self.PortsDisplay[i][0:len(CETRAC_Babelya_Display)] = CETRAC_Babelya_Display
		# Table to count the number of devices by type by device in order to give the interface a number
		self.PortsByType = [[0]*CETRAC_MAXTYPES for _ in range(self.nbMaxDevices)]
		# Widgets reference tables for dynamic interface generation
		self.grpBox = [0]*self.nbMaxDevices
		self.Bdev = [0]*self.nbMaxDevices
		self.BUP = [0]*self.nbMaxDevices
		self.BDW = [0]*self.nbMaxDevices
		self.S = [[0]*self.nbMaxPorts for _ in range(self.nbMaxDevices)]
		self.G = [[0]*self.nbMaxPorts for _ in range(self.nbMaxDevices)]
		# Basic windows initialization
		app.aboutToQuit.connect(self.closeEvent)
		dialog.resize(self.largWindow, LOG_HEIGHT_INIT)	
		self.Log.resize(self.largWindow, LOG_HEIGHT_INIT)
		self.DataPort.resize(self.largDataPort, LOG_HEIGHT_INIT)
		self.DataPort.move(self.largWindow-self.largDataPort,self.hauteurDevice*self.nbDevicesDisplay)	
		# Connect functions to buttons for the basic windows
		self.NetOK.clicked.connect(self.ValidEtherDev)	
		# Get the Ethernet device list
		InitPcapList()
		# Get the Messages list
		readMessages(self.TabMessages)
		# Load the configuration file
		self.ConfigParser()

		# Selection of the wanted Ethernet Device only if none is defined from config or called with -E option
		self.PopUpEther.hide()
		self.DataPort.hide()
		if self.ethenetPortNumber == -1 or ethernetMode == 1:
			for dev in tabDevDescription :
				self.NetPort.addItem(dev.decode())
			self.PopUpEther.setWindowModality(QtCore.Qt.ApplicationModal)
			self.PopUpEther.show()
		else:
			self.ValidEtherDev()

	def ConfigParser(self):
		self.config = cp.ConfigParser()
		self.config.read("Statusryprt.cfg")
		if "Log" in self.config:
			if "file" in self.config.options("Log"):
				cf = self.config["Log"]["File"].upper()
				self.logFile = (cf == "Y" or cf == "YES")
			if "directory" in self.config.options("Log"):
				self.logDirectory = self.config["Log"]["directory"]
		if "MMRead" in self.config:
			if "discover" in self.config.options("MMRead"):
				cf = self.config["MMRead"]["discover"].upper()
				self.discover = (cf != "N" and cf != "NO")
			if "readstatus" in self.config.options("MMRead"):
				cf = self.config["MMRead"]["readstatus"].upper()
				self.readStatus = (cf != "N" and cf != "NO")
		if "Ethernet" in self.config:
			if "port" in self.config.options("Ethernet"):
				try:
					self.ethenetPortNumber = int(self.config["Ethernet"]["port"])
				except ValueError:
					self.ethenetPortNumber = -1
				# if Ethernet port is not a number or greater than port count (countDev) ask the user
				if self.ethenetPortNumber >= countDev:
					self.ethenetPortNumber = -1
		if "Layout" in self.config:
			if "devicesnumber" in self.config.options("Layout"):
				try:
					self.devicesNumberCfg = int(self.config["Layout"]["devicesnumber"])
					if self.devicesNumberCfg < 1:
						self.devicesNumberCfg = 0
				except ValueError:
					self.devicesNumberCfg = 0
				if self.devicesNumberCfg > self.nbMaxDevices:
					self.devicesNumberCfg = self.nbMaxDevices
				if self.devicesNumberCfg > 0:
					for dev in range(self.devicesNumberCfg):
						dv = "device" + str(dev + 1)
						if dv in self.config.options("Layout"):
							dcfg = self.config["Layout"]["Device" + str(dev + 1)].split(",")
							try:
								if int(dcfg[0]) > 0 and int(dcfg[0]) <= len(LineByEquipment):
									self.devNumberCfg[dev] = int(dcfg[0])
								else:
									self.devNumberCfg[dev] = 0
							except (ValueError, IndexError):
								self.devNumberCfg[dev] = 0
							try:
								if int(dcfg[1]) > self.nbMinPorts and int(dcfg[1]) <= self.nbMaxPorts:
									self.portNumberCfg[dev] = int(dcfg[1])
								else:
									if int(dcfg[1]) > self.nbMaxPorts:
										self.portNumberCfg[dev] = self.nbMaxPorts
									else:
										self.portNumberCfg[dev] = self.nbMinPorts
							except (ValueError, IndexError):
								self.portNumberCfg[dev] = 10
						else:
							self.devNumberCfg[dev] = dev
							self.portNumberCfg[dev] = 10
		if "Ports" in self.config:
			for dev in range(self.devicesNumberCfg):
				dv = "device" + str(dev + 1)
				if dv in self.config.options("Ports"):
					# if dev is in the config file starts by resetting the types ans display to 0
					self.PortsTypes[dev] = [0] * self.nbMaxPorts
					self.PortsDisplay[dev] = [0] * self.nbMaxPorts
					dcfg = self.config["Ports"]["Device" + str(dev + 1)].split(",")
					i = 0
					j = 0
					for port in dcfg:
						try:
							type = CETRAC_Names.index(port)
						except:
							type = 0
						self.PortsTypes[dev][i] = type
						if type != 0:
							self.PortsDisplay[dev][j] = i
							j = j + 1
						i += 1
					self.PortsDisplayed[dev] = i
				# If a log file is needed open a new file
		if self.logFile:
			if not os.path.isdir(self.logDirectory):
				self.logDirectory = "."
			fileName = self.logDirectory + "/" + "stat-" + datetime.now().strftime("%m%d%Y-%H%M%S") + ".log"
			self.log = open(fileName, "w")
	#--------------------------------------------------------------------------------
	# --- When Ethernet port is choosen auto-discover the network and build the interface
	#--------------------------------------------------------------------------------
	def ValidEtherDev(self):
		global ErrManaged
		
		if self.ethenetPortNumber == -1 or ethernetMode == 1:
			self.PopUpEther.hide()
			self.ethenetPortNumber = self.NetPort.currentIndex()
		# Write selected port in configuration file
		if not "Ethernet" in self.config:
			self.config["Ethernet"] = {}
		self.config["Ethernet"]["port"] = str(self.ethenetPortNumber)
		with open("Statusryprt.cfg", "w") as configfile:
			self.config.write(configfile)
		# Start Ethernet frame reading
		print("Selected Ethernet device : ",self.ethenetPortNumber)
		InitPcap(tabDevName[self.ethenetPortNumber])
		# Auto seach procedure to discover the CetraC network with available devices numbers and IDs
		if self.discover:
			self.CetraCDiscover()
		# If no device found use the config file layout description if any
		if self.nbDevicesDetected == 0:
			if self.devicesNumberCfg != 0:
				self.nbDevicesDetected = self.devicesNumberCfg
				self.nbMaxPortsDetected = 0
				for i in range (self.nbDevicesDetected):
					self.PortsDisplayed[i] = self.portNumberCfg[i]
					if self.portNumberCfg[i] > self.nbMaxPortsDetected:
						self.nbMaxPortsDetected = self.portNumberCfg[i]
					LineByEquipment[self.devNumberCfg[i]] = i
					self.EquipmentByLine[i] = self.devNumberCfg[i]

			# As the data is comming from cfg file we consider all the possible internal ports as initialized
			for i in range(self.nbDevicesDetected):
				for j in range(self.nbMaxPorts):
					self.PortsInitialyzed[i][j]=1
					
		# For the Max number of ports to display and the number of ports to display by device rely on cfg file
		# Do not rely on the number of initialized ports as we dont'know if we want to display them of not
		self.nbMaxPortsDetected = 1
		for i in range(self.nbDevicesDetected):
			nbports = 0
			for j in range(self.nbMaxPorts):
				if self.PortsDisplay[i][j] != 0:
					nbports = nbports + 1
			self.PortsDisplayed[i] = nbports
			if nbports > self.nbMaxPortsDetected:
				self.nbMaxPortsDetected = nbports
				
		# If no device found in the configuration and no discovery data available consider 3 devices with 10 ports each
		if self.nbDevicesDetected == 0:
			self.nbDevicesDetected = 3
			self.PortsDisplayed[0] = 10
			self.PortsDisplayed[1] = 10
			self.PortsDisplayed[2] = 10
			self.nbMaxPortsDetected = 10

		# Now devices are known setup the display accordignly
		if self.nbDevicesDetected <= self.nbMaxDevices:
			self.nbDevicesDisplay = self.nbDevicesDetected
		else:
			self.nbDevicesDisplay = self.nbMaxDevices
		if verboseMode == 1 :
			print("LineByEquipment",LineByEquipment)
			print("EquipmentByLine",self.EquipmentByLine)
		self.InitWindowDevices()
#		 If a css file is avaliable use it
		file = QFile("Statusryprt.css")
		if file.open(QFile.ReadOnly):
			styleSheet = bytes(file.readAll()).decode('UTF-8')
			for i in range(self.nbDevicesDisplay):
				self.grpBox[i].setStyleSheet("")
			app.setStyleSheet(styleSheet);
		# Get back colors after css application
		self.BackgroundDevName = self.Bdev[0].palette().color(QPalette.Background).name()
		self.BackgroundBouton = self.G[0][0].palette().color(QPalette.Background).name()
		self.BackgroundLabel = self.S[0][0].palette().color(QPalette.Background).name()		
		# Timer initialization for scheduler
		self.timer=QTimer()
		self.timer.timeout.connect(self.MyScheduler)
		self.timer.setInterval(self.Timer_IntervalMs)
		self.timer.start()
		self.Log.clear()
		# Now we start to read the status frames and let's go
		ErrManaged = True
		return
	#--------------------------------------------------------------------------------
	# --- Functions for the buttons
	#--------------------------------------------------------------------------------
	def BoutonReset(self):
		self.Log.clear()
		self.DataPort.clear()
		return
		
	def DispDataPort(self,dev,internalPort):
		# dev starts from 0
		if dev == self.DevSelected:
			if verboseMode == 2:
				print("*",dev,"-",internalPort)
			if internalPort == 1 or internalPort == 2:
				self.DataPort.clear()
				self.DataPort.appendPlainText("No data available for port 10G")
			else:
				self.GetDataPort(self.DevSelected,internalPort,1)
		return
		
	def DevSelect(self,dev):
		# dev starts from 0
		if self.readStatus:
			if self.DevSelected == dev:
				self.ResetSelectedPort()
				self.DevSelected = -1
				self.DataPort.clear()
				self.DataPort.hide()
				self.Bdev[dev].setStyleSheet("background-color: " + self.BackgroundDevName)
			elif self.DevSelected == -1:
				self.DevSelected = dev
				self.DataPort.show()
				self.Bdev[dev].setStyleSheet("background-color: " + self.BackgroundDevNameSelect)
				self.BtDevSelected = self.Bdev[dev]
				self.GetDataPort(dev,0,1)
			else:
				self.ResetSelectedPort()
				self.BtDevSelected.setStyleSheet("background-color: " + self.BackgroundDevName)
				self.DevSelected = dev
				self.Bdev[dev].setStyleSheet("background-color: " + self.BackgroundDevNameSelect)
				self.BtDevSelected = self.Bdev[dev]
				self.GetDataPort(dev,0,1)
		return

	def closeEvent(self):
		if self.logFile:
			self.log.close()
		return
	#--------------------------------------------------------------------------------
	# --- Supporting functions
	#--------------------------------------------------------------------------------
	def GetInternalPort(self,dev,Port):
		return(self.PortsDisplay[dev][Port])
		
	def GetDisplayPort(self,dev,internalPort):
		try:
			i = self.PortsDisplay[dev].index(internalPort)
		except:
			i = -1
		return(i)
		
	def SetErrorPort(self,dev,internalPort):
		# Here dev starts from 0 as the number comes from the LineByEquipment table starting from 0
		self.Total_Error += 1
		iPort = dev*self.nbMaxPorts+internalPort-1
		self.FlagError[iPort] = 1
		self.TimeError[iPort] = time.time()
		port = self.GetDisplayPort(dev,internalPort)
		if verboseMode == 1 :
			print("Erreur",dev,"Int:",internalPort,"Ext:",port)
		self.S[dev][port].setStyleSheet("background-color: " + self.BackgroundLabelError + "; color: #000000")
		return
		
	def ResetErrorPort(self,dev,internalPort):
		# Here dev starts from 0 as the number comes from the LineByEquipment table starting from 0
		self.Total_Error -= 1
		iPort = dev*self.nbMaxPorts+internalPort-1
		self.FlagError[iPort] = 0
		port = self.GetDisplayPort(dev,internalPort)
		self.S[dev][port].setStyleSheet("background-color: " + self.BackgroundLabel + "; color: #000000")
		return	
		
	def SetSelectedPort(self,dev,displayPort):	
		self.G[dev][displayPort].setStyleSheet("background-color: " + self.BackgroundBoutonSelect + "; font-size: 14px")
		self.selectedPort = displayPort
		self.selectedDevice = dev
		return
		
	def ResetSelectedPort(self):
		if self.selectedPort != -1:
			self.G[self.selectedDevice][self.selectedPort].setStyleSheet("background-color: " + self.BackgroundBouton + "; font-size: 14px")
		self.selectedPort = -1
		return
	
	def GetDataPort(self,dev,internalPort,step):
		# dev starts from 0
		global RprtDevice
		global RprtPort
		global RprtStep
		global RespFlag

		displayPort = self.GetDisplayPort(dev,internalPort)
		if verboseMode == 2:
			print(str(dev)+","+str(internalPort))
			print(self.PortsInitialyzed[dev][internalPort])
		if (self.PortsInitialyzed[dev][internalPort] == 1 or internalPort == 0):
			if verboseMode == 2:
				print("*")
			if internalPort > 0:
				self.ResetSelectedPort()
				self.SetSelectedPort(dev,displayPort)
			if step == 1:
				# If called from the interface do nothing if a demand is allready pending
				if RprtStep == 0:
					RprtDevice = dev
					RprtPort = internalPort
					RprtStep = step
					RespFlag = False
					if internalPort == 0:
						mmRead = createCetraCMMread(2*(RprtDevice)+1,0)
					else:
						mmRead = createCetraCMMread(2*(RprtDevice)+1,0x100+(RprtPort-5)*8)
					self.SendCetraCMMread(mmRead)
			elif step == 2:
				RprtDevice = dev
				RprtPort = internalPort
				RprtStep = step
				RespFlag = False
				if internalPort == 0:
					self.SendCetraCMMread(createCetraCMMread(2*(RprtDevice)+1,4))
				else:
					self.SendCetraCMMread(createCetraCMMread(2*(RprtDevice)+1,0x100+(RprtPort)*8 + 4))
			elif step == 3:
				RprtDevice = dev
				RprtPort = internalPort
				RprtStep = step
				RespFlag = False
				if internalPort != 0:
					self.SendCetraCMMread(createCetraCMMread(2*(RprtDevice)+1,0x200+(RprtPort)*8))
			elif step == 4:
				RprtDevice = dev
				RprtPort = internalPort
				RprtStep = step
				RespFlag = False
				if internalPort != 0:
					self.SendCetraCMMread(createCetraCMMread(2*(RprtDevice)+1,0x200+(RprtPort)*8 + 4))
			else:
				RprtDevice = dev
				RprtPort = internalPort
				RprtStep = step
				RespFlag = False
				if internalPort == 0:
					mmRead = createCetraCMMread(2*(RprtDevice)+1,4)
				else:
					mmRead = createCetraCMMread(2*(RprtDevice)+1,0x100+(RprtPort-5)*8 + 4)
				self.SendCetraCMMread(mmRead)
		return	
		
	def LogError(self,LogMessage):
		buflog = datetime.now().strftime("%H:%M:%S")+" - "+LogMessage
		self.Log.appendPlainText(buflog)
		if self.logFile:
			newlog = buflog.replace(' - ', ',')
			self.log.write(newlog + "\n")
		return

	def SendCetraCMMread(self, mmRead):
		global pd
		global ebuf
		pcap.sendpacket(pd, mmRead, 188)
	#------------------------------------------------------------------------------------------
	#									CetraCDiscover
	#
	#  Seach the network for all CetraC devices and get their ID and initialization vector
	#------------------------------------------------------------------------------------------
	def CetraCDiscover(self):
		global RespComp
		global RespBoard
		global RespEquip
		global RespFlag
		global RespData
		global LineByEquipment

		for dev in range(1,21):
			RespFlag = False
			WaitCount = 0
			self.SendCetraCMMread(createCetraCMMread(2*(dev-1)+1,4))
			packet_count = ct.c_int(0)
			status = pcap.dispatch(pd, -1, myHandler, ct.cast(ct.pointer(packet_count), ct.POINTER(ct.c_ubyte)))
			while RespFlag == False:
				if WaitCount > 20:
					print("No device {:2d} found".format(dev))
					break
				else:
					WaitCount = WaitCount + 1
					time.sleep(0.01)
					status = pcap.dispatch(pd, -1, myHandler, ct.cast(ct.pointer(packet_count), ct.POINTER(ct.c_ubyte)))
			if RespFlag == True:
				print("Device {:2d}".format(dev))
				print("Initialization vector: {0:032b}".format(RespData[0]))
				print("Equipment: {:2d} Board: {:2d} Component: {:2d}".format(RespEquip,RespBoard,RespComp))
				LineByEquipment[RespEquip] = self.nbDevicesDetected
				self.EquipmentByLine[self.nbDevicesDetected] = RespEquip
				self.nbDevicesDetected += 1
				initVector = RespData[0]				
							
##################################################################################################################################
# Until the ports can tell who they are only rely on cfg file except for initialization vector
##################################################################################################################################

				for port in range(self.nbMaxPorts):
					if initVector & 0x01:
						self.PortsInitialyzed[self.nbDevicesDetected-1][port] = 1						
					initVector = initVector >> 1
					
				self.PortsDisplayed[self.nbDevicesDetected-1] = 10
		return
	#----------------------------------------------------------------------------------
	#								InitWindowDevices
	#
	#  Initialization of the whole interface after computation and widget creation
	#----------------------------------------------------------------------------------
	def InitWindowDevices(self):
		self.largWindow = DEVICE_BUTTON_WIDTH + self.nbMaxPortsDetected * PORTLABEL_WIDTH
		for i in range(self.nbDevicesDisplay):
			self.grpBox[i] = QtWidgets.QGroupBox(self.centralwidget)
			self.grpBox[i].setGeometry(QtCore.QRect(0, i*self.hauteurDevice, self.largWindow, self.hauteurDevice))	
			self.grpBox[i].setObjectName("groupBox_"+str(i))
			self.grpBox[i].setStyleSheet("\n"
"QLabel { font: 40px arial, sans-serif,bold; color: #000000; background-color: " + self.BackgroundLabel + "; text-align: top; border-width: 1px; border-color: #1e1e1e; border-style: solid; border-radius:0}\n"
"QPushButton { background-color: " + self.BackgroundBouton + " ; border-color: " + self.BorderBouton + "; font-size: 22px}\n"
"")		
			for port in range(self.PortsDisplayed[i]):		
				self.S[i][port] = QtWidgets.QLabel(self.grpBox[i])
				self.S[i][port].setGeometry(QtCore.QRect(DEVICE_BUTTON_WIDTH + PORTLABEL_WIDTH*port, 0, PORTLABEL_WIDTH, PORTLABEL_HEIGHT))
				self.S[i][port].setAlignment(QtCore.Qt.AlignCenter)
				self.S[i][port].setObjectName("SPort"+str(port)+"_"+str(i))
				self.G[i][port] = QtWidgets.QPushButton(self.grpBox[i])
				self.G[i][port].setGeometry(QtCore.QRect(DEVICE_BUTTON_WIDTH + PORTLABEL_WIDTH*port, PORTLABEL_HEIGHT, PORTLABEL_WIDTH, PORTBUTTON_HEIGHT))
				self.G[i][port].setObjectName("GPort"+str(port)+"_"+str(i))
				internalPort = self.GetInternalPort(i,port)
				if self.PortsInitialyzed[i][internalPort] == 1:
					self.G[i][port].setStyleSheet("font-size: 14px; color : #000000;")
				else:
					self.G[i][port].setStyleSheet("font-size: 14px; color : #808080;")
										
##################################################################################################################################					
# Here we set the text inside the port button initially 1 and 2 pour 10G and 1 , 2 , 3 , ... for the others was orginally stated
# Changeg by counting the number of ports of the same type and give the same porr type order number starting from 1
##################################################################################################################################
				
				portNum = self.PortsByType[i][self.PortsTypes[i][internalPort]] + 1
				self.PortsByType[i][self.PortsTypes[i][internalPort]] = portNum
				self.PortsLabel[i][internalPort] = portNum
				
				self.S[i][port].setText(QtCore.QCoreApplication.translate("MainWindow", str(portNum)))
						
				self.G[i][port].setText(QtCore.QCoreApplication.translate("MainWindow", CETRAC_Names[self.PortsTypes[i][internalPort]]))
				
				
			self.Bdev[i] = QtWidgets.QPushButton(self.grpBox[i])
			self.Bdev[i].setGeometry(QtCore.QRect(0, 0, DEVICE_BUTTON_WIDTH, PORTLABEL_HEIGHT))
			self.Bdev[i].setObjectName("Bdev_"+str(i))
			self.Bdev[i].setText(QtCore.QCoreApplication.translate("MainWindow", "Device "+str(i+1)))
			self.BUP[i] = QtWidgets.QPushButton(self.grpBox[i])
			self.BUP[i].setGeometry(QtCore.QRect(0, PORTLABEL_HEIGHT, DEVICE_BUTTON_WIDTH / 2, PORTBUTTON_HEIGHT))
			# Write device component number in the left buttons
			self.BUP[i].setStyleSheet("font-size: 14px")
			if self.EquipmentByLine[i] != 0:
				self.BUP[i].setText(str(self.EquipmentByLine[i]))
			else:
				self.BUP[i].setText("")
			self.BUP[i].setObjectName("BUP_"+str(i))
			self.BDW[i] = QtWidgets.QPushButton(self.grpBox[i])
			self.BDW[i].setGeometry(QtCore.QRect(DEVICE_BUTTON_WIDTH / 2, PORTLABEL_HEIGHT, DEVICE_BUTTON_WIDTH / 2, PORTBUTTON_HEIGHT))
			if i == 0:
				icon1 = QtGui.QIcon()
				icon1.addPixmap(QtGui.QPixmap("Clear.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
				self.BDW[i].setIcon(icon1)
				self.BDW[i].setIconSize(QtCore.QSize(50, 16))
		# Window can't be resized
		dialog.setFixedSize(self.largWindow, self.hauteurDevice* self.nbDevicesDisplay+ self.hauteurLog)	
		self.Log.resize(self.largWindow, self.hauteurLog)
		self.Log.move(0,self.hauteurDevice*self.nbDevicesDisplay)
		self.DataPort.resize(self.largDataPort, self.hauteurLog)
		self.DataPort.move(self.largWindow-self.largDataPort,self.hauteurDevice*self.nbDevicesDisplay)
		for dev in range(self.nbDevicesDisplay):
			self.grpBox[dev].move(0,(self.nbDevicesDisplay-dev-1)*self.hauteurDevice)
			self.grpBox[dev].show()		
		# Connect buttons to functions
		self.BDW[0].clicked.connect(self.BoutonReset)		
		for dev in range(self.nbDevicesDisplay):
			self.Bdev[dev].clicked.connect(partial(self.DevSelect,dev))		
			for i in range(self.PortsDisplayed[dev]):
				port = self.GetInternalPort(dev,i)				
				self.G[dev][i].clicked.connect(partial(self.DispDataPort,dev,port))
				
		return
	#----------------------------------------------------------------------------------
	#									InitWindowMain
	#
	#  Initialization of the whole interface after computation and widget creation
	#----------------------------------------------------------------------------------
	def InitWindowMain(self, MainWindow):
		
		MainWindow.setObjectName("MainWindow")
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
		sizePolicy.setHorizontalStretch(0)
		sizePolicy.setVerticalStretch(0)
		sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
		MainWindow.setSizePolicy(sizePolicy)
		self.centralwidget = QtWidgets.QWidget(MainWindow)
		self.centralwidget.setObjectName("myCentral")
		self.Log = QtWidgets.QPlainTextEdit(self.centralwidget)
		self.Log.setGeometry(QtCore.QRect(0, 0, WINDOW_WIDTH_INIT, LOG_HEIGHT_INIT))
		self.Log.setStyleSheet("color: #000000;\n"
" background-color: #C0C0C0;\n"
" border-width: 5px;\n"
" border-color: #008000;\n"
" border-style: solid;\n"
" border-radius:0;\n"
" padding: 1px;\n"
" font-family:  'Courrier New';\n"
" font-size: 11px;\n"
" padding-left: 5px;\n"
" padding-right: 5px;\n"
"")
		self.Log.setReadOnly(True)
		self.Log.setPlainText("")
		self.Log.setObjectName("Log")      
		self.PopUpEther = QtWidgets.QWidget(self.centralwidget)
		self.PopUpEther.setEnabled(True)
		self.PopUpEther.setGeometry(QtCore.QRect((WINDOW_WIDTH_INIT - ETHSELECT_WIDTH) / 2, (LOG_HEIGHT_INIT - ETHSELECT_HEIGHT) / 2, ETHSELECT_WIDTH, ETHSELECT_HEIGHT))
		self.PopUpEther.setStyleSheet(" border-width: 3px; border-color: #1e1e1e; border-style: solid; border-radius:0")
		self.PopUpEther.setObjectName("PopUpEther")
		self.labelPort = QtWidgets.QLabel(self.PopUpEther)
		self.labelPort.setGeometry(QtCore.QRect(2, 2, ETHSELECT_WIDTH - ETHSELECT_OK_WIDTH, (ETHSELECT_HEIGHT / 2) - 2))
		self.labelPort.setStyleSheet(" color: #ffffff;\n"
" background-color: #00A000;\n"
" border-width: 1px;\n"
" border-color: #1e1e1e;\n"
" border-style: solid;\n"
" border-radius:0;\n"
" padding: 1px;\n"
" font-size: 18px;\n"
" padding-left: 5px;\n"
" padding-right: 5px;\n"
"")
		self.labelPort.setObjectName("labelPort")
		self.NetPort = QtWidgets.QComboBox(self.PopUpEther)
		self.NetPort.setGeometry(QtCore.QRect(2, ETHSELECT_HEIGHT / 2, ETHSELECT_WIDTH - 4, (ETHSELECT_HEIGHT / 2) - 2))
		self.NetPort.setStyleSheet(" border-width: 1px;\n"
" border-color: #1e1e1e;\n"
" border-style: solid;\n"
" border-radius:0;\n"
" padding: 1px;\n"
" font-size: 12px;\n"
" padding-left: 5px;\n"
" padding-right: 5px;")
		self.NetPort.setObjectName("NetPort")
		self.NetOK = QtWidgets.QPushButton(self.PopUpEther)
		self.NetOK.setGeometry(QtCore.QRect(ETHSELECT_WIDTH - ETHSELECT_OK_WIDTH - 2, 2, ETHSELECT_OK_WIDTH, (ETHSELECT_HEIGHT / 2) - 2))
		self.NetOK.setStyleSheet(" color: #ffffff;\n"
" background-color: #00A000;\n"
" border-width: 1px;\n"
" border-color: #1e1e1e;\n"
" border-style: solid;\n"
" border-radius:0;\n"
" padding: 1px;\n"
" font-size: 18px;")
		self.NetOK.setObjectName("NetOK")
		self.NetOK.raise_()
		self.labelPort.raise_()
		self.NetPort.raise_()
		self.DataPort = QtWidgets.QPlainTextEdit(self.centralwidget)
		self.DataPort.setGeometry(QtCore.QRect(WINDOW_WIDTH_INIT - DATAPORT_WIDTH, 240, DATAPORT_WIDTH, LOG_HEIGHT_INIT))
		self.DataPort.setReadOnly(True)
		self.DataPort.setPlainText("")
		self.DataPort.setObjectName("DataPort")
		self.DataPort.setStyleSheet("color: #000000;\n"
" background-color: #C0C0C0;\n"
" border-width: 5px;\n"
" border-color: #008000;\n"
" border-style: solid;\n"
" border-radius:0;\n"
" padding: 1px;\n"
" font-family:  'Courrier New';\n"
" font-size: 11px;\n"
" padding-left: 5px;\n"
" padding-right: 5px;\n"
"")		
		MainWindow.setCentralWidget(self.centralwidget)	
		MainWindow.setWindowTitle(QtCore.QCoreApplication.translate("MainWindow", "Statusryprt"))
		self.labelPort.setText(QtCore.QCoreApplication.translate("MainWindow", "Chose you network interface"))
		self.NetOK.setText(QtCore.QCoreApplication.translate("MainWindow", "OK"))	
		QtCore.QMetaObject.connectSlotsByName(MainWindow)	
		return

	#--------------------------------------------------------------------------------
	#										Scheduler
	#--------------------------------------------------------------------------------
	def MyScheduler(self):
		# Every 25ms use the libpcab non-blocking mode to get the read packets
		# Error messages are displayed or stored
		# MMresponse are processed if present
		# Set the error display for ports with new errors or reset for the ports no more in error

		global ErrDevice
		global ErrPort
		global ErrData
		global ErrFlag
		global ErrNb
		global LineByEquipment
		global ErrLock
		global verboseMode
		global RespFlag
		global RespData
		global RespTimeCount
		global RprtDevice
		global RprtPort
		global RprtStep
		
		buf = ""
		# Process the status frames buffer until empty
		if ErrFlag :
			for i in range(ErrNb):
				port = ErrPort[i]
				equip = ErrDevice[i]
				dev = LineByEquipment[ErrDevice[i]]
				if verboseMode == 1 :
					print (ErrNb,"N° de port :",port,"N° d'équipement :",equip,"N° de device :",dev)
				# Check for port number and create messages according to error code
				Err = ErrData[i]
				# if dev = -1 it's because the configuration file is false and no interface line was assigned to that device number
				if dev != -1:
					portType = self.PortsTypes[dev][port]
					MsgErr = getErrorMessage(self.TabMessages, portType, Err)
					if port > 0 and port < len(self.PortsTypes[dev]) and port < len(self.PortsLabel[dev]):
						buf = "Device{:2d} - {:s} #{:2d} ({:2d}) - {:s} - ( {:08X}) : {:4d}".format(dev+1,CETRAC_Names[self.PortsTypes[dev][port]],self.PortsLabel[dev][port],port,MsgErr,ErrData[i],ErrNb)
						self.LogError(buf)
						self.SetErrorPort(dev,port)
					else:
						buf = "Device{:2d} - {:s} #{:s} ({:2d}) - {:s} - ( {:08X}) : {:4d}".format(dev+1,"Unknown type","No label",port,"Unknowned error",ErrData[i],ErrNb)
						self.LogError(buf)
						print("WARNING port{:2d} has no interface assigned by the configuration file".format(port))
				else:
					print("WARNING Equipment{:2d} has no interface line assigned by the configuration file".format(equip))
			ErrFlag = False
		ErrNb = 0
		# Process the MMread answers if there is a pending demand
		if RprtStep != 0:
			# An answer is available
			if RespFlag:
				if RprtStep == 1:
					if RprtPort == 0:
						self.DataPort.clear()
						self.DataPort.appendPlainText("Board Serial number    : 0x{0:08x}".format(RespData[0]))
						self.DataPort.appendPlainText("IP version             : 0x{0:08x}".format(RespData[1]))
					else:
						self.DataPort.clear()
						self.DataPort.appendPlainText("CRC32 error counter               : {}".format(RespData[0]))
						self.DataPort.appendPlainText("Size low limitation error counter : {}".format(RespData[1]))
						self.DataPort.appendPlainText("Size high limitation error counter: {}".format(RespData[2]))
						self.DataPort.appendPlainText("A664P7 Policing error counter     : {}".format(RespData[3]))
					# Start step 2 of the MMRead data
					self.GetDataPort(RprtDevice,RprtPort,2)
				elif RprtStep == 2:
					if RprtPort == 0:
						self.DataPort.appendPlainText("Initialization vector  : {0:032b}".format(RespData[0]))	
						self.DataPort.appendPlainText("Initialization counter : {}".format(RespData[1]))
						val33V = (RespData[2] & 0xFFF) * 3.3 / 2048.
						boadTemp = ((RespData[2] & 0x7F0000) >> 16)
						fpgaTemp = ((RespData[2] & 0x7F000000) >> 24)
						fanStatus = RespData[3] & 0x0F
						fanError = (RespData[3] & 0xF0) >> 4
						powerSupply_1 = (RespData[3] & 0x100) >> 8
						powerSupply_2 = (RespData[3] & 0x200) >> 9
						self.DataPort.appendPlainText("3.3V Value 1           : {:.2f} Volts".format(val33V))					
						self.DataPort.appendPlainText("Board temperature      : {:3d} ° Celsius".format(boadTemp))
						self.DataPort.appendPlainText("FPGA temperature       : {:3d} ° Celsius".format(fpgaTemp))						
						self.DataPort.appendPlainText("Fan Status             : {}".format(fanStatus))						
						self.DataPort.appendPlainText("Fan Error              : {}".format(fanError))						
						self.DataPort.appendPlainText("Power 1:{:2d}      Power 2:{:2d}      ".format(powerSupply_1 , powerSupply_2))
						RprtStep = 0
					else:
						self.DataPort.appendPlainText("Filtering error counter           : {}".format(RespData[0]))
						self.DataPort.appendPlainText("Physical layer error counter      : {}".format(RespData[1]))
						self.DataPort.appendPlainText("TX frame counter                  : {}".format(RespData[2]))
						self.DataPort.appendPlainText("RX frame counter                  : {}".format(RespData[3]))
						self.GetDataPort(RprtDevice, RprtPort, 3)
				elif RprtStep == 3:
					PortName = bytearray(12)
					try:
						TypeName = CETRAC_PortType[(RespData[0] & 0xFF)]
					except:
						TypeName = " Inconnu N°:{:2x}".format(RespData[0] & 0xFF)
					self.DataPort.appendPlainText("Port Type  : {}".format(TypeName))
					PortName[0] = ((RespData[1] & 0xFF000000) >> 24)
					PortName[1] = ((RespData[1] & 0xFF0000) >> 16)
					PortName[2] = ((RespData[1] & 0xFF00) >> 8)
					PortName[3] = RespData[1] & 0xFF
					PortName[4] = ((RespData[2] & 0xFF000000) >> 24)
					PortName[5] = ((RespData[2] & 0xFF0000) >> 16)
					PortName[6] = ((RespData[2] & 0xFF00) >> 8)
					PortName[7] = RespData[2] & 0xFF
					PortName[8] = ((RespData[3] & 0xFF000000) >> 24)
					PortName[9] = ((RespData[3] & 0xFF0000) >> 16)
					PortName[10] = ((RespData[3] & 0xFF00) >> 8)
					PortName[11] = RespData[3] & 0xFF
					self.DataPort.appendPlainText("Port Name  : {}".format(PortName.decode()))
					self.GetDataPort(RprtDevice, RprtPort, 4)
				elif RprtStep == 4:
					try:
						SpeedName = CETRAC_PortSpeed[(RespData[0] & 0xFF)]
					except:
						SpeedName = " Inconnu N°:{:2x}".format(RespData[0] & 0xFF)
					self.DataPort.appendPlainText("Link Speed : {}".format(SpeedName))
					RprtStep = 0
			else:
				RespTimeCount = RespTimeCount + 1
				if RespTimeCount > 50 :
					buf = "Timeout status query: Device {:2d} Port {:2d}".format(RprtDevice,RprtPort)
					self.LogError(buf)
					RespTimeCount = 0
					RprtStep = 0			
		# Restart packet dispatch of libpcap
		packet_count = ct.c_int(0)
		status = pcap.dispatch(pd, -1, myHandler, ct.cast(ct.pointer(packet_count), ct.POINTER(ct.c_ubyte)))
		# Reset to 0 the error flags for ports displayed in error for enough duration
		if self.Total_Error != 0:
			tt = time.time()
			for i in range (0,self.nbDevicesDisplay*self.nbMaxPorts):
				if self.FlagError[i] != 0:
					if (tt-self.TimeError[i]) > self.ErrorDisplayDuration:
						self.ResetErrorPort(i // self.nbMaxPorts,(i % self.nbMaxPorts) + 1)
		return

#================================================================================
#									MAIN
#================================================================================
if __name__ == '__main__':
	global pd
	global ebuf
	global verboseMode
	global ethernetMode
	
	app = QApplication(sys.argv)
	dialog = QMainWindow()
	verboseMode = 0
	ethernetMode = 0
	for ar in sys.argv:
		if ar == "-v":
			verboseMode = 1
		elif ar == "-V":
			verboseMode = 2
		elif (ar == "-E" or ar == "-e"):
			ethernetMode = 1
	prog = MyCETRAC(dialog)
	dialog.setWindowIcon(QIcon("cetrac.png"))
	dialog.setWindowTitle("CetraC  Statusryprt")
	dialog.show()
	sys.exit(app.exec_())