#include "cetracDriver.h"
#include "cetracFonctions.h"

void nettoyage(DEVICE_EXTENSION *pDev);
void CetracEmptyRequests(PDEVICE_EXTENSION DeviceExt);

int cetrac_open(struct inode *inode,struct file *filep)
{
    struct cetrac_board *brd = container_of(inode->i_cdev,
					struct cetrac_board, cdev);
    PDEVICE_EXTENSION Current_Board = dev_get_drvdata(brd->filedev.parent);

    /* lit le fichier de peripherique au peripherique utile pour cetrac_flush... */
    filep->private_data = Current_Board;

    // compte le nombre d'utilisateurs
    ++Current_Board->compteur_utilisation;

    dev_dbg(&brd->filedev, "%s: count: %d\n", __func__, Current_Board->compteur_utilisation);

    return 0;
}

int cetrac_release(struct inode *inode,struct file *filep)
{
    PDEVICE_EXTENSION Current_Board = filep->private_data;

    // decompte le nombre d'utilisateurs
    --Current_Board->compteur_utilisation;
    dev_dbg(&Current_Board->brd->filedev, "%s: count: %d\n", __func__, Current_Board->compteur_utilisation);

    if (Current_Board->compteur_utilisation == 0)
    {
        if (Current_Board->DriverState != CETRAC_STATE_NORMAL) {
            dev_info(&Current_Board->brd->filedev, "No more users, full cleanup\n");
            nettoyage(Current_Board);
        }
    }

    return 0;

}

int cetrac_flush(struct file *filep, fl_owner_t id)
{
    PDEVICE_EXTENSION Current_Board;

    Current_Board = filep->private_data;

    dev_dbg(&Current_Board->brd->filedev, "%s\n", __func__);

    // spin_lock_bh(&Current_Board->RequestCS);

    // to do : joujou

    // spin_unlock_bh(&Current_Board->RequestCS);
    return 0;
}

/**
 *  Nettoyage
 */

void nettoyage(DEVICE_EXTENSION *pDev)
{
    KNOWNLIST   *pKLA, *pKLA_;
    WAITINGANCHOR   *pWait, *pWaitBis;


    // liberation des KnownList

    if (down_interruptible(&pDev->KnownListCS) != 0) {
	dev_err(&pDev->brd->filedev, "down_i error during clean up");
        return;
    }
    list_for_each_entry_safe(pKLA, pKLA_, &pDev->KnownListAnchor, list)
	cetrac_free_knownlist(pKLA, TRUE);
    up(&pDev->KnownListCS);

    // liberation des WaitingAnchor
    spin_lock_bh(&pDev->WaitingCS);
    list_for_each_entry_safe(pWait, pWaitBis, &pDev->WaitingAnchor, list)
        cetrac_free_waitinganchor(pWait);
    spin_unlock_bh(&pDev->WaitingCS);

    // deblocage des requetes en cours, et etat general
    spin_lock_bh(&pDev->RequestCS);
    pDev->DriverState = CETRAC_STATE_NORMAL;
    pDev->bCurrentRequest = false;
    CetracEmptyRequests(pDev);
    spin_unlock_bh(&pDev->RequestCS);

    return;
}
