#include    "cetracDriver.h"
#include    <linux/kernel.h>
#include <linux/pci.h>          // kmalloc/kfree !

enum    {   ETAT_GLOBAL, REQUETES, EVENTS, PATCH };

#define SIZE_DEBUG_BUFF 256000

#define DEVERMINE(fmt,args...) printk(KERN_WARNING "cetrac:" fmt, ##args)

// == fin commun avec ioctl

#define CETRAC_STATE_NORMAL              0x0                             /* Etat normal */
#define CETRAC_STATE_HDRW_NOT_INIT       CETRAC_DRIVER_HRDW_NOT_INIT      /* La carte n'est pas en etat de repondre au */
                                                                        /* demarrage du driver (statut de la carte = 0) */
#define CETRAC_STATE_HDRW_NO_RESPONSE    CETRAC_DRIVER_HRDW_NO_RESPONSE   /* La carte n'a pas repondu lors d'un acces */
#define CETRAC_STATE_UNSTABLE            CETRAC_DRIVER_INTERNAL_ERROR     /* Une erreur interne au driver s'est produite */

VOID CetracEmptyRequests(PDEVICE_EXTENSION DeviceExt);

/**
 * print particuliers
 */
void    dumpEvent(char *nomEvent, EVENT *pEvent)
{
    if (pEvent == NULL)
        DEVERMINE("%s : pEvent : NULL\n", nomEvent);
    else
        DEVERMINE("Event '%s': Type_idt 0x%06X, Data %08X\n", nomEvent, pEvent->Type_Idt, pEvent->Data);
}

/**
 *  Etat global
 */
void etat_global(char *pTxt, DEVICE_EXTENSION *pDev)
{
    int     i;

    strcpy(pTxt, "=== ETAT_GLOBAL ===\n\tVoir messages kernel [dmesg | grep cetrac | tail -50])");

    // sprintf(local, "DriverState : %08X ", pDev->DriverState);
    // strncat(pTxt, local, SIZE_DEBUG_BUFF);
    DEVERMINE("=== ETAT_GLOBAL ===\n");
    DEVERMINE("Nb utilisateurs : %d\n", pDev->compteur_utilisation);
    DEVERMINE("DriverState : %08X\n", pDev->DriverState);

    switch(pDev->DriverState)
    {
        case CETRAC_STATE_NORMAL           : DEVERMINE("CETRAC_STATE_NORMAL\n");           break;
        case CETRAC_STATE_HDRW_NOT_INIT    : DEVERMINE("CETRAC_STATE_HDRW_NOT_INIT\n");    break;
        case CETRAC_STATE_HDRW_NO_RESPONSE : DEVERMINE("CETRAC_STATE_HDRW_NO_RESPONSE\n"); break;
        case CETRAC_STATE_UNSTABLE         : DEVERMINE("CETRAC_STATE_UNSTABLE\n");         break;
        default : DEVERMINE("unknown state)\n");         break;
    }

    DEVERMINE("bCurrentRequest : %d\n", pDev->bCurrentRequest);

    DEVERMINE("bCurrentRequestWaiting : %d\n", pDev->bCurrentRequestWaiting);
    // DEVERMINE("bCurrentReset : %d\n", pDev->bCurrentReset);


    DEVERMINE("RequestQueue[] : ");
    for ( i = 0; i < CETRAC_NB_PRIORITIES; ++i)
    {
        DEVERMINE("%d ", pDev->RequestQueue[i]);
    }
    DEVERMINE("\n");

    DEVERMINE("flagRequestEvents[] : ");
    for ( i = 0; i < CETRAC_NB_PRIORITIES; ++i)
    {
        DEVERMINE("%d ", pDev->flagRequestEvents[i]);
    }
    DEVERMINE("\n");

    DEVERMINE("countWaitRequest[] : ");
    for ( i = 0; i < CETRAC_NB_PRIORITIES; ++i)
    {
        DEVERMINE("%d ", pDev->countWaitRequest[i]);
    }
    DEVERMINE("\n");

    DEVERMINE("countTrigRequest[] : ");
    for ( i = 0; i < CETRAC_NB_PRIORITIES; ++i)
    {
        DEVERMINE("%d ", pDev->countTrigRequest[i]);
    }
    DEVERMINE("\n");

    dumpEvent("CurrentRequestReturn", &pDev->CurrentRequestReturn);

    return;
}

/**
 *  =================
 *  Etat des requetes
 *  =================
 */
void dumpEventList(EVENTLIST *pEVL);
void etat_requetes(char *pTxt, DEVICE_EXTENSION *pDev)
{
    KNOWNLIST   *pKLA;

    strcpy(pTxt, "Etat REQUETES demande :\n\tVoir messages kernel [dmesg | grep cetrac | tail -50])");

    DEVERMINE("Etat REQUETES demande : \n");

    pKLA = pDev->KnownListAnchor;

    if ( pKLA == NULL )
    {
        DEVERMINE("    Liste vide\n");
        return;
    }
    // else
    while (pKLA != NULL)
    {
        dumpEventList(pKLA->EventList);
        pKLA = pKLA->Next;
    }

    return;
}

void dumpEventList(EVENTLIST *pEVL)
{
    int     nbEvents;
    int     isFifoEmpty;
    EVENT   *pEvent;
    int     i;

    DEVERMINE("EVENTLIST :\n");

    DEVERMINE("   bWaiting : %d\n", pEVL->bWaiting);

    DEVERMINE("   bInProgress : %d\n", pEVL->bInProgress);

    DEVERMINE("   bFifoFull : %d\n", pEVL->bFifoFull);

    DEVERMINE("   RecordedIdt : %d\n", pEVL->RecordedIdt);

    dumpEvent("CurrentEvent", &pEVL->CurrentEvent);

    DEVERMINE("   bCurrentQueue : %d\n", pEVL->bCurrentQueue);

    DEVERMINE("   statusEvent : %d\n", pEVL->statusEvent);

    // fifo
    DEVERMINE("   FIFO :\n");

    nbEvents = pEVL->Fifo.FifoSize;
    DEVERMINE("   FifoSize : %d\n", pEVL->Fifo.FifoSize);

    isFifoEmpty = pEVL->Fifo.bEmpty;
    DEVERMINE("   bEmpty : %d\n", pEVL->Fifo.bEmpty);

    DEVERMINE("   bFull : %d\n", pEVL->Fifo.bFull);

    DEVERMINE("    tableau d'events a suivre :\n");

    // les events dans la fifo
    pEvent = pEVL->Fifo.FirstEvent;
    for ( i = 0; i < nbEvents; ++i )
    {
        dumpEvent("fifo_event", pEvent);
        // le suivant
        if ( pEvent < pEVL->Fifo.Anchor + nbEvents )
            ++pEvent;
        else
            pEvent = pEVL->Fifo.Anchor;
    }
}

/**
 * =====================
 *  Etat des evenements
 * =====================
 */
void dumpWaiting(WAITINGANCHOR *pWaitAnchor)
{
    WAITING     *pWaiting;
    DEVERMINE("  wait idt %08X\n", pWaitAnchor->Idt);

    pWaiting = pWaitAnchor->First;
    if ( pWaiting == NULL)
    {
        DEVERMINE("        Liste WAINTING vide\n");
        return;
    }

    while (pWaiting != NULL)
    {
        DEVERMINE("        bQueue %d\n", pWaiting->bQueue);
        // on suppose que pEventList a deja ete libéré
        pWaiting = pWaiting->Next;
    }
    return;
}

void etat_events(char *pTxt, DEVICE_EXTENSION *pDev)
{
    WAITINGANCHOR   *pWait;

    strcpy(pTxt, "Etat EVENEMENTS demande :\n\tVoir messages kernel [dmesg | grep cetrac | tail -50])");

    DEVERMINE("Etat EVENEMENTS demande : \n");

    pWait = pDev->WaitingAnchor;
    if (pWait == NULL)
    {
        DEVERMINE("   Liste Vide\n");
        return;
    }

    while (pWait != NULL)
    {
        dumpWaiting(pWait);
        pWait = pWait->Next;
    }

    return;
}

/**
 *  Nettoyage
 */

void _cleanEventsList(EVENTLIST *pEvl)
{
    kfree(pEvl->Fifo.Anchor);
    kfree(pEvl);
    // verifier s'il faut chatouiller les semaphores ...
}

void _cleanWaiting(WAITING   *pWait)
{
    WAITING *pWaitBis;

    while (pWait != NULL)
    {
        pWaitBis = pWait;
        pWait = pWait->Next;
        kfree(pWaitBis);
    }
}

void _nettoyage(char *pTxt, DEVICE_EXTENSION *pDev)
{

    KNOWNLIST   *pKLA;
    KNOWNLIST   *pKLABis;

    WAITINGANCHOR   *pWait;
    WAITINGANCHOR   *pWaitBis;

    // deblocage des requetes en cours
    spin_lock_bh(&pDev->RequestCS);
    CetracEmptyRequests(pDev);
    spin_unlock_bh(&pDev->RequestCS);

    // liberation des KnownList

    if (down_interruptible(&pDev->KnownListCS) != 0)
    {
	dev_err(&pDev->brd->filedev, "down_i error during __clean up");
        return;
    }

    pKLA = pDev->KnownListAnchor;

    while (pKLA != NULL)
    {
        _cleanEventsList(pKLA->EventList);
        pKLABis = pKLA;
        pKLA = pKLA->Next;
        kfree(pKLABis);
    }
    pDev->KnownListAnchor = NULL;
    up(&pDev->KnownListCS);

    // liberation des WaitingAnchor

    spin_lock_bh(&pDev->WaitingCS);
    pWait = pDev->WaitingAnchor;
    while (pWait != NULL)
    {
        _cleanWaiting(pWait->First);
        pWaitBis = pWait;
        pWait = pWait->Next;
        kfree(pWaitBis);
    }
    pDev->WaitingAnchor = NULL;
    spin_unlock_bh(&pDev->WaitingCS);

    // etat general
    pDev->DriverState = CETRAC_STATE_NORMAL;

    return;
}
/**
 * dispatch des commandes demandees
 */
void deverminage(echange_t *pDeverm, DEVICE_EXTENSION   *pDev)
{
    switch(pDeverm->cmd)
    {
    case ETAT_GLOBAL :
        etat_global(pDeverm->txt, pDev);
        break;

    case REQUETES :
        etat_requetes(pDeverm->txt, pDev);
        break;

    case EVENTS :
        etat_events(pDeverm->txt, pDev);
        break;

    case PATCH :
        _nettoyage(pDeverm->txt, pDev);
        break;

    default:
        sprintf(pDeverm->txt, "Default, ce qui a ete envoye : %d", pDeverm->cmd);
        break;
    }
    return;
}


