// ------------------------------------------------------------------------------------------------
/// @file cut_endian.h
/// @author PiV
/// @date 2015-2016
/// @brief The endianness management functions
// ------------------------------------------------------------------------------------------------
// - CETRAC Project - cetrac util                                                                 -
// - copyright (c) 2016-2016 Silkan                                                               -
// ------------------------------------------------------------------------------------------------

#ifndef CUT_ENDIAN_H
#define CUT_ENDIAN_H

#include "cut_types.h"


/// the type for endianess
typedef enum {
  CUT_BIG_ENDIAN,
  CUT_LIT_ENDIAN
} t_cut_endian;

/// the type to check fo endianness
typedef  union u_cut_lob {
  /// the value in host order
  uint16_t val;
  /// the value as an array 
  uint8_t  tab[2];
}  t_cut_lob;


/// @return the host endian
// static inline t_cut_endian cut_get_endianness () {
static CUT_INLINE t_cut_endian cut_get_endianness () {
  static const t_cut_lob lob = {.val = 1};
  return (lob.tab[0]==1) ? CUT_LIT_ENDIAN : CUT_BIG_ENDIAN;
}

/// @brief copy a 16 bits host value into a buffer in network order
/// @param dst, the destination buffer
/// @param val, the value to be copied
static CUT_INLINE void cut_hton16 (uint8_t dst[2], uint16_t val) {
  if (CUT_LIT_ENDIAN) {
     uint8_t* tmp = (uint8_t*) &val; 
     dst[0] = tmp[1];
     dst[1] = tmp[0];
  } else {
    *((uint16_t*) dst) = val;
  }  
}

/// @brief copy a 32 bits host value into a buffer in network order
/// @param dst, the destination buffer
/// @param val, the value to be copied
static CUT_INLINE void cut_hton32 (uint8_t dst[4], uint32_t val) {
  if (CUT_LIT_ENDIAN) {
     uint8_t* tmp = (uint8_t*) &val;
     dst[0] = tmp[3];
     dst[1] = tmp[2];
     dst[2] = tmp[1];
     dst[3] = tmp[0];
  } else {
    *((uint32_t*) dst) = val;
  }  
}

/// @brief copy a 64 bits host value into a buffer in network order
/// @param dst, the destination buffer
/// @param val, the value to be copied
static CUT_INLINE void cut_hton64 (uint8_t dst[8], uint64_t val) {
  if (CUT_LIT_ENDIAN) {
     uint8_t* tmp = (uint8_t*) &val;
     dst[0] = tmp[7];
     dst[1] = tmp[6];
     dst[2] = tmp[5];
     dst[3] = tmp[4];
     dst[4] = tmp[3];
     dst[5] = tmp[2];
     dst[6] = tmp[1];
     dst[7] = tmp[0];
  } else {
    *((uint64_t*) dst) = val;
  }  
}

#endif
