//-------------------------------------------------------------------------------------------------
/// @file cCetracAPI.h
/// @author Alexande.L
/// @date 2015-2017
/// @brief CORAIL CETRAC API.
// ------------------------------------------------------------------------------------------------
// - CETRAC Project - MITRAC
// - Copyright (c) - SILKAN
//-------------------------------------------------------------------------------------------------

#ifndef C_CETRAC_API_H_
#define C_CETRAC_API_H_

#include <stdio.h>

#include "cetracApi.h"
#include "inOutTable.h"
#include "mitracErrorCodes.h"
#include "mitracOutils.h"
#include "mitracSetupApi.h"

/// @brief Sequence number checking function.
/// @return 0 on success, otherwise -1
/// @param[in]  table  The table's identifier.
/// @param[in]  index  The table's index.
/// @param[out] position Where is the frame which
int check_SN(inOutTableStrct* table, unsigned int index);

/// @brief Initialization function.
/// @return 0 on success, otherwise -1
/// @param[in]  table  The table's identifier.
/// @param[in]  index  The table's index.
/// @param[out] pError The error return.
int check_IP_SIZE(inOutTableStrct* table, unsigned int index);

/// @brief Initialization function.
/// @return 0 on success, otherwise -1
/// @param[in]  table  The table's identifier.
/// @param[in]  index  The table's index.
/// @param[out] pError The error return.
int cCetracInit(inOutTableStrct* table,
                unsigned int     index,
                t_error*         pError);

/// @brief Reading function.
/// @return 0 on success, otherwise -1
/// @param[in]  table  The table's identifier.
/// @param[in]  index  The table's index.
/// @param[out] pError The error return.
int cCetracRead(inOutTableStrct* table,
                unsigned int     index,
                t_error*         pError,
                int first_read);

/// @brief Writing function.
/// @return 0 on success, otherwise -1
/// @param[in]  table  The table's identifier.
/// @param[in]  index  The table's index.
/// @param[out] pError The error return.
int cCetracWrite(inOutTableStrct* table,
                 unsigned int     index,
                 t_error*         pError);
					
/// @brief Reset function.
/// @return 0 on success, otherwise -1
/// @param[in]  table  The table's identifier.
/// @param[in]  index  The table's index.
/// @param[out] pError The error return.
int cCetracReset(inOutTableStrct* table,
                 unsigned int     index,
                 t_error*         pError);

#endif // C_CETRAC_API_H_
