//-------------------------------------------------------------------------------------------------
/// @file cVistasPlusAPI.h
/// @author Alexandre.L
/// @date 2015-2017
/// @brief CORAIL VISTAS Plus API.
// ------------------------------------------------------------------------------------------------
// - CETRAC Project - MITRAC
// - Copyright (c) - SILKAN
//-------------------------------------------------------------------------------------------------

#ifndef C_VISTAS_PLUS_API_H_
#define C_VISTAS_PLUS_API_H_

#include <stdio.h>
#include <string.h>

#include "inOutTable.h"
#include "mitracErrorCodes.h"
#include "mitracOutils.h"
#include "mitracSetupApi.h"
#include "mitracSVC.h"

void update_position_r(inOutTableStrct* table,
		       unsigned int     index);

/// @brief Initialization function.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  idt    The frame's identifier.
/// @param[out] pError The error return.
t_error cVistasPlusInit(const char*   idt,
                        unsigned int* pError);

/// @brief Reading function.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  idt     The frame's identifier.
/// @param[in]  dstSize The destination buffer size.
/// @param[out] dst     The destination buffer.
/// @param[out] pError  The error return.
t_error cVistasPlusRead(const char*   idt,
                        size_t        dstSize,
                        void*         dst,
                        unsigned int* pError);

/// @brief Writing function.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  idt         The frame's identifier.
/// @param[in]  payloadSize The payload size.
/// @param[in]  pPayload    The payload to write.
/// @param[out] pError      The error return.
t_error cVistasPlusWrite(const char*    idt,
                         size_t         payloadSize,
                         void*          pPayload,
                         unsigned int*  pError);

/// @brief Reset function.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  idt    The frame's identifier.
/// @param[out] pError The error return.
t_error cVistasPlusReset(const char*   idt,
                         unsigned int* pError);

#endif // C_VISTAS_PLUS_API_H_
