//-------------------------------------------------------------------------------------------------
/// @file cVistasPlusOrder.h
/// @author Alexandre.L
/// @date 2015-2017
/// @brief CORAIL VISTAS Plus API. Use of orders and status for the MEF control.
// ------------------------------------------------------------------------------------------------
// - CETRAC Project - MITRAC
// - Copyright (c) - SILKAN
//-------------------------------------------------------------------------------------------------

#ifndef C_VISTAS_PLUS_ORDER_API_H_
#define C_VISTAS_PLUS_ORDER_API_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "inOutTable.h"
#include "mitracErrorCodes.h"
#include "mitracSetupApi.h"
#include "mitracOutils.h"

/// Define a type for the initialization function.
typedef void (*f_init) (void);
/// Define a type for the termination function.
typedef void (*f_end) (void);
/// Define a type for the step function.
typedef void (*f_step) (void);
/// Define a type for the pause function.
typedef void (*f_pause) (void);

/// @brief Initialization function.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  idt    The frame's identifier.
/// @param[in]  i      The initialization function.
/// @param[in]  e      The termination function.
/// @param[in]  s      The step function.
/// @param[in]  p      The pause function.
/// @param[out] pError The error return.
t_error cVistasPlusOrderInit(const char*   idt,
                             f_init        i,
                             f_end         e,
                             f_step        s,
                             f_pause       p,
                             unsigned int* pError);

/// @brief Order writing function.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  idt    The frame's identifier.
/// @param[in]  order  The order to write.
/// @param[out] pError The error return.
t_error cVistasPlusWriteOrder(const char*   idt,
                              unsigned int  order,
                              unsigned int* pError);

/// @brief Order reading function.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  idt    The frame's identifier.
/// @param[out] pOrder The order to be retrieved.
/// @param[out] pError The error return.
t_error cVistasPlusReadOrder(const char*   idt,
                             unsigned int* pOrder,
                             unsigned int* pError);

/// @brief Status reading function.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  idt     The frame's identifier.
/// @param[out] pStatus The status to be retrieved.
/// @param[out] pError  The error return.
t_error cVistasPlusReadStatus(const char*   idt,
                              unsigned int* pStatus,
                              unsigned int* pError);

/// @brief Status writing function.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  idt    The frame's identifier.
/// @param[in]  status The status to write.
/// @param[out] pError The error return.
t_error cVistasPlusWriteStatus(const char*   idt,
                               unsigned int  status,
                               unsigned int* pError);

/// @brief Reset function.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  idt    The frame's identifier.
/// @param[out] pError The error return.
t_error cVistasPlusOrderReset(const char*   idt,
                              unsigned int* pError);
/// @brief Initialization function of the application.
/// @param[in] pTraceName the name of the trace.
void initAppli(void);

/// @brief Application termination function.
void endAppli(void);

/// @brief Function to execute an application's simulation step.
void stepAppli(void);

/// @brief Pause function of the application's simulation.
void pauseAppli(void);

#endif // C_VISTAS_PLUS_ORDER_API_H_
