/**
 * \file idDefinition.h
 * \brief Definition of the numeric ID for the orders, the commands and the status
 * \author HRakotomalala
 * \date 2018
 */

#ifndef ID_DEFINITION_H_
#define ID_DEFINITION_H_

// Supervisor turn ON/OFF
#define SUPERVISOR_ON 1
#define SUPERVISOR_OFF 0

typedef enum {
  NO_ACT = 0x00,
  LOAD_ACT = 0x01,
  UNLOAD_ACT =  0x02,
  INIT_ACT = 0x03,
  RUN_ACT = 0x04,
  STEP_ACT = 0x05,
  PAUSE_ACT = 0x06,
  RESET_ACT = 0x07,
  GETSTATUS_ACT = 0x08
} t_sup_act;

typedef enum {
  LOAD    = 0x1001, 
  UNLOAD  = 0x1002,
  INIT    = 0x1003,
  RUN     = 0x1004,
  STEP    = 0x1005,
  PAUSE   = 0x1006,
  RESET   = 0x1007,
} t_sup_command;

typedef enum {
  SUPERVISOR_ORDERS  = 0x2000, 
  DO_LOAD    = 0x2001,
  DO_UNLOAD  = 0x2002,
  DO_INIT    = 0x2003,
  DO_RUN     = 0x2004,
  DO_STEP    = 0x2005,
  DO_PAUSE   = 0x2006,
  DO_RESET   = 0x2007,
  GET_STATUS = 0x2008
} t_sup_order;

typedef enum {
  MYDDLEWARE_STATUS  = 0x3000,
  
  READY_STATUS = 0x0100,
  BUSY_STATUS = 0x0200,
  ERROR_STATUS = 0x0300,

  IN_STANDBY = 0x3100,
  LOAD_SUCCESS = 0x3101,
  IN_PAUSE = 0x3106,
  
  IN_RUN = 0x3204,
  IN_STEP = 0x3204,

  LOAD_ERROR = 0x3301,
  INIT_ERROR = 0x3303,
  RUN_ERROR = 0x3304
} t_sup_status;

#endif //ID_DEFINITION_H_
