//-------------------------------------------------------------------------------------------------
/// @file inOutTable.h
/// @author Alexandre.L
/// @date 2015-2017
/// @brief MITRAC In/Out table definition.
// ------------------------------------------------------------------------------------------------
// - CETRAC Project - MITRAC
// - Copyright (c) - SILKAN
//-------------------------------------------------------------------------------------------------

#ifndef IN_OUT_TABLE_H_
#define IN_OUT_TABLE_H_

#include <stdint.h>

#include "cetracTypes.h"
#include "cut_types.h"

#include "mitracSVC.h"
#include "mitracErrorCodes.h"

#define CETRAC_CLOCK 1
#define PROC_CLOCK 0

/// Define type of frame
typedef enum {
  TYPE_FRM_A664P7,
  TYPE_FRM_CETRAC,
  TYPE_FRM_ETH
} t_type_frm;

/// Define type of frame
typedef enum {
  TYPE_MSG_POLLING,
  TYPE_MSG_ONE_READING,
  TYPE_MSG_SAMPLING,
  TYPE_MSG_QUEUEING
} t_type_msg;

/// Define IN timer cetrac structure
typedef struct {
  // type of period
  int typeOfClk;
  // number of the timer
  unsigned char numTimerCetrac;
  /// Index of the timer
  unsigned char idTimerCetrac;
  /// value of period
  unsigned long long int valuePeriod;
} timerStrct;

/// Define In/OUT table (frame + parameters).
typedef struct {
  /// Address
  void          *pBufferAddr;
  /// Payload size.
  uint32_t      payloadSize;
  /// Payload offset.
  uint32_t      payloadOffset;
  // size to write on PCI cetrac card
  uint32_t      writtenSize;
  /// Total size.
  uint32_t      globalSize;
  /// CETRAC identifier.
  cetracU32      cetracId;
  /// Frame's identidier.
  char          idt[255];
  /// CETRAC file descriptor (driver link).
  cut_handle       fd;
  /// Capsule's services.
  t_mitracSVC   svc;
  /// Shared memory file descriptor.
  int           actrlMemfd;
  /// Shared memory file path.
  char          pSMFilePath[255];
  /// Data position in the shared memory file.
  unsigned int  shMemoryPosition;
  // Type of message
  t_type_msg typeOfMsg;
  // Type of frame
  t_type_frm typeOfFrm;
  // Number of messages
  int nbMsgMax;
  // Position of reading used only for polling message
  int p_read;
  // Position of writing used only for polling message
  int p_write;
  // First reading by  Mydlewr
  int firstCall;
  // Application error
  t_error appError;
  // Hardware error
  t_error hardError;
  /// Current serial number 
  unsigned char SN;
  /// Old serial number 
  unsigned char SN_old;
  // Flag for writing
  unsigned char flagToWrite;
  /// flag for triggering 
  int flag_check_SN;
} inOutTableStrct;

#endif // IN_OUT_TABLE_H_
