//-------------------------------------------------------------------------------------------------
/// @file mitracOutils.h
/// @author Alexandre.L
/// @date 2015-2017
/// @brief MITRAC tools functions.
// ------------------------------------------------------------------------------------------------
// - CETRAC Project - MITRAC
// - Copyright (c) - SILKAN
//-------------------------------------------------------------------------------------------------

#ifndef MITRAC_OUTILS_H_
#define MITRAC_OUTILS_H_

#include "inOutTable.h"

int mitracCheckPCi(void);

/// @brief Find an index into a table using a frame's identifier.
/// @return MITRAC_NO_ERROR on success otherwise MITRAC_ERROR.
/// @param[in]  table      The table.
/// @param[in]  nbElements The table's elements number.
/// @param[in]  pIdt       The frame's identifier.
/// @param[in]  firstIndex The first index.
/// @param[in]  lastIndex  The last index.
/// @param[out] pIndex     The desired index.
/// @param[out] pErrorCode The error return.
int mitracGetIndex(inOutTableStrct table[],
                   unsigned int    nbElements,
                   const char*     pIdt,
                   unsigned int    firstIndex,
                   unsigned int    lastIndex,
                   unsigned int*   pIndex,
                   unsigned int*   pErrorCode);

/// @brief Hexadecimal string writing function.
/// @param[in] pDst  The writing address of the hexadecimal data.
/// @param[in] value The character array to convert to hexadecimal.
/// @param[in] size  The size of the data.
void write_str(unsigned char* pDst,
               char*          value,
			   unsigned int   size);

/// @brief Display frame function.
/// @param[in] trame        The frame to display.
/// @param[in] taille       The total size.
/// @param[in] index        The buffer index.
/// @param[in] payloadIndex The 1st byte of the payload.
/// @param[in] payloadSize  The payload size.
void afficherTrame(void*        trame,
                   int          taille,
       	           unsigned int index,
		   unsigned int payloadIndex,
		   unsigned int payloadSize);

#endif // MITRAC_OUTILS_H_
