//-------------------------------------------------------------------------------------------------
/// @file mitracSVC.h
/// @author Alexandre.L
/// @date 2015-2017
/// @brief MITRAC services management.
// ------------------------------------------------------------------------------------------------
// - CETRAC Project - MITRAC
// - Copyright (c) - SILKAN
//-------------------------------------------------------------------------------------------------

#ifndef MITRAC_SVC_H_
#define MITRAC_SVC_H_

#include <stdint.h>
#include <stdio.h>
#include <string.h>

/// Define a type for a service function.
typedef void (*t_service)(void*, void*, unsigned int, unsigned int);

/// Define a service.
typedef struct {
  /// Writing address.
  unsigned char	*pDst;
  /// Start address for calculation. Set to NULL if not used.
  unsigned char	*pSrc;
  /// Memory size for calculation. Set to zero if not used.
  unsigned int 	size;
  /// CRC type to apply (8, 16, 32). Set to zero if not used.
  unsigned int 	type;
  /// Pointer to service function to apply.
  t_service      	pServiceFunc; 
} t_varParamService;

/// Define services.
typedef struct {
  /// Services table of the capsule.
  t_varParamService	*pServiceParam;
  /// Number of services to be applied.
  unsigned int		nServices;	    	
} t_mitracSVC;

/// @brief CRC calculation function.
/// @param[in] pDst The writing address.
/// @param[in] pSrc The start address for calculation.
/// @param[in] size The memory size for calculation.
/// @param[in] type The type to apply (8, 16, 32).
void crc_gen(unsigned char* pDst,
             unsigned char* pSrc,
             unsigned int   size,
             unsigned int   type);

/// @brief SN calculation function.
/// @param[in] pDst The writing address.
/// @param[in] pSrc Not used. Set to 0.
/// @param[in] size Not used. Set to 0.
/// @param[in] type Not used. Set to 0.
void SNcompute(unsigned char* pDst,
               unsigned char* pSrc,
               unsigned int   size,
               unsigned int   type);

#endif // MITRAC_SVC_H_
